//////////////////////////////////////////////////////////////////////////////
//
// TextureImage.cpp
//	Implementation of the cTextureImage class.
//
//	-	Loads an image file and converts it to an OpenGL texture.
//	-	Support for loading palette files which create additional OpenGL
//		textures, but with different colors (based on remapping information).
//	-	Tabs set to 3 spaces.
//
//////////////////////////////////////////////////////////////////////////////

#include "TextureImage.h"

//////////////////////////////////////////////////////////////////////////////
//	CONSTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cTextureImage::cTextureImage() :
	m_pImageData(NULL)
{
	clear();
}

//////////////////////////////////////////////////////////////////////////////
//	DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cTextureImage::~cTextureImage()
{
	clear();
}

//////////////////////////////////////////////////////////////////////////////
//
// Clears contents of object.
//
//	POST:	-	Any allocated memory is freed.
//
//////////////////////////////////////////////////////////////////////////////
void cTextureImage::clear()
{
	// Free memory held by image data.
	if (m_pImageData != NULL)
	{
		delete[] m_pImageData;
		m_pImageData = NULL;
	}

	// Free OpenGL textures, and clear collection.
	if (m_vID.size() > 0)
	{
		for(int i=0; i < m_vID.size(); i++)		// Loop to each OpenGL texture.
			glDeleteTextures(1, &m_vID[i]);		// Delete the texture.

		m_vID.clear();									// Empty the collection.
	}

	m_width	= 0;
	m_height = 0;
	m_ByPP	= 0;
}

//////////////////////////////////////////////////////////////////////////////
//
// -	Loads a Targa (TGA) file into an OpenGL texture.
//	-	Currently only supports uncompressed 24 or 32 bit targas.
//	-	Format of Targa header is below:
//
//		Size	Name					Details
//		----	----------------	------------------------------------------------
//		1		ID length			Length of a string located after the header
//		1		Color map type		0 for RGB
//										1 for a color map
//		1		Data type code		How image data is stored:
//										0	No image data included.
//										1	Uncompressed, color-mapped images.
//										2	Uncompressed, RGB images.
//										3	Uncompressed, black and white images.
//										9	Runlength encoded color-mapped images.
//										10	Runlength encoded RGB images.
//										11	Compressed, black and white images.
//										32	Compressed color-mapped data, using
//											Huffman, Delta, and runlength encoding.
//										33	Compressed color-mapped data, using Huffman,
//											Delta, and runlength encoding.  4-pass
//											quadtree-type process.
//		2		Color map origin	First palette entry.
//		2		Color map length	# of palette entries
//		1		Color map depth	# of bits per palette entry (16, 24 or 32)
//		2		X origin				Left image coordinate
//		2		Y origin				Bottom image coordinate
//		2		Width					Width of image
//		2		Height				Height of image
//		1		Bits per pixel		# of bits for each color value:
//										32			BGRA, each 1 byte (8 bits)
//										24			BGR, each 1 byte (8 bits)
//										16			Each color component is stored as 5
//													bits and the remaining bit is a
//													binary alpha value.  The colour
//													components are converted into single
//													byte components by simply shifting each
//													component up by 3 bits (multiply 8).
//
//		1		Image descriptor	Bit-field describing how image data is stored:
//										Bits 3-0	Number of attribute bits associated
//													with each pixel.  For the Targa 16, 
//													this would be 0 or 1.  For the Targa
//													24, it should be 0.  For Targa 32, it
//													should be 8.
//										Bit 4    Reserved.  Must be set to 0.       
//										Bit 5    Screen origin bit.
//													0 = Origin in lower left-hand corner.
//													1 = Origin in upper left-hand corner.
//										Bits 7-6	Data storage interleaving flag.
//													00 = non-interleaved.
//													01 = two-way (even/odd) interleaving.
//													10 = four way interleaving.
//													11 = reserved.
//
//	ARGS:	filename,	Name of targa file to load.
//
//	RETURNS:	true,	On success.
//				false, otherwise.
//
//////////////////////////////////////////////////////////////////////////////
bool cTextureImage::loadTGA(const char *filename)
{
	GLubyte	TGAcompare[12];	// Used to compare TGA header
	GLubyte	header[6];			// First 6 Useful Bytes From The Header
	GLuint	imageSize;			// Used To Store The Image Size When Setting Aside Ram

	// Uncompressed TGA Header
	GLubyte	TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};

	
	FILE *FileHandle = fopen(filename, "rb");		// Open targa file.
	
	// Perform various checks, and read file header.
	if(FileHandle==NULL																			||	// Does File Even Exist?
		fread(TGAcompare,1,sizeof(TGAcompare),FileHandle)	!=sizeof(TGAcompare) ||	// Are There 12 Bytes To Read?
		memcmp(TGAheader,TGAcompare,sizeof(TGAheader))		!=0						||	// Does The Header Match What We Want?
		fread(header,1,sizeof(header),FileHandle)				!=sizeof(header))			// If So Read Next 6 Header Bytes
	{
		// Close file if open.
		if (FileHandle != NULL)
			fclose(FileHandle);
		
		return false;
	}
	
	// Determine image width and height (dimension = highbyte * 256 + lowbyte)
	m_width = header[1] * 256 + header[0];
	m_height= header[3] * 256 + header[2];

	m_ByPP = header[4] / 8;		// Determine bytes per pixel.

	
	// Check that the width, height, or bits per pixel are valid.
	if( (m_width<=0) || (m_height<=0) || ((m_ByPP!=3) && (m_ByPP!=4)) )
	{
		// Illegal file contents so close and exit.
		fclose(FileHandle);
		return false;
	}

	imageSize = m_width * m_height * m_ByPP;		// Determine image size.
	m_pImageData = new GLubyte[imageSize];			// Allocate image data memory.

	// Check if memory was allocated and that all data is read into it.
	if(m_pImageData==NULL ||
		fread(m_pImageData, 1, imageSize, FileHandle) != imageSize)
	{
		// Failure to read image data,
		
		// Free memory if allocated.
		if(m_pImageData != NULL)
		{
			delete[] m_pImageData;
			m_pImageData = NULL;
		}
		
		// Close file handle and exit/
		fclose(FileHandle);
		return false;
	}

	// Check if TGA's data is stored with an irregular orientation.
	if (0x0020 & header[5])
	{
		GLuint x1, x2;	// Temporary variables

		// Loop through the image data (row by row from left to right).
		for (GLuint j = 0; j < m_height/2; j++)
		{
			// For each component (R,G,B,A) in the row...
			for (GLuint i = 0; i < m_width * m_ByPP; i++)
			{
				// Index of the upper row
				x1 = j * m_width * m_ByPP + i;

				// Index of the lower row
				x2 = (m_height - 1 - j) * m_width * m_ByPP + i;	
				
				// Swapping with XOR is faster than using a temp variable.
				m_pImageData[x1] ^= m_pImageData[x2];
				m_pImageData[x2] ^= m_pImageData[x1];
				m_pImageData[x1] ^= m_pImageData[x2];				
			}
		}
	}

	// Loop through the image data.
	for(GLuint i=0; i<int(imageSize); i+= m_ByPP)
	{
		// Swaps the 1st and 3rd bytes ('R'ed and 'B'lue)
		// Swapping with XOR is faster than using a temp variable.
		m_pImageData[i] ^= m_pImageData[i + 2];
		m_pImageData[i + 2] ^= m_pImageData[i];
		m_pImageData[i] ^= m_pImageData[i + 2];
	}
	
	fclose(FileHandle);		// Close targa file.

	createOpenGLTexture();	// Generate texture from image data.
	
	return true;
}



//////////////////////////////////////////////////////////////////////////////
//
//	-	Reads in a palette file, and generates OpenGL textures from the 
//		existing image data.
//	-	The palette file contains a source palette which contains entries found 
//		in the image data.  It also contains one ore more other palettes which
//		are utilized to generate new image data by remapping colors.
//	-	Structure of a palette file:
//
//		Size	Name					Details
//		----	----------------	------------------------------------------------
//		3		Identifier			The letters "pal"
//		1		Palettes				p = Number of palettes in file.
//		1		Palette size		n = Number of color entries per palette.
//		3np	Palette(s)			Contains n*p # of RGB triplets.
//											(Note, the first palette is the source 
//											palette and the others are target palettes.)
//
//	-	EXAMPLE: 1. A source palette has 175,175,175 (light gray)
//					2. A target palette will have 255,0,0 (red) in the
//						corresponding field.
//					3. All light gray pixel will become red
//
//	ARGS:	filename,		Name of palette file to open.
//
//	PRE:	m_pImageData,	Populated with image data.
//
//	RETURNS:	# of OpenGL textures created (which is also the # of palettes)
//
//////////////////////////////////////////////////////////////////////////////
int cTextureImage::loadPalette(const char *filename)
{
	ifstream						file;	// Handle to palette file.
	vector<cColorRGBPalette>	vPalettes;	// Holds color values.
	
	int iMaxPalettes = 0;					// # of palettes.
	GLubyte*	pImageDataHolder = NULL;	// Holds original image data location.
	


	try
	{
		GLubyte		header[5];		// Head of palette file


		file.open(filename);						// Open palette file.

		// Read in header values.
		file.read(reinterpret_cast<char *>(header), sizeof(header));

		// Make sure file opened and is a valid file.
		if(header[0] != 'p'	|| header[1] != 'a'	|| header[2] != 'l')
		{
			// File header not what was expected.
			throw exception();
		}


		iMaxPalettes		= header[3];	// Number of palettes in file.
		int iMaxEntries	= header[4];	// Number of entries per palette.
		int iPalette;					// palette index

		// Error if invalid # of palettes or # of entries.
		if (iMaxPalettes == 0 || iMaxEntries == 0)
			throw exception();


		// Loop through each palette, storing it's values.
		for (iPalette = 0; iPalette < iMaxPalettes; iPalette++)
		{
			// Add another palette to the collection.
			vPalettes.push_back(cColorRGBPalette());

			// Loop through each entry in the palette, reading the red-green-blue
			// values from disk.
			for (int iEntry = 0; iEntry < iMaxEntries; iEntry++)
				vPalettes[iPalette] += (cColorRGB() << file);

		} // END palette loop


		pImageDataHolder = m_pImageData;	// Store original image data location.

		cColorRGB	color;					// Used for pixel color manipulation.
		int			index;					// Entry position into palette.

		// Create a new image, one for each palette.
		// Start at 1 since palette at 0 is the source palette.
		for (iPalette = 1; iPalette < iMaxPalettes; iPalette ++)
		{
			// Allocate a new set of memory for the modified data.
			m_pImageData = new GLubyte[m_width * m_height * m_ByPP];

			// Loop through each pixel in the image.
			for (	int iPixel = 0; 
					iPixel < m_width * m_height * m_ByPP;
					iPixel += m_ByPP)
			{
				// Set the components of the color to check for from the original
				// image data.
				color.set(	
					pImageDataHolder[iPixel],			// red
					pImageDataHolder[iPixel+1],		// green
					pImageDataHolder[iPixel+2] );		// blue

				// Set alpha-bit if an Alpha channel is present.
				if (m_ByPP > 3)
					m_pImageData[iPixel+3] = pImageDataHolder[iPixel+3];


				// Look for this pixel's color in the source palette.
				if ( (index = vPalettes[0].getEntryPosition(color)) != 0 )
				{
					// Color exists in palette...

					index--;	// Adjust index to real offset value.

					// Set pixel to palette values.
					m_pImageData[iPixel]	=	(vPalettes[iPalette])[index].r();
					m_pImageData[iPixel+1] =(vPalettes[iPalette])[index].g();
					m_pImageData[iPixel+2] =(vPalettes[iPalette])[index].b();
				}
				else
				{
					// No match in palette so set pixel to old value.
					m_pImageData[iPixel]	=		color.r();
					m_pImageData[iPixel+1] =	color.g();
					m_pImageData[iPixel+2] =	color.b();
				}
			}

			createOpenGLTexture();	// Generate texture from image data.

			delete[] m_pImageData;	// Remove the temporary image data allocated.
		}

		
		// Set pointer back to memory location of original data.
		m_pImageData = pImageDataHolder;

	}
	catch(...)
	{
		// Catch all errors and clean up.
		
		file.clear();				// Reset stream state.
		
		if (file.is_open())		// Close file if open.
			file.close();
		
		// Check if the image data pointer was swapped and if memory was
		// allocated to the new pointer.
		if (pImageDataHolder != NULL)
		{
			if (m_pImageData != NULL)				// Delete new memory.
				delete[] m_pImageData;

			m_pImageData = pImageDataHolder;		// Swap back image pointer.
		}

		return 0;	// Exit with an error.
	}

	// Return one less than actual # because first palette is source palette.
	return iMaxPalettes - 1;
}



//////////////////////////////////////////////////////////////////////////////
//
//	Creates and OpenGL texture from a blob of data.
//
//	PRE:	m_pImageData,	Points to data to create an OpenGL texture out of.
//
//////////////////////////////////////////////////////////////////////////////
void cTextureImage::createOpenGLTexture()
{
	GLuint	type;					// Set The Default GL Mode To RBGA (32 BPP)
	GLuint	textureID;

	glGenTextures(1, &textureID);	// Generate OpenGL texture ID for image.
	m_vID.push_back(textureID);	//	Store ID in collection.


	// Bind texture to texture ID.
	glBindTexture(GL_TEXTURE_2D, textureID);						
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	
	// Determine what OpenGL argument to use based on bytes per pixel.
	if (m_ByPP==3)
		type=GL_RGB;
	else
		type=GL_RGBA;
	
	glTexImage2D(GL_TEXTURE_2D, 
		0, type, m_width, m_height, 
		0, type, GL_UNSIGNED_BYTE, m_pImageData);

}


// vi:set ts=3:
