// event.cpp: implementation of the cEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "event.h"          // Defines for this class.
#include "GameException.h"  // Exception handling.

// Defines for forward delcarations.
#include "sprite.h"
#include "eventData.h"
#include "screen.h"
#include "gameState.h"


//////////////////////////////////////////////////////////////////////////////
//
// CONSTRUCTOR
//
//	ARGS:	pGameState,		Game state object.
//			pScreen,			The screen object in which this event exists.
//			oEventData,		Reference to a populated event data object.
//
//////////////////////////////////////////////////////////////////////////////
cEvent::cEvent(cGameState* pGameState, cScreen* pScreen, cEventData& oEventData)
{
	BEGIN_ERRORHANDLER_GAME

	// Set game state object.
	m_pGameState = pGameState;

	// Set related screen
	m_pScreen = pScreen;

	// Initialize the state of the event to inactive to start
	m_eState = m_eDefState = EVENT_INACTIVE;

	// If the event is linked to a sprite, retrieve the sprite it's linked to.
	if ( oEventData.getSpriteID() )
	{
		m_pSprite = m_pScreen->getSprite( (oEventData.getSpriteID() - 1) );

		// Set the sprite to the event as well...we have reciprocal love here.
		m_pSprite->setEvent( this );
	}

	// Set the type of the event.
	m_eType = static_cast<eEVENT_TYPE>(oEventData.getEventType());

	// If it's not a portal event or a dialogue event, get the target event 
	// for the event (if necessary).
	if ( !(oEventData.getEventType() & yEVENT_PORTAL) && 
		(oEventData.getEventType() != EVENT_TYPE_DIALOGUE) )
	{
		if ( oEventData.getTargetID() )
		{
			m_pTargetEvent = m_pGameState->getEvent( oEventData.getTargetID() );
		}
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//	DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cEvent::~cEvent()
{
}


//////////////////////////////////////////////////////////////////////////////
//
//	Triggers event, and activates target.
// 
//	POST:	m_pTargetEvent -- Event becomes activated
//
//////////////////////////////////////////////////////////////////////////////
void cEvent::trigger()
{
	// If the target event exists, and isn't activated, activate it
	if ( m_pTargetEvent != NULL )
	{
		if ( !(m_pTargetEvent->isActive()) )
		{
			m_pTargetEvent->activate();
		}
	}

	// Trigger the event
	m_eState = static_cast< eEVENT_STATE >(m_eState | EVENT_TRIGGERED);
}

// Activate an event.
/*inline*/ void cEvent::activate() { 
	setState(static_cast< eEVENT_STATE >(m_eState | EVENT_ACTIVE)); 
}

// Sets the event to it's default state.
/*inline*/ void cEvent::reset() { setState(m_eDefState); }

// Attach a sprite to the event.
/*inline*/ void cEvent::setSprite( cSprite* pSprite ) { m_pSprite = pSprite; }

///////////////////////////////////////////////////////////////////////////
//	ACCESSORS
///////////////////////////////////////////////////////////////////////////

// Whether or not the event is complete.
/*inline*/ bool cEvent::isCompleted()	{ return ((m_eState & EVENT_COMPLETED) != 0);}

// Whether or not the event is active.
/*inline*/ bool cEvent::isActive()		{ return ((m_eState & EVENT_ACTIVE) != 0);	}

// Whether or not the event has been triggered.
/*inline*/ bool cEvent::isTriggered()	{ return ((m_eState & EVENT_TRIGGERED) != 0);}

// What state the event is in.
/*inline*/ eEVENT_STATE cEvent::getState(){ return m_eState; }	

// What type the event is.
// ???TMH - Remove once a generalized sequence with game engine is made.
/*inline*/ eEVENT_TYPE cEvent::getEventType() { return m_eType; }

// Retrieve a pointer to the attached sprite.
/*inline*/ cSprite* cEvent::getSprite() { return m_pSprite; }
// vi:set ts=3:
