//////////////////////////////////////////////////////////////////////////////
//
//	event.h
//	Defines for the cEvent class.
//
//	-	Base class to which all events must sub-class.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#ifndef __EVENT_H__
#define __EVENT_H__

//#include <vector>                   // STL <vector> (collection with index).
#include "inc/system.h"

#include "ColorRGB.h"           // Color and BYTE support.
#include "ColorRGBPalette.h"           // Color and BYTE support.
#include "inc/EventIDs.h"      // IDs for events.

// Forward defines (truly defined later).
class cSprite;
class cEvent;
class cEventData;
class cScreen;
class cGameState;

// Define type to iterate through collection of sprites.
typedef vector<cEvent*>::iterator EVENT_ITR;

// Collision among events, especially portals, needs to be tighter than normal
// sprite collision.  This is how "picky" the collision
// can be.  As the # approaches 0 it gets more exact, but then it becomes
// a pain to maneuver Link into a portal.
//
// ???TMH - Revisit this now that we have Link snapping to a 'grid' for his
//				movements.
const float fEPSILON_EVENT = 2.0f;

// Bit mask to determine whether an event is a portal.
const BYTE yEVENT_PORTAL	= 0x08;

// The states of an event.
enum eEVENT_STATE
{
	EVENT_INACTIVE		= 0x00,	// Has not yet been activated
	EVENT_ACTIVE		= 0x01,	// Has been activated, awaiting trigger
	EVENT_TRIGGERED	= 0x02,	// Has been triggered, awaiting completion
	EVENT_COMPLETED	= 0x04,	// Has been completed, non-renewable until reset.
};

class cEvent  
{
	///////////////////////////////////////////////////////////////////////////
	//	MEMBERS
	///////////////////////////////////////////////////////////////////////////
protected:
	// Low level game related...
	cGameState*	m_pGameState;	// Keeps track of the state of the game.
	cScreen*		m_pScreen;		// Screen that the event is related to.
	cSprite*		m_pSprite;		// Sprite that the event is attached to. (If
										// NULL, then screen-based)

	// Game play attributes...
	eEVENT_STATE	m_eState;		// Current state of the event.
	eEVENT_STATE	m_eDefState;	// Default state of the event.
	eEVENT_TYPE		m_eType;			// Type of event (portal, movable object, 
										// disappearing object)

	// Target attributes.
	cEvent*	m_pTargetEvent;	// Target event affected by triggering this event.


	///////////////////////////////////////////////////////////////////////////
	//	METHODS
	///////////////////////////////////////////////////////////////////////////
protected:
	cEvent(){}	// Default constructor non-creatable.

	//	Set the state of the event.
	virtual /*inline*/ void setState( eEVENT_STATE eState ) { m_eState = eState; }


public:
	// Constructor
	cEvent(cGameState* pGameState, cScreen* pScreen, cEventData& oEventData);
	
	virtual ~cEvent();			// Destructor


	// Activate an event.
	virtual /*inline*/ void activate();

	// Sets the event to it's default state.
	virtual /*inline*/ void reset();

	// Attach a sprite to the event.
	virtual /*inline*/ void setSprite( cSprite* pSprite );


	// ???TMH - Move to be pure virtual, and take implementation into sub-class.
	virtual void trigger();		// Initial trigger of event.

	// --- PURE VIRTUALS ---
	virtual void update()	= 0;		// Update event when triggered.	


	///////////////////////////////////////////////////////////////////////////
	//	ACCESSORS
	///////////////////////////////////////////////////////////////////////////

	// Whether or not the event is complete.
	/*inline*/ bool isCompleted();

	// Whether or not the event is active.
	/*inline*/ bool isActive();

	// Whether or not the event has been triggered.
	/*inline*/ bool isTriggered();

	// What state the event is in.
	/*inline*/ eEVENT_STATE getState();
	
	// What type the event is.
	// ???TMH - Remove once a generalized sequence with game engine is made.
	/*inline*/ eEVENT_TYPE getEventType();

	// Retrieve a pointer to the attached sprite.
	/*inline*/ cSprite* getSprite();
};

#endif  // __EVENT_H__
// vi:set ts=3:
