//////////////////////////////////////////////////////////////////////////////
//
// eventDialogue.cpp
//	Implementation of the cEventDialogue class.
//
//	-	Handles events in which dialogue (text) is given to the player by
//		displaying it on the screen.
//	-	Tabs set to 3 spaces.
//
//////////////////////////////////////////////////////////////////////////////

#include "eventDialogue.h"

//////////////////////////////////////////////////////////////////////////////
//
// CONSTRUCTOR
//
//	ARGS:	pGameState,		Game state object.
//			pScreen,			The screen object in which this dialogue event exists.
//			oEventData,		Reference to a populated event data object.
//
//////////////////////////////////////////////////////////////////////////////
cEventDialogue::cEventDialogue(	cGameState* pGameState, cScreen* pScreen, 
											cEventData& oEventData )
	:	cEvent( pGameState, pScreen, oEventData )
{
	BEGIN_ERRORHANDLER_GAME
	
	string szMessage;
	string szSubMessage;
	int iCharPosition = 0;
	int iLastCharPosition = 0;
	int iStringCount;


	cMap* pMap;

	// Get a pointer to the current map
	pMap = pScreen->getMap();

	// Set target string ID
	m_iTargetStringID = oEventData.getTargetID();

	// Get target string
	szMessage = *(pMap->getString( m_iTargetStringID - 1 ));

	// Split the string into parts if necessary, and get placement of string
	for ( iStringCount = 0; iStringCount < (szMessage.size() / iLINE_MAX_CHARACTERS) + 1; iStringCount++ )
	{
		if ( szMessage.size() > iCharPosition + iLINE_MAX_CHARACTERS )
		{
			iLastCharPosition = iCharPosition;

			iCharPosition += iLINE_MAX_CHARACTERS;

			// Find the nearest space before the maximum allowed chars on the line
			while ( szMessage.at( iCharPosition ) != ' ' )
			{
				iCharPosition--;
			}

			// Grab the substring from where we last stopped to just before the space
			szSubMessage = szMessage.substr( iLastCharPosition, iCharPosition-iLastCharPosition );
		}
		else
		{
			// If the size is less than maximum, just grab the rest of the string
			szSubMessage = szMessage.substr( iCharPosition );
		}

		// Center the position of the message vertically and horizontally
		// ???KWB - Make it slightly less hardcoded when constants are known
		m_viStringX.push_back( 128 - (szSubMessage.size()<<2) );
		m_viStringY.push_back( 128 - (iStringCount<<3) );
		m_vszFullString.push_back( szSubMessage );
	}

	// Initialize member variables

	// Currently displaying string should be nothing
	m_szCurrentString = "";
	m_iCurrentString = 0;
	m_iCurrentLength = 0;

	// Reset time delay
	m_dwTimeDelay = 0;

	// Set initial state to active and triggered
	m_eState = m_eDefState = static_cast<eEVENT_STATE>(EVENT_ACTIVE | EVENT_TRIGGERED);

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//	DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cEventDialogue::~cEventDialogue()
{
}


//////////////////////////////////////////////////////////////////////////////
//
//	Updates event after trigger
// 
//////////////////////////////////////////////////////////////////////////////
void cEventDialogue::update()
{
	// Make sure adequate delay has occurred between letters
	if ( m_dwTimeDelay > iLETTER_TIMER_DELAY )
	{
		m_dwTimeDelay -= iLETTER_TIMER_DELAY;

		// Check if the currently drawing string is completed
		if ( m_iCurrentLength < m_vszFullString[m_iCurrentString].size() )
		{
			// If not, add another letter to the currently drawing string
			m_szCurrentString += m_vszFullString[m_iCurrentString].at(m_iCurrentLength);
			m_iCurrentLength++;

			// Then, update the gameState's message in the appropriate place
			m_pGameState->setMessage( m_iCurrentString,  &m_szCurrentString, 
				m_viStringX[m_iCurrentString], m_viStringY[m_iCurrentString] );
		}
		// If the full string is up there, move to the next one or complete the event
		else
		{
			m_iCurrentLength = 0;
			m_szCurrentString = "";
			m_iCurrentString++;

			if ( m_iCurrentString >= m_vszFullString.size() )
			{
				m_eState = EVENT_COMPLETED;

				// Make sure Link can move again
				m_pGameState->unfreezeLink();
			}
		}
	}
	else
	{
		m_dwTimeDelay += m_pGameState->getTimeLapseMS();
	}
}


//////////////////////////////////////////////////////////////////////////////
//
//	Resets event.
//
//	POST:	All member variables are reset.
// 
//////////////////////////////////////////////////////////////////////////////
void cEventDialogue::reset()
{
	// Reset necessary variables
	m_szCurrentString = "";
	m_iCurrentLength = 0;
	m_iCurrentString = 0;
	m_dwTimeDelay = 0;
	m_eState = m_eDefState;
}

///////////////////////////////////////////////////////////////////////////
//	ACCESSORS
///////////////////////////////////////////////////////////////////////////
	
// Retrieve the ID for the target string.
/*inline*/ int cEventDialogue::getTargetStringID()	{ return m_iTargetStringID;	}

// Retrieve the strings.
/*inline*/ string* cEventDialogue::getFullString( int iStringIDNumber )
{ return &(m_vszFullString[ iStringIDNumber ]);	}

// Obtain the current string.
/*inline*/ string* cEventDialogue::getCurrentString()	{ return &m_szCurrentString;	}
// vi:set ts=3:
