//////////////////////////////////////////////////////////////////////////////
//
// gameState.cpp
//	Implementation of the cGameState class.
//
//	-	Tabs set to 3 spaces.
//
//////////////////////////////////////////////////////////////////////////////

#include "gameState.h"	// Headers for project.
#include "GameException.h"

#include "tiles.h"
#include "sprite.h"
#include "screen.h"
#include "map.h"
#include "event.h"
#include "eventPortal.h"
#include "inventory.h"
#include "objectLynk.h"

const float fSCROLL_SPEED = 3.5f;		// Speed at which screen scrolls.
//const float fSCROLL_SPEED = 14.0f;		// hyperfast

// ???TMH - Change these lists to be read from the configuration file.
const int MAX_GFX_FILES = 5;				// Number of tile set to load.
char pszTileSetNames[][MAX_MAP_NAME] =	// Names of tile sets to load...
	{
		"menu",						// Font, and menu graphics
		"good_characters",		// Link, Zelda, fairy, old man, etc...
		"items",						// Items, weapons, and weapon shots
		"overworld",				// Background for the overworld
		"enemies_overworld",		// Enemies who reside in the overworld
//		"dungeon",					// Background for dungeons
//		"enemies_dungeon",		// Enemies who reside in dungeons
//		"bosses_dungeon",			// Bosses in the dungeons		
	};

// ???TMH - Change MAX_MAP_FILES once the dungeon map data is created.
const int MAX_MAP_FILES = 2;			// Number of maps to load.
char pszMapNames[][MAX_MAP_NAME] =	// Names of maps to load...
	{
		"overworld",
		"caves",
//		"dungeon01",
//		"dungeon02",
//		"dungeon03",
//		"dungeon04",
//		"dungeon05",
//		"dungeon06",
//		"dungeon07",
//		"dungeon08",
//		"dungeon09"
	};

//////////////////////////////////////////////////////////////////////////////
//
//	Resets the game state.
//
//	ARGS:	pWindow,	Window object to get native window system information
//						(keyboard, window dimension, etc...)
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::initialize(cWindow *pWindow)
{
	BEGIN_ERRORHANDLER_GAME

	cMap*		pMap;	// Temporary used to load a map.
	cTiles*	pTiles;		// Temporary used to load a tile set.
	int		iCurrent;	// Offset of the current object being loaded.

	m_pWindow			= pWindow;				// Window system object.
	m_dwLastTickCount = timer_ms_gettime64();	// Obtain initial time value. (milliseconds)
	//printf(">>init time=%i\n", m_dwLastTickCount);
	m_pCurrentMap		= NULL;					// Stuff with clean empty value.
	m_pCurrentScreen	= NULL;					// Stuff with clean empty value.

	m_pTriggeredPortal= NULL;					// No portal is triggered at the start
	m_ePortalState = PORTAL_OFF;				// Initialize substate variable to off
	m_eLastPortalType = EVENT_TYPE_UNDEFINED;		// Initialize last portal to invalid type

	// dreamcast controller setup
	controller1 = maple_first_controller();

	// ???KWB - Slight hack to get Link to freeze manually
	m_bFreezeLink = false;						// Link should be able to move from the start

	// Initialize messages so nothing shows up on screen
	clearMessages();

	// ???TMH - Add reading from .CFG file here...
	printf("\n>> loading tiles..\n");

	// Load the tile sets (graphics) used for maps, sprites, etc...
	for (iCurrent = 0; iCurrent < MAX_GFX_FILES; iCurrent++)
	{
		// Create & load tiles.
		// ???TMH - Change hard coded "16"s to be values from config file.
		if (iCurrent == 0)
			pTiles = new cTiles(pszTileSetNames[iCurrent], 8, 8);		// Menu
		else
			pTiles = new cTiles(pszTileSetNames[iCurrent], 16, 16);	// Other gfx

		// Add the newly loaded set of graphics to the game state.
		addTiles(pTiles);
	}

	printf("\n>> loading maps..\n");

	// Load all of the the maps...
	for(iCurrent = 0; iCurrent < MAX_MAP_FILES; iCurrent++)
	{
		// Create a dungeon with underworld tiles.
		pMap = new cMap(this, pszMapNames[iCurrent]);

		// Add map to collection.
		m_vpMaps.push_back(pMap);
	}

	// Set the map to the overworld (first map in vector) and set the screen
	// to be shown be where link starts.
	setMap( m_vpMaps[MAP_OVERWORLD] );

	// Initialize inventory.
	m_pInventory = new cInventory(this);

	//  Initialize player object.
	m_pLink = new cObjectLynk(this, m_pInventory);

	// Set the OpenGL states...
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// Black background on clears.
	
	//glClearDepth(1.0f);						// XXX DOES NOT WORK ON DREAMCAST
													// this FUCKS everything.
	
	glDisable(GL_CULL_FACE);				// Remove back face	
	glEnable(GL_TEXTURE_2D);				// Enable texture mapping

	// Depth testing is on, so OpenGL will manage which objects overlap
	// other objects.  Otherwise it would be necessary for the program to
	// draw all object's in order.
	glDepthFunc(GL_LEQUAL);						// Type Of Depth Testing

	// glEnable(GL_DEPTH_TEST);					//  XXX NO EFFECT ON DREAMCAST
	//glDepthMask(GL_TRUE);

	// Enable Alpha Blending (disable alpha testing)
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
	
	END_ERRORHANDLER_GAME
}

//////////////////////////////////////////////////////////////////////////////
//	CONSTRUCTOR
//////////////////////////////////////////////////////////////////////////////

cGameState::cGameState()
{
	m_eState					= PLAY_GAME;	// Start game out playing.
	m_eNextState			= PLAY_GAME;	// ""
	m_eScrollDirection	= DOWN;			// Give an (arbitrary) scroll direction.

	// ???TMH - Change to initialize from window class.
	m_fPlayfieldWidth = 256.0f;
	m_fPlayfieldHeight = 176.0f;
	//m_fPlayfieldWidth = 512.0f;
	//m_fPlayfieldHeight = 352.0f;
	
	m_fScreenWidth = 640;
	m_fScreenHeight = 480;

	// ???TMH - Pass these in too instead of hard coded initialize?  Should
	//				these go into cSprite now, or is this something that cScreen
	//				is really responsible for?  Research.
	// (Default) boundries for sprites.
	m_fSpriteBoundryMinX = 16.0f;
	m_fSpriteBoundryMaxX = 240.0f;
	m_fSpriteBoundryMinY = 16.0f;
	m_fSpriteBoundryMaxY = 160.0f;

	// Set common inventory items to NULL.
	m_pInventory	= NULL;
	m_pLink			= NULL;

	// Timing.
	m_dwLastTickCount	= 0;
	m_dwTimeLapseMS	= 0;
	m_fFrameFactor		= 0.0f;
	m_fScrollAmount	= 0.0f;
}


//////////////////////////////////////////////////////////////////////////////
//	DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cGameState::~cGameState()
{
	vector<cTiles*>::iterator itrTiles;

	// Free the memory held by each tile object in the collection.
	for (itrTiles = m_vpTiles.begin(); itrTiles != m_vpTiles.end(); itrTiles++)
		delete (*itrTiles);

	// Free player object if it exists.
	if (m_pLink != NULL)
	{
		delete m_pLink;
		m_pLink = NULL;
	}

	// Free inventory object if it exists.
	if (m_pInventory != NULL)
	{
		delete m_pInventory;
		m_pInventory = NULL;
	}

	m_vpTiles.clear();	// Empty memory held by the collection itself.

	m_pbKeyStatus = NULL;	// Lose keyboard buffer location.
	
}



//////////////////////////////////////////////////////////////////////////////
//
//	-	Shut down the game.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::terminate()
{
	// Free memory from every map in collection, then delete collection.
	for (MAP_ITR itrMap = m_vpMaps.begin(); itrMap != m_vpMaps.end(); itrMap++)
		delete (*itrMap);
	m_vpMaps.clear();
}


//////////////////////////////////////////////////////////////////////////////
//
//	Determines the # of millseconds since this function was last called.
//
//	RETURN:	Number of millseconds since last call.
//
//////////////////////////////////////////////////////////////////////////////
unsigned int cGameState::computeTimeLapseMS()
{
	unsigned int dwNewLastTickCount = m_dwLastTickCount;	// Save old value.

	// Loop until at least 15ms have passed.
	while ((m_dwLastTickCount - dwNewLastTickCount) < 15)
		m_dwLastTickCount = timer_ms_gettime64();

	return m_dwTimeLapseMS = (m_dwLastTickCount - dwNewLastTickCount);

	return 30;
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Determine a factor to multiple all sprite movement's by so they move
//		consistently no matter what the frame rate.
//	-	Original source code from vidGL's getFrameFactor() by Mobydisk
//		(http://www.mobydisk.com)
//	-	???TMH - Assumes a 70fps, which works on my K6-2 500mHz with TNT2, but
//					may cause very fast sprites on a higher end system.  Need to
//					add a wait loop if we exceed 70fps.
//				 - Also of note, I forced a 15 millsecond wait in determining
//					the amount of milleseconds which have passed.  I need to
//             test this on other systems.
//	-	The faster it's updated the lower the value.
//
//			EXAMPLE:	1fps = 1.0		(choppy animation steps)
//						2fps = 0.5
//						3fps = 0.3333
//						4fps = 0.25
//						:		 :
//						70fps= 0.0143	(very smooth animation steps)
//
//	POST:	m_fFrameFactor,	Holds new frame factor values.
//
//	RETURN:	The frame factor value.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::computeFrameFactor()
{
	// If the time lapse is more than a second (or some whack value), force
	// value to 1fps so sprites don't jump through objects by having a HUGE
	// animation step to make. (This comes at the cost of slowing eveything
	// down.)
	if (m_dwTimeLapseMS <= 0 || m_dwTimeLapseMS > 1000)
		m_fFrameFactor = 1.0f;
	// To achieve the constant 14.28571 = (1000.0ms / 70fps)
	else
		m_fFrameFactor = m_dwTimeLapseMS / 14.28571f;	
}


//////////////////////////////////////////////////////////////////////////////
//
//	Set which map the game is currently working against.
//
//	ARGS:	pMap,	A point to a map.
//			x,		[-1] Horizontal coordinate of inital screen on the map to use.
//			y,		[-1] Verticlal coordinate of inital screen on the map to use.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::setMap(cMap *pMap, int x, int y)
{ 
	BEGIN_ERRORHANDLER_GAME

	m_pCurrentMap	 = pMap;	// Map to use.

	// Determine if starting screen on map will be loaded from default, or
	// from explicit coodinates.
	if (x != -1 && y != -1)
	{
		// Use explicit coordinates obtain starting screen.
		m_iMapPositionX = x;
		m_iMapPositionY = y;

	} else {
		// Use defaults to obtain starting screen.
		m_iMapPositionX = m_pCurrentMap->getStartScreenX();
		m_iMapPositionY = m_pCurrentMap->getStartScreenY();
	}

	setScreen();				// Now, actually obtain the screen.

	END_ERRORHANDLER_GAME
}	


//////////////////////////////////////////////////////////////////////////////
//
// -	Sets the current screen.
//	-	This is a two step process:
//			1. The screen object is obtained from the current map.
//			2.	The sprites for that screen are loaded as the current sprites.
//
//	PRE:	m_iMapPositionX,	Set to the new horizontal coordinate of the screen
//									within the map.
//			m_iMapPositionY,	Setto the new vertical coordinate of the screen
//									within the map.
//
//	POST:	m_pCurrentScreen,	Set to screen at (x,y)
//			m_vpSprites,		Set to collection of sprites.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::setScreen()
{
	BEGIN_ERRORHANDLER_GAME

	// Boundry check the coordinates of the screen to obtain.	

	if (m_iMapPositionX < 0)
		m_iMapPositionX = 0;

	if (m_iMapPositionX > (m_pCurrentMap->getWidth() - 1) )
		m_iMapPositionX = (m_pCurrentMap->getWidth() - 1);

	if (m_iMapPositionY < 0)
		m_iMapPositionY = 0;

	if (m_iMapPositionY > (m_pCurrentMap->getHeight() - 1) )
		m_iMapPositionY = (m_pCurrentMap->getHeight() - 1);

	// ???TMH - Add code to obtain recent screens from a cache.  This is
	//				to prevent all enemies from appearing on a screen Link was
	//				just at.

	// Get screen.
	m_pCurrentScreen = 
		m_pCurrentMap->getScreen(m_iMapPositionX,m_iMapPositionY);

	// Get sprites from screen.
	m_vpSprites = m_pCurrentScreen->getSprites();

	// Get events from screen.
	m_vpEvents = m_pCurrentScreen->getEvents();

	// Reset sprites on the screen as they are about to be shown.
	for(SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++)
		(*itr)->reset();

	// Reset events so they are untriggered and inactive
	for(EVENT_ITR itrEvent = m_vpEvents.begin(); itrEvent != m_vpEvents.end(); itrEvent++)
		(*itrEvent)->reset();

	// Reset the message string so it doesn't stay up
	clearMessages();

	END_ERRORHANDLER_GAME
}



//////////////////////////////////////////////////////////////////////////////
//
//	-	Call each time the game needs to be updated.
//	-	Computes how long it took for this function to be called since it's
//		previous call.
//	-	Updates all of the sprites on the screen.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::update()
{
	BEGIN_ERRORHANDLER_GAME

	computeTimeLapseMS();	// Determine time difference.	
	computeFrameFactor();	// Adjust the factor on how fast to move sprites.

	m_eState = m_eNextState;	// Set new state (if changed).

	//???DAC - GLOBAL key updates
	
	// update the input
	if (cont_get_cond(controller1, &cond) < 0) {
		printf(">> Error reading controller\n");
	}

	// EXIT APP ON START BUTTON PRESS

	// Update the contents of the game based on what state it is in.
	switch(m_eState)
	{
		case PLAY_GAME:
		{
			updatePlayGame();
			break;
		}
		case SCROLL_GAME:
		{
			updateScrollGame();
			break;
		}
		case PORTAL_GAME:
		{
			updatePortalGame();
			break;
		}
		default:
		{
			// Unknown game state, make an error message and throw.
			THROW_GAME_INT("Game state attempted an update while in an unknown "
				"state.", m_eState);
		}
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Calls update on a collection of sprites.
//	-	Performed removal of "dead" sprites.
//
//	ARGS: vpSpriteList,	The list of sprites to work on.
//
//	POST:	vpSpriteList,	May contain less sprites if any are dead.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::updateSprites(vector<cSprite*>& vpSpriteList )
{
	BEGIN_ERRORHANDLER_GAME

	// Go through the list of sprites and tell them to update themselves.
	for(	SPRITE_ITR itr = vpSpriteList.begin();
			itr != vpSpriteList.end();
			itr++)
	{
		// Tell sprite to perform it's update.
		(*itr)->update();

		// Check if the current sprite needs to be set inactive or removed
		// from the list.
		if ( (*itr)->isDead() && ((*itr)->getState() & ACTIVE) )
		{
			// Set state to be inactive, allowing sprite to clean-up anything
			// it may have going on.
			(*itr)->setState(INACTIVE);

			// Check if sprite is to be auto-deleted.  If not, it means some
			// other object is maintaining it's upkeep.
			if ((*itr)->isDeleteable())
			{
				// Yes this can be autodeleted...

				delete (*itr);		// Free the memory from the sprite.
				(*itr) = NULL;		// Set pointer to NULL.
			}

			// Remove the dead sprite from the list.
			itr = vpSpriteList.erase(itr);	

			// May need to break out of loop immediately if the sprite erased 
			// was the last sprite in the list.
			if (itr == vpSpriteList.end())
				break;			

		} // END if dead

	} // END for each sprite

	END_ERRORHANDLER_GAME
}

//////////////////////////////////////////////////////////////////////////////
//
//	-	Checks for trigger of active events.
//	-	Performs tasks for triggered events.
//
//	ARGS: vpEventList,	The list of events to work on.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::handleEvents(vector<cEvent*>& vpEventList )
{
	eEVENT_STATE eState;

	BEGIN_ERRORHANDLER_GAME

	// Go through the list of sprites and tell them to update themselves.
	for(	EVENT_ITR itr = vpEventList.begin();
			itr != vpEventList.end();
			itr++)
	{
		eState = (*itr)->getState();

		// Check for a triggered, active event.
		if ( (eState & EVENT_TRIGGERED) &&
			(eState & EVENT_ACTIVE) )
		{
			// ???TMH - We need to generalize the calls between the engine and
			//				events so we don't have to do a type look-up.  (This 
			//				switch statement works, but isn't an OO solution.)
			switch( (*itr)->getEventType() )
			{
				case EVENT_TYPE_DIALOGUE:
				{
					m_bFreezeLink = true;
					(*itr)->update();
					break;
				}

				default:
				{
					THROW_GAME_INT( "Invalid event triggered in handleEvents()", 
						(*itr)->getEventType() );
					break;
				}
			};
		}


	} // END for each event

	END_ERRORHANDLER_GAME
}

//////////////////////////////////////////////////////////////////////////////
//
//	Performs update during a normal game play.
//
//	POST:	m_vpSprites,	May contain less sprites if any are dead.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::updatePlayGame()
{
	BEGIN_ERRORHANDLER_GAME

	updateSprites(m_vpSprites);	// Update active sprites.

	// Reset Link's movement ability
	m_bFreezeLink = false;

	handleEvents(m_vpEvents);		// Update active events


	// ???TMH - Change this member variable to a name for freezing all game 
	//				elements, and have the move() of sprites check it (via an 
	//				accessor) before moving the sprite.  
	//				Once this is done, remove the kluge here.
	if ( !(m_bFreezeLink) )
		m_pLink->update();				// Update player.


	// Continually make passes against the add sprite container until empty.
	while( !m_vpAddSprites.empty() )
	{
		// Create yet another temp sprite holder.  This is in case the update
		// on the temp sprites performs more adds.
		vector<cSprite*> vpAddSpritesTemp;

		// A safety feature, as this while loop should never be executed more
		// than one or two times.
		int iThinkWereInAnInfiniteLoop = 0;
		if ((iThinkWereInAnInfiniteLoop++) > 10)
			printf("INFINITE LOOP!\n");
			THROW_GAME("An infinite loop situation occured while updating.");

		// Copy sprites and clear the list which can be added to.
		vpAddSpritesTemp = m_vpAddSprites;
		m_vpAddSprites.clear();

		// Theoretically, this call could add even more spites into the
		// active sprite list.
		updateSprites(vpAddSpritesTemp);

		// Loop through new sprites list and add them to the active list.
		for(	SPRITE_ITR itr = vpAddSpritesTemp.begin();
				itr != vpAddSpritesTemp.end();
				itr++)
		{
			m_vpSprites.push_back(*itr);
		}

		// Explicitly clear out the temp container of sprites.
		vpAddSpritesTemp.clear();
	}

	// ???TMH - Temporary hack to delay keyboard input.
	static float fTotalDelay = 0.0f;

	fTotalDelay += getTimeLapseMS();
	if (fTotalDelay < 100.0f)
		return;
	fTotalDelay = 0.0f;
		
	// ???TMH - Debug code to force screen movement.
	
	// use Y + DPAD to invoke it
	if (!(cond.buttons & CONT_Y)) {

		// ------- UP a screen -------	
		if (!(cond.buttons & CONT_DPAD_UP))
		{
			m_iMapPositionY--;
			setScreen();
		}

		// ------- RIGHT a screen -------
		if (!(cond.buttons & CONT_DPAD_RIGHT))
		{
			m_iMapPositionX++;
			setScreen();
		}

		// ------- DOWN a screen -------
		if (!(cond.buttons & CONT_DPAD_DOWN))
		{
			m_iMapPositionY++;
			setScreen();
		}
	
		// ------- LEFT a screen -------
		if (!(cond.buttons & CONT_DPAD_LEFT))
		{
			m_iMapPositionX--;
			setScreen();
		}
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Performs update during a gameplay scroll, from one screen to the next.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::updateScrollGame()
{
	BEGIN_ERRORHANDLER_GAME

	// Check if scroll is just starting.  If so, setup for the scroll...
	if (m_vpOldSprites.size() == 0)
	{
		m_vpOldSprites = m_vpSprites;		// Copy sprites.

		// ???TMH - Here seems to be the place to add logic for special 
		//				scrolling conditions (such as the lost woods, etc...)

		// Switch position of screen to obtain from map.
		// If a boundry is hit, wrap around to other side.
		switch(m_eScrollDirection)
		{
			case DOWN:
				m_iMapPositionY++; 
				if (m_iMapPositionY == m_pCurrentMap->getHeight())
					m_iMapPositionY = 0;
				break;

			case UP:
				m_iMapPositionY--;
				if (m_iMapPositionY < 0)
					m_iMapPositionY = m_pCurrentMap->getHeight() - 1;
				break;

			case LEFT:
				m_iMapPositionX--;
				if (m_iMapPositionX < 0)
					m_iMapPositionX = m_pCurrentMap->getWidth() - 1;
				break;

			case RIGHT:
				m_iMapPositionX++;
				if (m_iMapPositionX == m_pCurrentMap->getWidth())
					m_iMapPositionX = 0;
				break;

			default:
			{
				// Unknown scroll direction. Throw an error...
				THROW_GAME_INT("During a scroll update, game state recieved an "
					"unknown direction value.",
					m_eScrollDirection);
			}

		} // END switch direction

		// Get the screen associated with the new map position.
		setScreen();

		// Reset the amount the screen has scrolled.
		m_fScrollAmount = 0;

	} // END if scroll is just starting.


	// Scroll some more.
	m_fScrollAmount += fSCROLL_SPEED * m_fFrameFactor;

	// Check when to stop scrolling left/right.
	if ((m_fScrollAmount > m_fPlayfieldWidth) && 
		(m_eScrollDirection == LEFT || m_eScrollDirection == RIGHT))
	{
		m_fScrollAmount = m_fPlayfieldWidth;	// Don't over scroll.
		m_vpOldSprites.clear();	// Remove sprites from previous screen.
		setState(PLAY_GAME);
		m_pLink->flipScreenPositionAfterScroll();
	}

	// Check when to stop scrolling up/down.
	if ((m_fScrollAmount > m_fPlayfieldHeight) && 
		(m_eScrollDirection == UP || m_eScrollDirection == DOWN))
	{
		m_fScrollAmount = m_fPlayfieldHeight;	// Don't over scroll.
		m_vpOldSprites.clear();	// Remove sprites from previous screen.				
		setState(PLAY_GAME);
		m_pLink->flipScreenPositionAfterScroll();
	}

	m_pLink->update();		// Update Link (make walk and move position).
	m_pInventory->update();	// Draw inventory screen and inventory game items.


	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Performs update during a gameplay portal, from one screen to the next.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::updatePortalGame()
{
	float fx, fy;
	BEGIN_ERRORHANDLER_GAME


	switch( m_ePortalState )
	{
		// First time into the portal gamestate
		case PORTAL_INIT:
		{
			if ( m_pTriggeredPortal )
			{
				// Deactivate enemies
				for ( SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++ )
				{
					// ???KWB - Is there another way to distinguish an enemy?
					if ((*itr)->getCollisionMask() & SPRITE_COLLIDE_ENEMY )
					{
						(*itr)->setState( INACTIVE );
					}
				}

				// Save map/screen information from where Link is leaving

				// Get the map Link is leaving from
				m_pLastMap = m_pCurrentMap;

				// Get the screen Link is leaving from
				m_iLastScreenX = getCurrentScreenX();
				m_iLastScreenY = getCurrentScreenY();

				// Set up Link to align perfectly with the portal
				m_pLink->setDirection( UP );
				fx = m_pTriggeredPortal->getSprite()->getX();
				fy = m_pTriggeredPortal->getSprite()->getY();
				m_pLink->setPosition( fx, fy );

				// Set Link's priority underneath the ground
				m_pLink->setDepth( Z_PLANE_SECRET );

				// Get the spot Link was at
				m_fLastX = m_pLink->getX();
				m_fLastY = m_pLink->getY();

				// Get the type of portal that it was
				m_eLastPortalType = m_pTriggeredPortal->getEventType();

				// If Link is supposed to "walk in" to the portal, make it so he does
				if ( (m_pTriggeredPortal->getEventType() == EVENT_TYPE_PORTAL_WALK_INACTIVE) ||
				     (m_pTriggeredPortal->getEventType() == EVENT_TYPE_PORTAL_WALK_ACTIVE) )
				{
					setPortalState( PORTAL_WALK_IN );
				}
				else
				{
					setPortalState( PORTAL_OPEN );
				}
			}
			else
			{
				THROW_GAME( "Portal initialization was triggered without a portal" );
			}
			break;
		}
		case PORTAL_WALK_IN:
		{
			// Make Link walk into the portal
			m_pLink->update();
			m_pLink->setPosition( m_pLink->getX(), m_pLink->getY()-(m_fFrameFactor * 0.2f) );

			if ( m_fLastY - m_pLink->getY() >= m_fSpriteBoundryMinY )
			{
				setPortalState( PORTAL_OPEN );
			}
			break;
		}
		case PORTAL_OPEN:
		{
			// Change maps, screens, etc
			if ( !(m_pTriggeredPortal) )
			{
				// If the game gets here, it's exiting a cave or dungeon
				setMap( m_pLastMap, m_iLastScreenX, m_iLastScreenY );

				// Deactivate enemies temporarily
				for ( SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++ )
				{
					// ???KWB - Is there another way to distinguish an enemy?
					if ((*itr)->getCollisionMask() & SPRITE_COLLIDE_ENEMY )
					{
						(*itr)->setState( INACTIVE );
					}
				}

				// Set him looking down on exits
				m_pLink->setDirection( DOWN );

				// Set Link's depth underneath the floor for exiting
				m_pLink->setDepth( Z_PLANE_SECRET );

				// If Link is supposed to "walk out" of the portal, make it so he does
				if ( (m_eLastPortalType == EVENT_TYPE_PORTAL_WALK_INACTIVE) ||
				     (m_eLastPortalType == EVENT_TYPE_PORTAL_WALK_ACTIVE) )
				{
					// Set Link's position back to a square below where he was
					m_pLink->setPosition( m_fLastX, m_fLastY-m_fSpriteBoundryMinY );
					setPortalState( PORTAL_WALK_OUT );
				}
				else
				{
					// Set Link's position back to where he was
					m_pLink->setPosition( m_fLastX, m_fLastY );
					setPortalState( PORTAL_END );
				}
			}
			else
			{
				// If the game gets here, Link is porting to a new screen
				// Port Link to the target screen, in the target map.
				setMap( m_vpMaps[ m_pTriggeredPortal->getTargetMapID() ], m_pTriggeredPortal->getTargetScreenX(), m_pTriggeredPortal->getTargetScreenY() );

				// Set Link's position and direction as necessary
				// ???KWB - For now, it's hard-coded, so it works for the caves and dungeons
				m_pLink->setPosition( 112.0f, 0.0f );
				m_pLink->setDirection( UP );

				// Set Link's depth back to normal for entering caves and dungeons
				m_pLink->setDepth( Z_PLANE_CHARACTER );

				if ( m_pTriggeredPortal->getTargetMapID() == MAP_CAVES )
				{
					setPortalState( PORTAL_WALK_OUT );
				}
				else
				{
					setPortalState( PORTAL_END );
				}
			}
			break;
		}
		case PORTAL_WALK_OUT:
		{
			if ( m_pTriggeredPortal )
			{
				// Make Link walk out into the cave
				m_pLink->update();
				m_pLink->setPosition( m_pLink->getX(), m_pLink->getY()+(m_fFrameFactor * 1.0f) );

				if ( m_pLink->getY() >= (m_fSpriteBoundryMinY / 2) )
				{
					setPortalState( PORTAL_END );
				}
			}
			else
			{
				// Make Link walk out of the cave into the overworld
				m_pLink->update();
				m_pLink->setPosition( m_pLink->getX(), m_pLink->getY()+(m_fFrameFactor * 0.2f) );

				fy = m_pLink->getY();

				// Have to allow for some margin of error
				if ( (fy >= m_fLastY - 0.2f) && (fy <= m_fLastY + 0.2f) )
				{
					fy = m_fLastY;
					fx = m_fLastX;
					setPortalState( PORTAL_END );
				}
			}
			break;
		}
		case PORTAL_END:
		{
			// Make sure Link's priority returns to normal
			m_pLink->setDepth( Z_PLANE_CHARACTER );

			// Reactivate enemies
			for ( SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++ )
			{
				// ???KWB - Is there another way to distinguish an enemy?
				if ((*itr)->getCollisionMask() & SPRITE_COLLIDE_ENEMY )
				{
					(*itr)->setState( ACTIVE );
				}
			}

			// Reset all the states and the triggered portal
			setPortalState( PORTAL_OFF );
			setState( PLAY_GAME );
			m_pTriggeredPortal = NULL;
			break;
		}
		default:
		{
			THROW_GAME_INT( "Game attempted to enter an invalid portal substate"
					"m_ePortalState = ", m_ePortalState );
			break;
		}
	};

	END_ERRORHANDLER_GAME
}

//////////////////////////////////////////////////////////////////////////////
//
//	-	Call each time the screen content of the game needs to be updated.
//	-	The more often this is called per second, the smoother the animation of
//		the sprites in the game will look.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::draw()
{
	BEGIN_ERRORHANDLER_GAME


	// Clear the contents of the screen.
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glLoadIdentity();		// Reset the view.

	// Draw the contents of the game based on what state it is in.
	switch(m_eState)
	{
		case PLAY_GAME:
		{
			// Go through the list of sprites active for this screen and tell
			// them to draw themselves.

			for(SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++)
				if ((*itr)->getState() & ACTIVE)
				{
					if ( (*itr)->getCollisionMask() & SPRITE_COLLIDE_ENEMY )
					{
						BEGIN_ERRORHANDLER_GAME
						(*itr)->draw();
						END_ERRORHANDLER_GAME
					}
					else if ( (*itr)->getCollisionMask() & SPRITE_COLLIDE_PLAYER )
					{
						BEGIN_ERRORHANDLER_GAME
						(*itr)->draw();
						END_ERRORHANDLER_GAME
					}
					else if ( (*itr)->getCollisionMask() & SPRITE_COLLIDE_SOLID )
					{
						BEGIN_ERRORHANDLER_GAME
						(*itr)->draw();
						END_ERRORHANDLER_GAME
					}
					else
					{
						BEGIN_ERRORHANDLER_GAME
						(*itr)->draw();
						END_ERRORHANDLER_GAME
					}
				}

			// Display text message if necessary
			BEGIN_ERRORHANDLER_GAME
			for ( int iMessageCount = 0;
					iMessageCount < 3;
					iMessageCount++ )
			{
				if ( m_vszMessages[ iMessageCount ] != "" )
				{
					getTextTiles()->drawString( m_vszMessages[ iMessageCount ],
						m_viMsgX[ iMessageCount ], m_viMsgY[ iMessageCount ] );
				}
			}
			END_ERRORHANDLER_GAME

			BEGIN_ERRORHANDLER_GAME
			m_pLink->draw();		// Draw player object.
			END_ERRORHANDLER_GAME

			break;
		}


		case SCROLL_GAME:
		{
			// Scroll between two screens.

			SPRITE_ITR itr;
			float fSign = 1.0f;	// Helps determine direction of translate.


			// Switch scroll direction to negative if moving up or left.
			if (m_eScrollDirection == UP || m_eScrollDirection == RIGHT)
				fSign = -1.0f;


			// Move sprites out of the way for the old sprites.
			if (m_eScrollDirection == UP || m_eScrollDirection == DOWN)
				glTranslatef(0.0f, fSign * m_fScrollAmount, 0.0f);
			else
				glTranslatef(fSign * m_fScrollAmount, 0.0f, 0.0f);

			// Draw old sprites, from screen which was just shown.
			for(itr = m_vpOldSprites.begin(); itr != m_vpOldSprites.end(); itr++)
				(*itr)->draw();

			//Link draw used to be here.
			glPushMatrix(); //save matrix for link's draw()
  
			// Move sprites out of the way for the old sprites.
			if (m_eScrollDirection == UP || m_eScrollDirection == DOWN)
				glTranslatef(0.0f , -fSign * m_fPlayfieldHeight, 0.0f);
			else
				glTranslatef(-fSign * m_fPlayfieldWidth , 0.0f, 0.0f);

			// Draw sprites for screen about to show up.
			for(itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++)
				(*itr)->draw();

			glPopMatrix(); //restore matrix so link will appear in the right place

			//Link will slowly move in the direction of the new screen
			if (m_eScrollDirection == UP || m_eScrollDirection == DOWN)
				glTranslatef(0.0f, -fSign * m_fScrollAmount * 0.08f, 0.0f);
			else
				glTranslatef(-fSign * m_fScrollAmount * 0.05f, 0.0f, 0.0f);

			m_pLink->draw();  // Draw Link. He needs to be drawn last so the void doesn't show.

			break;
		}

		case PORTAL_GAME:
		{
			// ???KWB - Draw sprites unless the portal substate is PORTAL_OPEN.
			//				That gets a nice quick "black screen" effect.

			if ( m_ePortalState != PORTAL_OPEN )
			{
				// Go through the list of sprites active for this screen and tell
				// them to draw themselves.
				BEGIN_ERRORHANDLER_GAME

				for(SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++)
					if ((*itr)->getState() & ACTIVE)
						(*itr)->draw();
				END_ERRORHANDLER_GAME

				BEGIN_ERRORHANDLER_GAME
				m_pLink->draw();		// Draw player object.
				END_ERRORHANDLER_GAME
			}

			break;
		}
		default:
		{
			// Unknown game state, make an error message and throw.	
			THROW_GAME_INT("Game state attempted a draw while receiving an "
				"unknown state value.",
				m_eState);
		}

	} // END game state

	// draw the inventory
	m_pInventory->draw();

	// DREAMCAST ONLY
	// need to draw black bars because there is NO clipping on DC
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glColor3f(0.0f, 0.0f, 0.0f);
	// values are exxagerated
	glBegin(GL_QUADS);
		glVertex3f(-150.0f, -150.0f, Z_PLANE_CEILING); // bottom
		glVertex3f(	850.0f, -150.0f, Z_PLANE_CEILING);
		glVertex3f( 850.0f,    0.0f, Z_PLANE_CEILING);
		glVertex3f(-150.0f,    0.0f, Z_PLANE_CEILING);
		glVertex3f(-150.0f, 224.0f, Z_PLANE_CEILING); // top
		glVertex3f(	850.0f, 224.0f, Z_PLANE_CEILING);
		glVertex3f( 850.0f, 850.0f, Z_PLANE_CEILING);
		glVertex3f(-150.0f, 850.0f, Z_PLANE_CEILING);
		glVertex3f(-150.0f, -100.0f, Z_PLANE_CEILING); // left
		glVertex3f(-150.0f,  850.0f, Z_PLANE_CEILING);
		glVertex3f(   0.0f,  850.0f, Z_PLANE_CEILING);
		glVertex3f(   0.0f, -100.0f, Z_PLANE_CEILING);
		glVertex3f(256.0f, -100.0f, Z_PLANE_CEILING); // right
		glVertex3f(256.0f,  850.0f, Z_PLANE_CEILING);
		glVertex3f(850.0f,  850.0f, Z_PLANE_CEILING);
		glVertex3f(850.0f, -100.0f, Z_PLANE_CEILING);
	glEnd();
	glColor3f(1.0f, 1.0f, 1.0f);
	glEnable(GL_TEXTURE_2D);

	glFlush();

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// Add a set of graphics to the game.
//
//	ARGS:	pTiles,		A tile set object which will be used in the game.
//
//	POST:	m_vpTiles,	Contains tile set.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::addTiles(cTiles* pTiles)
{
	m_vpTiles.push_back(pTiles);	// Add tile object to game's collection.
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Add a sprite to a collection which will eventually be merged into the
//		active sprite list of the game.
//	-	New sprites are not directly added to the active sprite list because
//		this call to add may have occurred during the update loop.  Meaning,
//		adding a new sprite to the collection would invalidate the iterator
//		in the loop.
//
//	ARGS:	pSprite,	Sprite to add.
//
//	POST:	m_vpAddSprites,	Contains new sprite.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::addActiveSprite(cSprite* pSprite)
{ 
	m_vpAddSprites.push_back(pSprite);
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Checks if the boundries of a sprite overlap the boundries of any of
//		the current sprites marked as a specific type of tile.
//
//	ARGS:	pSprite,				Target sprite to check against.
//			eCollisionType,	What type of collisions to check against.
//
//	PRE:	m_vpSprites,	Collection of active sprites is populated.
//
//	RETURNS:	A pointer to a collection of sprites which are being intersected
//				by the target sprite.
//
//////////////////////////////////////////////////////////////////////////////
vector<cSprite*>* cGameState::getCollidingSprites(cSprite* pSprite, 
	eSPRITE_COLLIDE eCollisionType)
{
	// Clear any contents in the collection from previous calls.
	m_vpCollideSprites.clear();

	// Go through the list of sprites active for this screen
	for(SPRITE_ITR itr = m_vpSprites.begin(); itr != m_vpSprites.end(); itr++)
	{
		// Perform bit-wise comparison of the current sprite pointed to by the
		// iterator with the collision type passed in....
		if ( (*itr)->getCollisionMask() & eCollisionType)

			// ...a match is made so check the actual boundies of the sprite
			// pointed to be the iterator and see if it overlaps the boundry of 
			// the passed in sprite.
			if ( (*itr)->collide(pSprite) == true)
				m_vpCollideSprites.push_back(*itr);	// Add sprite to collection.
	}

	// Return pointer to collection of sprites which collided.
	return &m_vpCollideSprites;
}

//////////////////////////////////////////////////////////////////////////////
//
// -	Returns a list of sprites (from the active collection) which match the
//		type(s) passed in.
//
//	ARGS:	eCollisionType,	Type(s) of sprites to look for in collection.
//			
//	PRE:	m_vpSprites,	Collection of active sprites is populated.
//
//	RETURNS:	A pointer to a collection of sprites.
//
//////////////////////////////////////////////////////////////////////////////
vector<cSprite*>* cGameState::getSprites(eSPRITE_COLLIDE eCollisionType)
{
	// Clear any contents in the collection from previous calls.
	m_vpTypeSprites.clear();

	// Set sprite iterator to start of master sprite collection.
	SPRITE_ITR itr = m_vpSprites.begin();


	// Decide how to obtain sprites based on whether 0 is used or not.
	switch (eCollisionType)
	{
		// Bitwise comparison won't work for 0/default
		case 0:
			for(; itr != m_vpSprites.end(); itr++)
			{
				// Check for 0.
				if ( (*itr)->getCollisionMask() == 0)
					m_vpTypeSprites.push_back(*itr);		// Add sprite to collection.
			}
			break;

		// All sprites (even 0/default) are obtained.
		case 0xFF:
			for(; itr != m_vpSprites.end(); itr++)
				m_vpTypeSprites.push_back(*itr);			// Add sprite to collection.

			break;
		
		// Use bitwise comparison to match.
		default:
			for(; itr != m_vpSprites.end(); itr++)
			{
				// Perform bit-wise comparison of the current sprite in the collection
				// against the type we are looking for.
				if ( (*itr)->getCollisionMask() & eCollisionType)
					m_vpTypeSprites.push_back(*itr);		// Add sprite to collection.
			}
			break;
	}

	return &m_vpTypeSprites;	// Return pointer to collection.
}

//////////////////////////////////////////////////////////////////////////////
//
// -	Resets all members related to the onscreen message.
//		
//
//	POST:	m_vszMessages,	Collection of messages is cleared.
//			m_viMsgX,		Collection of locations is reset.
//			m_viMsgY,		Collection of locations is reset.
//
//////////////////////////////////////////////////////////////////////////////
void cGameState::clearMessages()
{
	for ( int iMessageCount = 0; 
			iMessageCount < 3; 
			iMessageCount++ )
	{
		m_vszMessages[ iMessageCount ] = "";
		m_viMsgX[ iMessageCount ] = 0;
		m_viMsgX[ iMessageCount ] = 0;
	}

}

///////////////////////////////////////////////////////////////////////////
//	ACCESSORS
///////////////////////////////////////////////////////////////////////////

// Actual size of the playfield.
/*inline*/ float cGameState::getPlayfieldWidth()		{ return m_fPlayfieldWidth;	}
/*inline*/ float cGameState::getPlayfieldHeight()	{ return m_fPlayfieldHeight;	}

// Playfield boundry information for sprites.
/*inline*/ float cGameState::getSpriteBoundryMinX() { return m_fSpriteBoundryMinX; }
/*inline*/ float cGameState::getSpriteBoundryMaxX() { return m_fSpriteBoundryMaxX; }
/*inline*/ float cGameState::getSpriteBoundryMinY() { return m_fSpriteBoundryMinY; }
/*inline*/ float cGameState::getSpriteBoundryMaxY() { return m_fSpriteBoundryMaxY; }

// Current screen coordinates.
/*inline*/ int cGameState::getCurrentScreenX(){	return m_iMapPositionX;	}
/*inline*/ int cGameState::getCurrentScreenY(){	return m_iMapPositionY;	}


// Return the time lapse.
/*inline*/ unsigned int cGameState::getTimeLapseMS(){ return m_dwTimeLapseMS; }

// The factor (mulitpler) by which the frame's contents are adjusted.
/*inline*/ float cGameState::getFrameFactor() { return m_fFrameFactor; }


// Return a set of tiles.
/*inline*/ cTiles* cGameState::getSpriteSetTiles(eSPRITE_SET eSpriteSet) { return m_vpTiles[eSpriteSet]; }

// Return the tiles for text writing
/*inline*/ cTiles* cGameState::getTextTiles() { return m_vpTiles[SPRITE_SET_MENU]; }

/*inline*/ cObjectLynk* cGameState::getLink() { return m_pLink; }
/*inline*/ cInventory* cGameState::getInventory() { return m_pInventory; }

// Get's and sets the state of the game.
/*inline*/ eSTATE_TYPE cGameState::getState()					{ return m_eState; }
/*inline*/ void cGameState::setState(eSTATE_TYPE eState)	{ m_eNextState = eState; }

// Obtains a copy of the sprites used in the screen.
/*inline*/ vector<cSprite*>* cGameState::getSprites() {
	return (&m_vpSprites); 
}

// Retrieve a pointer to a specific sprite.
/*inline*/ cSprite* cGameState::getSprite(int iSpriteID) {
	return m_vpSprites[ iSpriteID ]; 
}

// Retrieve a pointer to a specific event.
/*inline*/ cEvent* cGameState::getEvent( int iEventID ){ 
	return m_vpEvents[ iEventID ]; 
}

// Obtains pointer to the currently active map.
/*inline*/ cMap* cGameState::getMap() {
	return m_pCurrentMap; 
}

// Obtains a pointer to the map of choice.
/*inline*/ cMap* cGameState::getMap( int iMapID ) {
	return m_vpMaps[ iMapID ]; 
}
	

// Set the triggered portal for the portal state
/*inline*/ void cGameState::setTriggeredPortal( cEventPortal* pPortal ) { m_pTriggeredPortal = pPortal; }
/*inline*/ void cGameState::setPortalState( eSUBSTATE_PORTAL_TYPE ePortalState ) { m_ePortalState = ePortalState; }

// Let Link move around again
/*inline*/ void cGameState::unfreezeLink() { m_bFreezeLink = false; }

// Set the string to appear on screen
/*inline*/ void cGameState::setMessage( int iMessageNumber, string* pszMessage, int ix, int iy ) {
	m_vszMessages[ iMessageNumber ] = *(pszMessage);
	m_viMsgX[ iMessageNumber ] = ix;
	m_viMsgY[ iMessageNumber ] = iy;
}

// Get and set the # of hit points the player/Link has left.
/*inline*/ int	cGameState::getHitPoints() { return m_pLink->getHitPoints();}

// vi:set ts=3:
