//////////////////////////////////////////////////////////////////////////////
//
//	gameState.h
//	Defines for the cGameState class.
//
//	-	Holds the current state of the game.
//	-	Holds the set of sprites which are currently active in the game.
//	-	Collision detection is performed in this object, since it is the holder
//		of the sprites.
//	-	???TMH - Eventually for save games, this class will need to be persisted
//					to disk.  If a game cannot be reloaded from a combination of
//					the .MAP file and this class, then is that an indication that
//					some data is missing from here?  Maybe, but be cautious as this
//					class is pretty fat right now.
//	-	Tabs set to 3 spaces //
// 
//////////////////////////////////////////////////////////////////////////////


#ifndef __GAMESTATE_H__
#define __GAMESTATE_H__

#include "inc/system.h"
#include "inc/SpriteIDs.h"
#include "inc/EventIDs.h"
#include "inventory.h"
#include "window.h"

// forward defines (truly defined later)
class cTiles;
class cSprite;
class cScreen;
class cMap;
class cEvent;
class cEventPortal;
class cObjectLynk;
class cInventory;

// Define the keys that the user uses for interaction.
// ???TMH - Move this into some keyboard handling class & make configurable.
/*#define KEY_PLAYER_SWORD_ATTACK		SDLK_SPACE		// A button
#define KEY_PLAYER_ITEM_USE			SDLK_a			// B button
#define KEY_PLAYER_START				SDLK_c			// Start button
#define KEY_PLAYER_SELECT				SDLK_b			// Select button
#define KEY_PLAYER_MOVE_LEFT			SDLK_LEFT		// Left control pad
#define KEY_PLAYER_MOVE_RIGHT			SDLK_RIGHT		// Right control pad
#define KEY_PLAYER_MOVE_UP				SDLK_UP			// Up control pad
#define KEY_PLAYER_MOVE_DOWN			SDLK_DOWN		// Down control pad
*/

// Add states as they are supported (inventory, save game, title
//	screen, enter name screen, you get the gist....)
enum eSTATE_TYPE
{
	PLAY_GAME,				// Playing the game.
	SCROLL_GAME,			// Scrolling between screens (in game play).
	PORTAL_GAME,			// Jump to another screen (in game play).
	OPEN_INVENTORY,		// Opening the inventory page.
	CLOSE_INVENTORY		// Closing the inventory page.
};

// ???KWB - Added so that the various stages of the portal
//				process could be separated
enum eSUBSTATE_PORTAL_TYPE
{
	PORTAL_OFF = -1,			// Default position (there is no portal in progress)
	PORTAL_INIT,				// Initial pass into the portal state
	PORTAL_WALK_IN,			// Start entering the portal
	PORTAL_OPEN,				// Change maps
	PORTAL_WALK_OUT,			// Walk out the other end of the portal (into the cave/dungeon)
	PORTAL_END,					// End the portal state, clean up what needs it, and return to play mode
};


class cGameState  
{
	///////////////////////////////////////////////////////////////////////////
	//	MEMBERS
	///////////////////////////////////////////////////////////////////////////
private:
	cWindow*		m_pWindow;				// Hold object responsible for window.
	cMap*			m_pCurrentMap;			// Hold the current map being played.
	cScreen*		m_pCurrentScreen;		// Hold the currently displayed screen.
	uint32		m_dwLastTickCount;	// Holds last value from timing call.
	uint32		m_dwTimeLapseMS;		// Holds difference in last timing call.
	bool*		m_pbKeyStatus;			// Points to buffer off keyboard status.
	float		m_fFrameFactor;		// Multipler for frame animation.
	float		m_fScrollAmount;		// How much the screen is scrolled by.
	
	float		m_fPlayfieldWidth;	// Width of the game's play field.
	float		m_fPlayfieldHeight;	//	Height of the game's play field.
	float		m_fScreenWidth;		// Width of entire screen (fixed)
	float		m_fScreenHeight;		// Height of entire screen (fixed)
	
	float		m_fSpriteBoundryMinX;//	Playfield boundries for sprites...
	float		m_fSpriteBoundryMaxX;
	float		m_fSpriteBoundryMinY;
	float		m_fSpriteBoundryMaxY;

	int		m_iMapPositionX;		// X coordinate of screen in the current map.
	int		m_iMapPositionY;		// Y coordinate of screen in the current map.

	cEventPortal*				m_pTriggeredPortal;	// Keep track of the portal that was triggered
	eSUBSTATE_PORTAL_TYPE	m_ePortalState;		// Keep track of the portal substate
	cMap*							m_pLastMap;				// Last map Link was at
	int							m_iLastScreenX;		// X-coordinate of Last screen Link was at
	int							m_iLastScreenY;		// Y-coordinate of Last screen Link was at
	float							m_fLastX;				// X-coordinate of the last spot Link was at
	float							m_fLastY;				// Y-coordinate of the last spot Link was at
	eEVENT_TYPE					m_eLastPortalType;	// The type of portal Link last entered (so we know 
																// if he has to walk out of it)

	// ???KWB - This boolean is a bit of a hack so I can prevent Link from moving when
	//				I want for now.
	bool							m_bFreezeLink;			// Whether or not Link should be updated

	// ???KWB - Added to allow messages on screen, multiple lines
	string					m_vszMessages[3];		// Optional onscreen messages
	int							m_viMsgX[3];			// Messages X location
	int							m_viMsgY[3];			// Messages Y location

	eSTATE_TYPE m_eState;			// The "state" of the game.
	eSTATE_TYPE m_eNextState;		// The "state" the game is about to go in.


	vector<cSprite*> m_vpSprites;		// Sprites for current screen.	
	vector<cSprite*> m_vpOldSprites;	// Sprites scrolling off of screen.
	vector<cSprite*> m_vpAddSprites;	// Sprites about to be active.
	
	vector<cTiles*>  m_vpTiles;			// Holds sets of graphics.

	// Hold a collection of sprites which intersect some target sprite after
	// a call to getCollidingSprites() is made.
	vector<cSprite*>	m_vpCollideSprites;

	// Hold a collection of sprites that are populated when a call to
	// getSprites() is made.
	vector<cSprite*>	m_vpTypeSprites;

	vector<cMap*>	m_vpMaps;		// Holds all the maps in the game.
	vector<cEvent*> m_vpEvents;		// Events for current screen.

	cInventory*		m_pInventory;	// Player's inventory.
	cObjectLynk*		m_pLink;			// Main player object.


public:
	eDIRECTION	m_eScrollDirection;	// Direction of screen scrolling.

	// dreamcast controller hack FIXME
	cont_cond_t cond;	// controller state
	uint8 controller1;	// controller #1

	///////////////////////////////////////////////////////////////////////////
	//	METHODS
	///////////////////////////////////////////////////////////////////////////
private:
	unsigned int computeTimeLapseMS();	// Determine MSs since last call.	
	void computeFrameFactor();	// Determine multipler to use for updates.

public:
	cGameState();				// Constructor.
	virtual ~cGameState();	// Destructor.

	void	initialize(cWindow *pWindow);			// Initialize the game state.
	void	terminate();								//	Shut down the game.

	void	setMap(cMap *pMap, int x=-1, int y=-1);	// Set the current map.
	void	setScreen();										// Set the current screen.

	void	update();					// Update screen's content.	
	void	updatePlayGame();			// Update during normal game play.
	void	updateScrollGame();		// Update during scroll.
	void	updatePortalGame();		// Update during portal.
	void	draw();						// Draw screen's content.

	// Updates a collection of sprites.
	void	updateSprites(vector<cSprite*>& vpSpriteList);

	// Handle active and triggered events
	void handleEvents(vector<cEvent*>& vpEventList );

	// Add a set of graphics to the game.
	void addTiles(cTiles* pTiles);

	// Inject object into the active sprite list.
	void addActiveSprite(cSprite* pSprite);

	
	// Builds a collection of sprites (of a specific type) who's boundries
	// overlap the boundries of a target sprite.
	vector<cSprite*>* getCollidingSprites(cSprite* pSprite, 
		eSPRITE_COLLIDE eCollisionType);

	// Returns a list of sprites (from the active collection) which match the
	// collision type(s) passed in.
	vector<cSprite*>* getSprites(eSPRITE_COLLIDE eCollisionType);

	// Clear all messages and locations
	void clearMessages();

	///////////////////////////////////////////////////////////////////////////
	//	ACCESSORS
	///////////////////////////////////////////////////////////////////////////

	// Actual size of the playfield.
	/*inline*/ float getPlayfieldWidth();
	/*inline*/ float getPlayfieldHeight();

	// Playfield boundry information for sprites.
	/*inline*/ float getSpriteBoundryMinX();
	/*inline*/ float getSpriteBoundryMaxX();
	/*inline*/ float getSpriteBoundryMinY();
	/*inline*/ float getSpriteBoundryMaxY();

	// Current screen coordinates.
	/*inline*/ int getCurrentScreenX();
	/*inline*/ int getCurrentScreenY();


	// Return the time lapse.
	/*inline*/ unsigned int getTimeLapseMS();

	// The factor (mulitpler) by which the frame's contents are adjusted.
	/*inline*/ float getFrameFactor();

	
	// Return a set of tiles.
	/*inline*/ cTiles* getSpriteSetTiles(eSPRITE_SET eSpriteSet);

	// Return the tiles for text writing
	/*inline*/ cTiles* getTextTiles();

	/*inline*/ cObjectLynk* getLink();
	/*inline*/ cInventory* getInventory();

	// Get's and sets the state of the game.
	/*inline*/ eSTATE_TYPE getState();			
	/*inline*/ void setState(eSTATE_TYPE eState);

	// Obtains a copy of the sprites used in the screen.
	/*inline*/ vector<cSprite*>* getSprites();

	// Retrieve a pointer to a specific sprite.
	/*inline*/ cSprite* getSprite(int iSpriteID);

	// Retrieve a pointer to a specific event.
	/*inline*/ cEvent* getEvent( int iEventID );

	// Obtains pointer to the currently active map.
	/*inline*/ cMap* getMap();

	// Obtains a pointer to the map of choice.
	/*inline*/ cMap* getMap( int iMapID );
	

	// Get and set the # of hit points the player/Link has left.
	/*inline*/ int	getHitPoints();
	
	// Set the triggered portal for the portal state
	/*inline*/ void setTriggeredPortal( cEventPortal* pPortal ); 
	/*inline*/ void setPortalState( eSUBSTATE_PORTAL_TYPE ePortalState );

	// Let Link move around again
	/*inline*/ void unfreezeLink();

	// Set the string to appear on screen
	/*inline*/ void setMessage( int iMessageNumber, string* pszMessage, int ix, int iy );
};

#endif // __GAMESTATE_H__
// vi:set ts=3:
