//////////////////////////////////////////////////////////////////////////////
//
//	inventory.cpp
//	Implementation for the cInventory class.
//
//  - The main character of the game.
//	 -	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

/*
*/

#include "inventory.h"          // Defines for this class.
#include "GameException.h"      // Exception handling.
#include "objectCandle.h"        //  The candle weapon, that shoots fire.

// Defines for forward delcarations.
#include "tiles.h"
#include "objectLynk.h"
#include "gameState.h"

#include "objectInventoryItem.h"	// Base class for inventory items.
#include "objectBoomerang.h"		// Boomerang.
#include "objectMagicalRod.h"		// Magical rod.
#include "objectStillSword.h"		// Link's still sword.
#include "objectBow.h"				// Bow.

//////////////////////////////////////////////////////////////////////////////
//
// CONSTRUCTOR
//
//	ARGS:	pGameState,		Points to game state object.
//
//////////////////////////////////////////////////////////////////////////////
cInventory::cInventory(cGameState* pGameState)
{
	m_pGameState = pGameState;		// Set the game state object.

	// Get graphics used to draw menu/inventory information.
	m_pMenuTiles = pGameState->getSpriteSetTiles(SPRITE_SET_MENU);

	// Get graphics used to draw weapons and other items.
	m_pItemTiles = pGameState->getSpriteSetTiles(SPRITE_SET_ITEMS);

	// Allocate memory for Link's still sword...
	m_pSword = new cObjectStillSword(m_pGameState);

	// Allocate memory and add Link's other items.
	m_vItems.push_back(new cObjectBoomerang(m_pGameState));		// Boomerang
	m_vItems.push_back(new cObjectBow(m_pGameState));				// Bow
	m_vItems.push_back(new cObjectCandle(m_pGameState));			// Candle.
	m_vItems.push_back(new cObjectMagicalRod(m_pGameState));		// Rod.

	// Set selected item to the beginning of the bag o' items.
	m_itrCurrentItem = m_vItems.begin();

	// Set Link's # of heart containers and how much they hold.
	m_iHeartContainers	= 10;	// ???TMH - Change to 3, high # for debugging.
	m_iHeartsPerContainer= 2;
	m_iMaximumHitPoints = m_iHeartContainers*m_iHeartsPerContainer;

	m_bMagicKey = false;		// Link doesn't start with the magic key.

	// ???TMH - Start off with some bogus values for debugging purposes.
	m_iRupees	= 15;
	m_iKeys		= 5;
	m_iBombs		= 10;

}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cInventory::~cInventory()
{
	// Free all items in inventory vector.
	for (vector<cObjectInventoryItem*>::iterator itrItems = m_vItems.begin();
		itrItems != m_vItems.end();
		itrItems++)
		delete (*itrItems);

	// Free sword.
	delete m_pSword;
}


//////////////////////////////////////////////////////////////////////////////
// 
//	Update the weapons.
//
//////////////////////////////////////////////////////////////////////////////
void cInventory::update()
{	
	BEGIN_ERRORHANDLER_GAME

	// Inventory updates based on the game state.
	switch(m_pGameState->getState())
	{
		case PLAY_GAME:
			m_pSword->update();
			break;
		
		case SCROLL_GAME:			// Fall through...
		case PORTAL_GAME:
			// ???TMH - This will cause reset to be repeatedly call until scroll,
			//				or portal complete.  This should be changed to only
			//				be performed once, and then not allow any weapons to
			//				start until in a PLAY_GAME state.
			getSword()->reset();
			getCurrentItem()->reset();
			break;

		default:
		{
			// Unknown game state, make an error message and throw.		
			THROW_GAME_INT("Inventory attempted an update while containing an "
				"unknown state value.", m_pGameState->getState() );
		}
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Draws the inventory at the top of the screen during game play.
//	-	???TMH - Add conditional logic to display complete inventories (one
//					type for overworld, one for underworld) when selected by the
//					user during game play.
//
//////////////////////////////////////////////////////////////////////////////
void cInventory::draw()
{

	// Load identity so world matrix is reset if scrolling.  Otherwise the
	// inventory will scroll with the screen.
	glLoadIdentity();

	// Turn off texture mapping and set color to black.
	glDisable(GL_TEXTURE_2D);
	glColor3f(0.0f, 0.0f, 0.0f);

	// Draw a square which acts as the black background for the inventory.
	glBegin(GL_QUADS);
		glVertex3f(0.0f,		224.0f, Z_PLANE_CEILING - 0.01f);
		glVertex3f(256.0f,	224.0f, Z_PLANE_CEILING - 0.01f);
		glVertex3f(256.0f,	176.0f, Z_PLANE_CEILING - 0.01f);
		glVertex3f(0.0f,		176.0f, Z_PLANE_CEILING - 0.01f);
	glEnd();

	// Now that the background has been painted, turn on texturing again and
	// make sure the color is full brightness.
	glEnable(GL_TEXTURE_2D);
	glColor3f(1.0f, 1.0f, 1.0f);

	// XXX DREAMCAST HACK
	// depth buffer is slightly different
	glTranslatef(0.0f, 0.0f, -0.1f);

	// Draw out the inventory, left to right...


	// ----- Map -----
	//	???TMH - Add logic to adjust map location for non-overworld maps.

	// Draw background grid.
	for (int iMapY = 0; iMapY < 4; iMapY++)
	{
		for ( int iMapX = 0; iMapX < 8; iMapX++)
		{
			// Draw blank map tile.
			m_pMenuTiles->draw(16.0f + (8.0f * iMapX), 184.0f + (8.0f * iMapY),
				Z_PLANE_CEILING, 0x2F);
		}
	}

	// Draw Link's position on the grid.
	// There are four 8x8 tiles representing one of four positions Link can
	// be at.  So not only does the map location need to be calculated, but
	// which tile to display needs to be determined too.

	// Used to determine if there is an offset for the sprite to use.
	float fHorizontalOffset	= 0.0f;
	float fVerticalOffset	= 0.0f;


	// If a screen coordinate is odd, add one to offset.
	if (m_pGameState->getCurrentScreenX() & 0x0001)
		fHorizontalOffset = 1.0f;
	if (m_pGameState->getCurrentScreenY() & 0x0001)
		fVerticalOffset = 1.0f;

	// Draw one of the four tiles in the appropriate location
	m_pMenuTiles->draw(
		16.0f + (4.0f * (m_pGameState->getCurrentScreenX() - fHorizontalOffset)),
		208.0f- (4.0f * (m_pGameState->getCurrentScreenY() - fVerticalOffset)),
		Z_PLANE_CEILING + 0.01f,
		0x30 + fHorizontalOffset + (fVerticalOffset * 2.0f));



	// ----- Rupies, Keys, and Bombs -----
	
	char szValue[5];	// Value of an inventory item being displayed.

	// Draw rupee, X, and amount.
	m_pMenuTiles->drawString( "$x", 88, 208 );
	sprintf(szValue, "%i", m_iRupees);
	m_pMenuTiles->drawString( szValue, 104, 208 );
	
	// Draw key, X, and amount.
	m_pMenuTiles->drawString( "%x", 88, 192 );
	sprintf(szValue, "%i", m_iKeys);
	m_pMenuTiles->drawString( szValue, 104, 192 );
	
	// Draw bomb, X, and amount.
	m_pMenuTiles->drawString( "@x", 88, 184 );
	sprintf(szValue, "%i", m_iBombs);
	m_pMenuTiles->drawString( szValue, 104, 184 );

	// ???KWB - test text string, with changing color
	//m_pMenuTiles->drawString( "THIS IS #2EASY! #1LOOK, #3INSTA-DEBUG #0CODE.", 20, 100 );

	// ----- "B" item and "A" item -----

	// Draw "B" container.
	m_pMenuTiles->draw(120, 208, Z_PLANE_CEILING, 0x34);	// top
	m_pMenuTiles->draw(128, 208, Z_PLANE_CEILING, 0x0B);
	m_pMenuTiles->draw(136, 208, Z_PLANE_CEILING, 0x36);
	m_pMenuTiles->draw(120, 200, Z_PLANE_CEILING, 0x37);  // middle 1	
	m_pMenuTiles->draw(136, 200, Z_PLANE_CEILING, 0x37);
	m_pMenuTiles->draw(120, 192, Z_PLANE_CEILING, 0x37);	// middle 2
	m_pMenuTiles->draw(136, 192, Z_PLANE_CEILING, 0x37);
	m_pMenuTiles->draw(120, 184, Z_PLANE_CEILING, 0x38);	// bottom
	m_pMenuTiles->draw(128, 184, Z_PLANE_CEILING, 0x35);
	m_pMenuTiles->draw(136, 184, Z_PLANE_CEILING, 0x39);

	// ???TMH - Add selected B item

	// Draw "A" container.
	m_pMenuTiles->draw(144, 208, Z_PLANE_CEILING, 0x34);	// top
	m_pMenuTiles->draw(152, 208, Z_PLANE_CEILING, 0x0A);
	m_pMenuTiles->draw(160, 208, Z_PLANE_CEILING, 0x36);
	m_pMenuTiles->draw(144, 200, Z_PLANE_CEILING, 0x37);  // middle 1
	m_pMenuTiles->draw(160, 200, Z_PLANE_CEILING, 0x37);
	m_pMenuTiles->draw(144, 192, Z_PLANE_CEILING, 0x37);	// middle 2
	m_pMenuTiles->draw(160, 192, Z_PLANE_CEILING, 0x37);
	m_pMenuTiles->draw(144, 184, Z_PLANE_CEILING, 0x38);	// bottom
	m_pMenuTiles->draw(152, 184, Z_PLANE_CEILING, 0x35);
	m_pMenuTiles->draw(160, 184, Z_PLANE_CEILING, 0x39);	
	

	// Item in "B"
	m_pItemTiles->draw( 124, 192, Z_PLANE_CEILING,
		getCurrentItem()->getInventoryDisplayID(),
		getCurrentItem()->getInventoryDisplayColor() );

	// Item in "A".
	m_pItemTiles->draw( 148, 192, Z_PLANE_CEILING, 
		getSword()->getInventoryDisplayID(),
		getSword()->getInventoryDisplayColor() );


	// ----- Life (hearts) -----

	// Draw text above hearts.
	m_pMenuTiles->draw(184, 208, Z_PLANE_CEILING, 0xFB);	// -
	m_pMenuTiles->draw(192, 208, Z_PLANE_CEILING, 0xE5);	// L
	m_pMenuTiles->draw(200, 208, Z_PLANE_CEILING, 0xE2);	// I
	m_pMenuTiles->draw(208, 208, Z_PLANE_CEILING, 0xDF);	// F
	m_pMenuTiles->draw(216, 208, Z_PLANE_CEILING, 0xDE);	// E
	m_pMenuTiles->draw(224, 208, Z_PLANE_CEILING, 0xFB);	// -


	// Draw hearts based on # of heart containers.
	for(int iHeartLoop = 0; iHeartLoop < m_iHeartContainers; iHeartLoop++)
	{

		if ( m_pGameState->getHitPoints() <
			((iHeartLoop * m_iHeartsPerContainer) + 1) )
		{
			// Draw an empty heart.
			m_pItemTiles->draw(
				171 + ( 8 * ( iHeartLoop % 8 ) ),
				184 + ( 8 * ( iHeartLoop / 8 ) ),
				Z_PLANE_CEILING, (ID_ITEM_HEART + 2), COLOR_SET_ITEMS_RED);
		
		} else {
  
			// Check if a half heart needs to be drawn.
			if (	m_pGameState->getHitPoints() <=
					((m_iHeartsPerContainer * iHeartLoop) +
						(m_iHeartsPerContainer / 2)) )
			{
				// Draw half heart.
				m_pItemTiles->draw(
					171 + ( 8 * ( iHeartLoop % 8 ) ),
					184 + ( 8 * ( iHeartLoop / 8 ) ),
					Z_PLANE_CEILING, (ID_ITEM_HEART + 1), COLOR_SET_ITEMS_RED);
			
			} else {

				// Draw full heart.
				m_pItemTiles->draw(
					171 + ( 8 * ( iHeartLoop % 8 ) ),
					184 + ( 8 * ( iHeartLoop / 8 ) ),
					Z_PLANE_CEILING, (ID_ITEM_HEART), COLOR_SET_ITEMS_RED);
			}
		}

	} // END loop hearts
}


//////////////////////////////////////////////////////////////////////////////
//
//	Cycle forward through the Inventory Items.
//
//	PRE:	m_itrCurrentItem,	Set to an inventory item.
//
//	POST:	m_itrCurrentItem,	Set to next item in inventory.
//
//////////////////////////////////////////////////////////////////////////////
void cInventory::nextItem()
{
	// Increment, then if at end ( which is really one past the
	//	end of the entries ), go to the beginning.
	if ( ++m_itrCurrentItem == m_vItems.end() )
	{
		m_itrCurrentItem = m_vItems.begin();
	}
}


/////////////////////////////////////////////////////////////////////////////
//
//	Cycle backward through the Inventory Items.
//
//	PRE:	m_itrCurrentItem,	Set to an inventory item.
//
//	POST:	m_itrCurrentItem,	Set to previous inventory item.
//
/////////////////////////////////////////////////////////////////////////////
void cInventory::previousItem()
{
	// If at the beginning, go to the end ( one past the actual last entry )
	// then decrement no matter what
	if ( m_itrCurrentItem == m_vItems.begin() )
	{
		m_itrCurrentItem = m_vItems.end();
	}
	m_itrCurrentItem--;
}


//////////////////////////////////////////////////////////////////////////////
//
//	Reset all the Items
//
//////////////////////////////////////////////////////////////////////////////
void cInventory::reset()
{
	// Reset Sword
	m_pSword->reset();

	// Reset all the Inventory Items
	for( vector<cObjectInventoryItem*>::iterator itrItems = m_vItems.begin();
		itrItems != m_vItems.end();
		itrItems++ )
	{
		(*itrItems)->reset();
	}
}


//////////////////////////////////////////////////////////////////////////////
//
//	Try to spend a rupee
//
//	RETURNS:	true,		If a rupee was available and subtracted.
//				false,	If Link is broke.
//
//////////////////////////////////////////////////////////////////////////////
bool cInventory::spendRupee()
{
	// If subracting Rupee keeps Link with some spare change...
	if( m_iRupees-- > 0 )
		return true;				// ... note he still has cash on him.
	
	m_iRupees = 0;		// If value fell below 0, snap back to 0.

	return false;		// Link is in the poor house!
}
// vi:set ts=3:
