//////////////////////////////////////////////////////////////////////////////
//
//	map.cpp
//	Implementation for the cMap class.
//
//	-	Holds a grid of cSreens to make a "map".
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "map.h"                // Defines for this class.
#include "GameException.h"  // Exception handling.

// Defines for forward delcarations.
#include "tiles.h"
#include "screen.h"
#include "screenTemplate.h"
#include "gameState.h"

//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
//	-	Loads a map file and constructs screens contained in map.
//
//		Offset	Size	Info
//		------	----	-------------------------------------------------
//		0			4		The string "MAP" including 0x00 at end.
//		4			1		The (unique) ID of the map file.
//		5			1		Width of the map in screens.
//		6			1		Height of the map in screens.
//		7			1		# of templates in map (0 if none).
//		8			1     The X value of the starting screen.
//		9			1		The Y value of the starting screen.
//		10			1		Link's starting horizontal (tile) position.
//		11			1		Link's starting verticle (tile) position.
//		12			1		# of strings (0 if none)
//		13			3		Reservered for future, so all bytes set to 0x00
//
//	ARGS:	pGameState,	An object containing the "state" of the game.
//			szMapname,	The (file)name of the map to load.
//
//	POST:	-	Map's ID is added to the list of IDs in the gamestate.
//			-	Screens within map are constructed.
//
//	THROWS:	cGameException,	On an error reading map file.
//
//////////////////////////////////////////////////////////////////////////////
cMap::cMap(cGameState* pGameState, char szMapname[MAX_MAP_NAME]) :
	m_yID(0), m_yWidth(0), m_yHeight(0), m_yTemplates(0),
	m_yStartingScreenX(0), m_yStartingScreenY(0),
	m_yStartingTileX(0), m_yStartingTileY(0),
	m_yNumberOfStrings(0)
{

	// Use a traditional try/catch block (instead of the macros), so if any 
	// errors are thrown the catch block can check if a file handle is left
	// open, and close it.
	//try
	{
		char		szMapHead[16];	// Buffer to check the .map file header.
		cScreenTemplate* pTemplate;	// Temporary to construct template objects
		cScreen*	pScreen;			// Temporary to construct screen objects.

		// The first 4 bytes of a file which is a Map file.
		const char	szMapHeaderID[] = { 'M', 'A', 'P', 0x00 };

		m_pFileHandle = NULL;				// Reset file handle to nothing.		


		// Make sure the pointer to a game state is legal.
		if (pGameState == NULL)
		{
			// Bad pointer.  Throw with an appropriate error message.
			//THROW_GAME("NULL game state pointer passed into map constructor.");
			printf(">> NULL game state pointer passed into map constructor.");
			exit(-1);
		}

		// Build path to map data.
		string sFileName("/rd/");
		sFileName += szMapname;
		sFileName += ".map";

		printf(">> attempting to load map '%s'\n", sFileName.c_str() );
	
		m_pFileHandle = fopen(sFileName.c_str(), "rb");		// Open map file.

		// Attempt to read header from file.
		if(!m_pFileHandle)
		{
			// Unable to get a file handle.  Construct and throw error message.
			//string sErrorMessage ("cMap was unable to open the file: ");
			//sErrorMessage += sFileName;
			//THROW_GAME(sErrorMessage.c_str());
			//THROW_GAME("unable to open the map file!");
			printf(">>  FAILURE opening map!\n");
			exit(-1);
		}

		// Read in header and compare with the ID fields.
		if(
			(fread(&szMapHead, 1, sizeof(szMapHead), m_pFileHandle) != sizeof(szMapHead)) ||
			(strncmp(szMapHead, szMapHeaderID, sizeof(szMapHeaderID)) != 0) )
		{
			// Header did not match (or failed read) so error.
			//string sErrorMessage("An invalid map header was read in the file: ");
			//sErrorMessage += sFileName;
			printf(">>  invalid map header!\n");
			exit(-1);
		}


		// Fill in member variables from the blob of map header data.
		m_yID					= szMapHead[4];	// The ID of this map.
		m_yWidth				= szMapHead[5];	// Width of entire map (in screens).
		m_yHeight			= szMapHead[6];	// Height of entire map (in screens).
		m_yTemplates		= szMapHead[7];	// # of templates in map file.
		m_yStartingScreenX= szMapHead[8];	// Starting screen X coordinate.
		m_yStartingScreenY= szMapHead[9];	// Starting screen Y coordinate.
		m_yStartingTileX	= szMapHead[10];	// Starting tile X coordinate.
		m_yStartingTileY	= szMapHead[11];	// Starting tile Y coordinate.
		m_yNumberOfStrings= szMapHead[12];	// Number of strings.

		// ???TMH - Eventually remove this check.  It looks for a common
		//				footprint in all map files which are of the previous format.
		if (szMapHead[8]==0x10 && szMapHead[9]==0x0b && szMapHead[10]==00)
		{
			// Header did not match (or failed read) so error.
			//string sErrorMessage("Old school .MAP with 8 byte header "
			//	"attempted to be read from file: ");
			//sErrorMessage += sFileName;
			//THROW_GAME(sErrorMessage.c_str());
			printf(">>   Old map file found!\n");
			exit(-1);
		}
		
		// Read in templates
		for( int iTemplate = 0; iTemplate < m_yTemplates; iTemplate++ ) 
		{
			// Create a new screen template object, passing it the open file 
			// handle.  It's up to the screen template object to know how to 
			// read it's contents from the file.
			pTemplate = new cScreenTemplate(pGameState, m_pFileHandle);

			// Copy the contents of the newly created screen template object 
			// into the map's vector of templates.
			m_vpTemplates.push_back(pTemplate);
		}

		// Read in strings if necessary
		for ( int iString = 0; iString < m_yNumberOfStrings; iString++ )
		{
			char szSize;
			char szBuffer[64];

			fread( &szSize, 1, sizeof(szSize), m_pFileHandle);

			fread( szBuffer, 1, szSize, m_pFileHandle);
			string* szMessage = new string(szBuffer);

			m_vpszMessages.push_back(szMessage);
		}

		// Read in screen of data.  It's stored in a X*Y grid.
		for(BYTE y = 0; y < m_yHeight; y++ )
		{
			for(BYTE x = 0; x < m_yWidth; x++ )
			{
				// Create a new screen object, passing it the open file handle.
				// It's up to the screen object to know how to read it's contents
				// from the file.
				// ???KWB - Added map pointer so the screen can grab a template if it needs to
				pScreen = new cScreen(pGameState, this, m_pFileHandle);

				// Copy the contents of the newly created screen object into the
				// map's collection of screens, based on it's coordinates within
				// the map.
				m_mpScreens[ COORDINATES(x, y) ] = pScreen;
			}
		}


		fclose(m_pFileHandle);	// Close file.
		m_pFileHandle = NULL;	// Force handle to point to an invalid value in
										// case it's attempted to be accessed
										// (accidently) in the future.
	}
	// NOT IN DREAMCAST
	/*catch(...)
	{
		// If any error occurs make sure to close the file handle (if open).
		if (m_pFileHandle != NULL)
			fclose(m_pFileHandle), m_pFileHandle = NULL;

		throw;	// Re-throw error to a higher handler...
	}*/

}


//////////////////////////////////////////////////////////////////////////////
//
//	DESTRUCTOR
//
//	POST:	m_mpScreen,	Memory from screens is freed, and collection emptied.
//
//////////////////////////////////////////////////////////////////////////////
cMap::~cMap()
{
	// Free the memory of each screen in the collection.
	for(SCREENS_ITR itrScreen = m_mpScreens.begin();	
		itrScreen != m_mpScreens.end();
		itrScreen++)
		delete itrScreen->second;
	
	m_mpScreens.clear();		// Wipe clean the contents of the collection.

	// Freem the memory of each screen template in the vector
	for( TEMPLATE_ITR itrTemplate = m_vpTemplates.begin();
		itrTemplate != m_vpTemplates.end();
		itrTemplate++ )
		delete *itrTemplate;
}

// ACCESSORS
// Extract a pointer to screen object, based on x,y coordinates in game map.
/*inline*/ cScreen* cMap::getScreen(char x, char y)
{ return m_mpScreens[ pair<unsigned char,unsigned char>(x,y) ]; }

// Extract a pointer to a screen template object
/*inline*/ cScreenTemplate* cMap::getTemplate( int iTemplateID )
{ return m_vpTemplates[ iTemplateID ]; }

// Dimensions of the map.
/*inline*/ BYTE cMap::getWidth()	{ return m_yWidth;	}
/*inline*/ BYTE cMap::getHeight() { return m_yHeight;	}

// Starting position on map (and screen).
/*inline*/ BYTE cMap::getStartScreenX()	{ return m_yStartingScreenX;	}
/*inline*/ BYTE cMap::getStartScreenY()	{ return m_yStartingScreenY;	}
/*inline*/ BYTE cMap::getStartTileX()	{ return m_yStartingTileX;		}
/*inline*/ BYTE cMap::getStartTileY()	{ return m_yStartingTileY;		}
	
// Number of strings in map.
/*inline*/ BYTE cMap::getNumberOfStrings()	{ return m_yNumberOfStrings; }

// Extract a pointer to a specific string in a map
/*inline*/ string* cMap::getString( int iStringID ) {
	return m_vpszMessages[ iStringID ]; 
}

// vi:set ts=3:
