//////////////////////////////////////////////////////////////////////////////
//
//	map.h
//	Defines for the cMap class.
//
//
//	-	Each map is initialized from reading a .map file.  This file provides
//		the dimensions of the map, the data for screen objects, and a unique
//		identifier for this object.  (This ID is used to "link" maps up to
//		each other in the game.)
//
//	-	A map object also may contains palettes, which are a pairs of lists
//		containing RGB values.  The source values are converted to the target
//		values.  This is to allow the same graphics to be utilized with
//		different colors during game play.
//
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#ifndef __MAP_H__
#define __MAP_H__

// Turn off 4786: "identifier was truncated to '255' characters..."
#ifdef WIN32
#pragma warning(disable:4786)
#endif

#include "inc/system.h"
#include <map>
#include "ColorRGB.h"           // Color and BYTE support.
#include "ColorRGBPalette.h"           // Color and BYTE support.

// Forward declares (truly defined later).
class cTiles;
class cScreen;
class cScreenTemplate;
class cGameState;

// The maximum size a map (file)name can be.
const int MAX_MAP_NAME = 32;

//	A pair made for hold two (x,y) coordinate.
typedef pair<BYTE ,BYTE>	COORDINATES;

// Type of screens within a map.
typedef map<COORDINATES, cScreen*> SCREENS_TYPE;

// STL iterator for looping through collection of screen.
typedef SCREENS_TYPE::iterator SCREENS_ITR;

class cMap  
{
	///////////////////////////////////////////////////////////////////////////
	//	MEMBERS
	///////////////////////////////////////////////////////////////////////////
protected:
	FILE*				m_pFileHandle;	// Handle for reading from files.
	SCREENS_TYPE	m_mpScreens;	// Screens inside of the map.

	BYTE m_yID;						// Unique map ID of this object.
	BYTE m_yWidth;					// Number of screens in X direction in map.
	BYTE m_yHeight;				// Number of screens in Y direction in map.
	BYTE m_yTemplates;			// Number of templates in map file.
	BYTE m_yStartingScreenX;	// The default horizontal screen to start on.
	BYTE m_yStartingScreenY;	// The default verticle screen to start on.
	BYTE m_yStartingTileX;		// The default horizontal tile to start on.
	BYTE m_yStartingTileY;		// The default verticle tile to start on.
	BYTE m_yNumberOfStrings;	// Number of strings in map file.

	// Templates for the current map.
	vector<cScreenTemplate*>	m_vpTemplates;

	// Strings for the current map
	vector<string*> m_vpszMessages;


	///////////////////////////////////////////////////////////////////////////
	//	METHODS
	///////////////////////////////////////////////////////////////////////////
protected:
	cMap(){}	// Default constructor non-creatable.


public:
	// Constructor takes file name and tile set.
	cMap(cGameState* pGameState, char szMapname[MAX_MAP_NAME]);

	// Destructor
	virtual ~cMap();	


	///////////////////////////////////////////////////////////////////////////
	//	ACCESSORS
	///////////////////////////////////////////////////////////////////////////

	// Extract a pointer to screen object, based on x,y coordinates in
	//	game map.
	/*inline*/ cScreen* getScreen(char x, char y);

	// Extract a pointer to a screen template object
	/*inline*/ cScreenTemplate* getTemplate( int iTemplateID );

	// Dimensions of the map.
	/*inline*/ BYTE getWidth();
	/*inline*/ BYTE getHeight();

	// Starting position on map (and screen).
	/*inline*/ BYTE getStartScreenX();
	/*inline*/ BYTE getStartScreenY();
	/*inline*/ BYTE getStartTileX();
	/*inline*/ BYTE getStartTileY();
	
	// Number of strings in map.
	/*inline*/ BYTE getNumberOfStrings();

	// Extract a pointer to a specific string in a map
	/*inline*/ string* getString( int iStringID );
};

// STL iterator for looping through collection of maps.
typedef vector<cMap*>::iterator MAP_ITR;

#endif // __MAP_H__
// vi:set ts=3:
