//////////////////////////////////////////////////////////////////////////////
//
//	OBJECTBoomerang.cpp
//	Implementation for the cObjectBoomerang class.
//
//	-	Encapsulation for the boomerang Link uses.
//	-	Tabs set to 3 spaces.
//	-  This is actually now an 'AX'
// 
//////////////////////////////////////////////////////////////////////////////

#include "objectBoomerang.h"

//////////////////////////////////////////////////////////////////////////////
//	CONSTANTS
//////////////////////////////////////////////////////////////////////////////

// ???DAC - a really slow cool boomerang
//const float fBOOMERANG_TOSS_SPEED		= 6.8f;
//const float fBOOMERANG_MAX_RETURN_SPEED= 0.8f;
//const float fBOOMERANG_ACCELERATION		= 0.25f;
//const long	lBOOMERANG_SWING_DELAY		= 100;
//const long	lBOOMERANG_ANIMATION_DELAY = 12;

const float fBOOMERANG_TOSS_SPEED       = 4.8f;
const float fBOOMERANG_MAX_RETURN_SPEED= 1.8f;
const float fBOOMERANG_ACCELERATION     = 0.25f;

const long  lBOOMERANG_SWING_DELAY      = 100;
const long  lBOOMERANG_ANIMATION_DELAY = 24;

//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
// -	Construct a boomerang
//		
//	ARGS:	pGameState,		Game state object.
//
//////////////////////////////////////////////////////////////////////////////
cObjectBoomerang::cObjectBoomerang(cGameState* pGameState)		
{
	// Set game state object.
	m_pGameState = pGameState;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(SPRITE_SET_ITEMS);

	// Don't auto-delete when boomerang "dies".
	m_bDeleteable = false;

	// Set defaults.
	cSprite::reset();

	// Link starts with a green boomerang
	m_yColorSet = m_yColorSetBeforeFlashing = COLOR_SET_ITEMS_GREEN;

	// Override defaults...
	m_iDamage = 1;			// Amount of damage sprite inflicts.
	m_iHitPoints = 0;		// Boomerang starts off "dead".
	m_iAnimateOffset = 0;

	m_eCollisionMask = static_cast<eSPRITE_COLLIDE>
		(SPRITE_COLLIDE_PLAYER | SPRITE_COLLIDE_SHOT | SPRITE_COLLIDE_ENEMY);

	// Set the inventory display ID of the boomerang
	m_iInventoryDisplayID = ID_ITEM_BOOMERANG;
	// set swing delay for Link
	m_lSwingDelay = lBOOMERANG_SWING_DELAY;

	// Dimensions of boomerang.
	m_fWidth	= m_pTiles->getWidth();
	m_fHeight	= m_pTiles->getHeight();

	// Border around sprite used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectBoomerang::~cObjectBoomerang()
{
}


//////////////////////////////////////////////////////////////////////////////
//
//	Override default implementation to call an enemy's hit.
//
//	ARGS:	pSprite,	The sprite which may be colliding with the boomerang.
//
//	POST:	If pSprite is an enemy and it collides then hit() is called on it.
//
//	RETURNS:	true,		Target sprite is an enemy and overlaps this sprite.
//				false,	boomerang is not interacting with an enemy and/or they are
//							not colliding with each other.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectBoomerang::collide(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// Make sure target sprite is an enemy, then check if it's colliding.
	if (	(pSprite->getCollisionMask() & SPRITE_COLLIDE_ENEMY) &&
			cSprite::collide(pSprite) )
	{
		pSprite->hit(this);	// Hit the enemy sprite.

		// If the boomerang was on it's way out, it bounces.
		if ( m_eCurrentBehavior == BOOMERANG_OUTGOING )
		{
			m_eCurrentBehavior = BOOMERANG_INCOMING;
		}
		
		return true;
	}

	// If boomerang is incoming and collides with player, it is caught.
	if ( m_eCurrentBehavior == BOOMERANG_INCOMING &&
		(pSprite->getCollisionMask() & SPRITE_COLLIDE_PLAYER) &&
			cSprite::collide(pSprite) )
	{
		// Link caught the boomerang
		m_eCurrentBehavior = BOOMERANG_CAUGHT;
		
		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Called to reset the boomerang.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectBoomerang::reset()
{
	m_iHitPoints = 0;
	m_eState = INACTIVE;
	m_fSpeed = 0;
	m_iAnimateOffset = 0;
}


//////////////////////////////////////////////////////////////////////////////
//
//	Activate - Toss the boomerang.
//
//	ARGS:	fx,			Horizontal location of Link when tossing.
//			fy,			Vertical location of Link when tossing.
//			eDirection,	Direction of Link during toss.
//
//	POST:	m_fx,				Boomerang's new hortizontal location.
//			m_fy,				Boomerang's new vertical location.
//			m_eDirection,		Direction boomerang is travelling.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectBoomerang::activate(float fx, float fy, eDIRECTION eDirection)
{ 
	BEGIN_ERRORHANDLER_GAME

	if ( this->isDead() )
	{
	// Set the location and direction of the boomerang.
		m_fx = fx;
		m_fy = fy;
		m_eDirection = eDirection;

		reset();										// Call the reset.

		// Set initial behavior as outgoing.
		m_eCurrentBehavior = BOOMERANG_OUTGOING;

		// Start out full speed.
		m_fSpeed = fBOOMERANG_TOSS_SPEED;
		m_iHitPoints = 1;							// Enough to keep sprite alive.

		m_lAnimationTime = 0;						// Reset animation time holder. 
	
		m_eState = ACTIVE;							// Force to be active.
		m_pGameState->addActiveSprite(this);	// Inject into active sprites.
		return true;
	}
	
	return false;

	END_ERRORHANDLER_GAME
}



//////////////////////////////////////////////////////////////////////////////
//
//	Updates the movement of the boomerang.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectBoomerang::update()
{
	BEGIN_ERRORHANDLER_GAME


	float fFrameFactor = m_pGameState->getFrameFactor();


	// Adjust when the color changes.
	m_lAnimationTime -= m_pGameState->getTimeLapseMS();

	// If the flash time is exceeded...
	if (m_lAnimationTime < 0)
	{
		// Reset the the flash timer and jump to the next color.
		m_lAnimationTime = lBOOMERANG_ANIMATION_DELAY;	
		m_iAnimateOffset++;

		// If color is out of range reset it back to the first color.
		if (m_iAnimateOffset > 7)
			m_iAnimateOffset = 0;
	}

	// The boomerang will follow a straight path from the time Link
	//	throws it until it reaches its maximum range. Then it will
	//	seek and return to link.
	if (m_eCurrentBehavior == BOOMERANG_OUTGOING)
	{
		  // The boomerang is slowing down
		  // If it stops it is now incoming

		  // compensate for frame rate
		  // equation derived from xf = x0 + v0t + (at^2)/2
		  // (xf - x0)/t = v0 + at/2
		  // average velocity = initial velocity + (acceleration * time)/2
		  // ???ABL - throw distance still is not independent of frame rate
		  //	Going from an 800 mhz with no acceleration to a 1.8Ghz with GeForce4
		  //	makes it fly about half a tile further.

		m_fSpeed +=  (-fBOOMERANG_ACCELERATION * fFrameFactor) / 2.0f;

		// If the boomerang tries to leave play make it bounce.
		if ( m_fSpeed < 0.0f || isOutOfPlayfield() )
		{
			m_fSpeed = 0.0f;
			m_eCurrentBehavior = BOOMERANG_INCOMING;
		}
		else	// Make it move
		{
			switch (m_eDirection)
			{
			case DOWN:
				m_fy -= m_fSpeed * fFrameFactor;
				break;
			case UP:
				m_fy += m_fSpeed * fFrameFactor;
				break;
			case LEFT:
				m_fx -= m_fSpeed * fFrameFactor;
				break;
			case RIGHT:
				m_fx += m_fSpeed * fFrameFactor;
				break;
			}
		}
	} // END boomerang outgoing
	
	// Is the boomerang incoming?
	else if (m_eCurrentBehavior == BOOMERANG_INCOMING)
	{
		// If so, speed it up.
		m_fSpeed += (fBOOMERANG_ACCELERATION * fFrameFactor) / 2.0f;
		// whoa, Hold on there boomerang, you're going too fast
		if ( m_fSpeed > fBOOMERANG_MAX_RETURN_SPEED )
		{
			m_fSpeed = fBOOMERANG_MAX_RETURN_SPEED;
		}

		float fVelocityVector[2];	// Temporary vector

		// Point at link
		getVelocityVector(m_pGameState->getLink(), m_fSpeed, fVelocityVector);
		// Add vector to current position to move towards link
		m_fx += fVelocityVector[0] * fFrameFactor;
		m_fy += fVelocityVector[1] * fFrameFactor;
	
	} // END boomerang incoming
	
	// Is the boomerang caught?
	else if (m_eCurrentBehavior == BOOMERANG_CAUGHT)
	{
		m_iHitPoints = 0;			// Set to be removed from active sprites.
	}
	else
	{
		THROW_GAME_INT("Boomerang has undefined behavior.", m_eCurrentBehavior);
	}


	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Used to turn boomerang into a Link-seeking missle.
//	-	Get a vector towards oTarget's direction with magnitude speed.
//
//	ARGS:	oTarget,	Target sprite you want the vector to point at.
//			fSpeed,	The magnitude(speed) of the vector.
//			fVector,	To hold returned vector.
//
//	POST:	fVector,	The final vector pointing from this sprite to the target
//						sprite with magnitude of fSpeed.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectBoomerang::getVelocityVector(cSprite *oTarget, float fSpeed,
											float *fVector)
{
	BEGIN_ERRORHANDLER_GAME

	//  Get distance vector,
	fVector[0] = oTarget->getX() - m_fx;
	fVector[1] = oTarget->getY() - m_fy;

	//	Get magnitude,
	float fLength = sqrt( fVector[0] * fVector[0] + fVector[1] * fVector[1] );

	//	Divide by magnitude to normalize, then multiply by speed to create
	// the final vector.
	(fVector[0] /= fLength) *= fSpeed;
	(fVector[1] /= fLength) *= fSpeed;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Draw the boomerang in flight.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectBoomerang::draw()
{
	BEGIN_ERRORHANDLER_GAME

	// Draw Boomerang.
	m_pTiles->draw(m_fx, m_fy, Z_PLANE_SHOT, 
		ID_ITEM_BOOMERANG + m_iAnimateOffset, 
		m_yColorSet);

	END_ERRORHANDLER_GAME
}

// vi:set ts=3:
