//////////////////////////////////////////////////////////////////////////////
//
//	OBJECTCandle.cpp
//	Implementation for the cObjectCandle class.
//
//	-	Encapsulation for the candle Link uses.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////


#include "objectCandle.h"

const long lCANDLE_SWING_DELAY = 160;	// Max time for a Candle swipe.


//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
// -	The Candle is a brain-dead object which is not affected by other
//		sprites.  It's constantly reset by the "Link" object, each time
//		with a possibly different orientation and location on the screen.
//		
//	ARGS:	pGameState,		Game state object.
//
//////////////////////////////////////////////////////////////////////////////
cObjectCandle::cObjectCandle(cGameState* pGameState)		
{
	m_pProjectileFire = new cObjectProjectileFire(pGameState);

	// Set game state object.
	m_pGameState = pGameState;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(SPRITE_SET_ITEMS);

	// Don't auto-delete when Candle "dies" (stops swinging).
	m_bDeleteable = false;

	// Set defaults.
	cSprite::reset();


	// Link starts with a green Rod.
	m_yColorSet = m_yColorSetBeforeFlashing = COLOR_SET_ITEMS_BLUE;

	// Override defaults...
	m_iDamage = 0;			// Amount of damage sprite inflicts.
	m_iHitPoints = 0;		// Rod starts off "dead".
	m_eCollisionMask = static_cast<eSPRITE_COLLIDE>
		(SPRITE_COLLIDE_PLAYER | SPRITE_COLLIDE_SHOT);

	// Set the ID - Rod head side up
	m_iInventoryDisplayID = ID_ITEM_CANDLE;

	// Set the swing delay for Link.
	m_lSwingDelay = lCANDLE_SWING_DELAY;

	// Dimensions of Rod.
	m_fWidth	= m_pTiles->getWidth();
	m_fHeight	= m_pTiles->getHeight();

	// Border around sprite used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectCandle::~cObjectCandle()
{
	delete (m_pProjectileFire);
}


//////////////////////////////////////////////////////////////////////////////
//
//	The bow doesn't collide with anything
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectCandle::collide(cSprite* pSprite)
{
	return false;
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Called when a Rod swing is first performed.
//	-	Called to reset the location of the Rod.
//
//	PRE:	m_fx,				Screen horizontal coordinate in pixels of Link's
//								location.
//			m_fy,				Screen vertical coordinate in pixels of Link's
//								location.
//			m_eDirection,	Which direction to make the Rod facing.
//
//	POST:	m_fx,				Adjusted horizontal position.
//			m_fy,				Adjusted vertical position.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectCandle::reset()
{
	BEGIN_ERRORHANDLER_GAME

	m_iHitPoints = 0;
	m_eState = INACTIVE;
	m_pProjectileFire->reset();

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Overload, which uses the current direction.
//	-	Assume Rod position is moving, but does not perform a full reset
//		since the timer (ie: the "swing") doesn't begin again.
//
//	ARGS:	fx,	The new horizontal position (in pixels) for the candle.
//			fy,	The new vertical position (in pixels) for the candle.
//
//	POST:	m_fx,	Set to new horizontal position.
//			m_fy,	Set to new vertical position.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectCandle::reset(float fx, float fy)
{ 
	BEGIN_ERRORHANDLER_GAME

	
	// Set the new location.
	m_fx = fx;
	m_fy = fy;
	
	// Set the direction of the Rod.
	switch(m_eDirection)
	{
		// If direction is valid, store it in member variable and adjust
		// Rod based on direction.
		case DOWN:
			m_fy -= 14.0f;
			break;

		case UP:	
			m_fy += 10.0f;
			break;

		case LEFT:
			m_fx -= 12.0f;
			m_fy -= 2.0f;
			break;

		case RIGHT:	
			m_fx += 12.0f;
			m_fy -= 2.0f;
			break;

		// Anything else is illegal which should cause an error.
		default:
		{
			// Construct an error message and throw.
			THROW_GAME_INT("Only up, down, left, and right are legal directions "
				"values for a Rod to be reset facing.", m_eDirection);
			break;
		}
	} // END direction of swing
	
	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Updates the movement of the Rod.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectCandle::update()
{
	BEGIN_ERRORHANDLER_GAME

	// Animation time counts down for Rod.  If 0, it's no longer moving
	// through it's motions and can be removed from the active sprites.  (This
	// is done by making it's hitpoints 0.)
	if (m_lAnimationTime > 0)
	{
		// Update the animation time with the time that passed.
		m_lAnimationTime -= m_pGameState->getTimeLapseMS();

		// Snap to 0 if animation timer is done.
		if (m_lAnimationTime < 0)
			m_lAnimationTime = 0;
	}
	else
	{
		m_iHitPoints = 0;			// Set to be removed from active sprites.
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// Activate Candle - shoot arrow
//
//
//	ARGS:	fx,			Horizontal location of Link when swinging.
//			fy,			Vertical location of Link when swinging.
//			eDirection,	Direction of Link during swing.
//
//	POST:	m_fx,				Rod's new hortizontal location.
//			m_fy,				Rod's new vertical location.
//			m_eDirection,	Direction Rod is facing.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectCandle::activate(float fx, float fy, eDIRECTION eDirection)
{
	BEGIN_ERRORHANDLER_GAME

	// One arrow out at a time and a rupee must be spent
	if ( m_pProjectileFire->isDead() )
	{
		// Set the location and direction of the bow.
		m_eDirection = eDirection;
		reset(fx, fy);										// Call the reset.
		m_lAnimationTime = lCANDLE_SWING_DELAY;		// Reset animation time holder. 
		m_iHitPoints = 1;							// Enough to keep sprite alive.

		m_eState = ACTIVE;							// Force to be active.
		m_pGameState->addActiveSprite(this);	// Inject into active sprites.

		m_pProjectileFire->activate(fx, fy, eDirection);

		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Draw the Rod being swung.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectCandle::draw()
{
	// the actual candle is not drawn in action, just the fire
}

// vi:set ts=3:
