//////////////////////////////////////////////////////////////////////////////
//
//	objectLynk.cpp
//	Implementation for the cObjectLynk class.
//
//  - The main character of the game.
//	 -	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "objectLynk.h"          // Defines for this class.

//////////////////////////////////////////////////////////////////////////////
//
// CONSTRUCTOR
//
//	ARGS:	pGameState,		Points to game state object.
//
//////////////////////////////////////////////////////////////////////////////
cObjectLynk::cObjectLynk(cGameState* pGameState, cInventory* pInventory) {
	// Set game state object.
	m_pGameState = pGameState;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(SPRITE_SET_GOOD_CHARACTERS);

	// Not autodeleteable.  The Link object will be maintained manually.
	m_bDeleteable = false;

	// Have the base class initialize the bulk of the member variables.
	cSprite::reset();

	// Common attributes...
	m_yColorSet						= COLOR_SET_GOOD_CHARACTERS_LIME;// Color set.
	m_yColorSetBeforeFlashing	= COLOR_SET_GOOD_CHARACTERS_LIME;// Color set.
	m_eCollisionMask	= SPRITE_COLLIDE_PLAYER;			// Collision behavior.
	m_eSpawnType		= SPAWN_TYPE_TILE_COORDINATE;		// How sprite spawns.
	m_fx = 80.0f;													// Starting X position
	m_fy = 50.0f;													// Starting Y position
	m_eZ = Z_PLANE_CHARACTER;									// Z-Plane


	// Dimensions of Link come from the tiles.
	m_fWidth			= m_pTiles->getWidth();
	m_fHeight		= m_pTiles->getHeight();

	// Addition height information to allow Link's head to overlap certain
	// solid blocks.	
	m_fHeightHalf	= m_fHeight / 2.0f;


	// Border around Link used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;


	// Override defaults set in base class.
	m_fSpeed				= 1.2f;	// Speed of link.
	m_iHitPoints		= 20;		// ???TMH - Make a large # for debugging purposes.


	// Members not from the base class.
	m_bMagicSheild		= false;			// Does not have the magic shield.

	// ???TMH - These all need to be moved to the inventory class and/or
	//          a keyboard handling class.  All Link should be told to do is
	//          "pose" for an action.  The Link class should be unaware of the
	//          state of the weapons.
	m_bIsSwinging									= false;	// Link is not swinging the sword.	
	m_bSwordStartActivate						= false;
	m_bActivateSwordButtonReleased			= true;
	m_bInventoryItemStartActivate				= false;
	m_bActivateInventoryItemButtonReleased	= true;
	m_bInventoryItemStartSelectNext			= false;
	m_bSelectNextInventoryItemButtonReleased = true;

	// Obtain pointer to inventory object.
	m_pInventory = pInventory;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectLynk::~cObjectLynk()
{	
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	If anything collides with link, nothing happens because Link does not
//		deal damage to sprites which touch him.
//	-	What about enemy projectiles you say?  It's up to their implementation
//		of collide() to inflict the damage back to Link.  Remember, the
//		sprite implementing collide() DEALS damage...it does not TAKE damage
//		(at least not through that method).
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectLynk::collide(cSprite*)
{
	return false;
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Check to see if Link has triggered any events.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectLynk::eventCollide()
{
	BEGIN_ERRORHANDLER_GAME

	vector<cSprite*>* vpSprites;		// Sprites for current screen.	
	cEvent* pEvent = NULL;

	vpSprites = m_pGameState->getSprites();

	// Loop through active sprites, checking their events against Link.
	for(SPRITE_ITR itr = vpSprites->begin(); itr != vpSprites->end(); itr++)
	{
		// Check if the target sprite has an event on it.
		if ( (pEvent = (*itr)->getEvent()) )
		{
			// Make sure Link is in the range of an active event
			if (	pEvent->isActive() &&
					((*itr)->getX() <= m_fx + fEPSILON_EVENT) &&
					((*itr)->getY() <= m_fy + fEPSILON_EVENT) &&
					((*itr)->getX() >= m_fx - fEPSILON_EVENT) &&
					((*itr)->getY() >= m_fy - fEPSILON_EVENT)	)
			{
				// If Link is colliding with an active event, trigger it.
				pEvent->trigger();
				return true;
			}
			// If Link re-enters a screen right on a portal, it should remain
			// inactive until Link isn't colliding with it.
			else if ( !(pEvent->isActive()) 
						&&	(	(pEvent->getEventType() == EVENT_TYPE_PORTAL_TELEPORT_ACTIVE) 
							||	(pEvent->getEventType() == EVENT_TYPE_PORTAL_WALK_ACTIVE) ) 
						&&	!(cSprite::collide(*itr)) )
			{
				pEvent->activate();
			}

		}

	} // END loop through sprites

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Resets the state of Link.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::reset()
{
	THROW_GAME("Cannot call \"reset\" on Link sprite as it's always in play.");
}


//////////////////////////////////////////////////////////////////////////////
//
//	Decides which update to call based on which state the game is in.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::update()
{
	BEGIN_ERRORHANDLER_GAME

	// Base how Link updates on the game state.
	switch(m_pGameState->getState())
	{
		case PLAY_GAME:
			updatePlayGame();
			break;

		// Move Link as screen scrolls.
		case SCROLL_GAME:										// Fall through...

		// Move Link as a portal transition is active.
		case PORTAL_GAME:
			m_lRecoilTime = 0;								// Stop any recoiling.			
			m_lFlashTime = 0;									// Stop any flashing.
			m_yColorSet = m_yColorSetBeforeFlashing;	// Proper color set.
			if (isSwinging())
				m_pInventory->getSword()->reset();

			updateAdjustWalkingGfx();		// Make Link walk as scroll occurs.
			break;

		default:
		{
			// Unknown game state, make an error message and throw.		
			THROW_GAME_INT("Link attempted an update while containing an "
				"unknown state value.", m_pGameState->getState() );
		}
	}

	END_ERRORHANDLER_GAME
}



//////////////////////////////////////////////////////////////////////////////
//
//	-	Updates Link's position, and actions.
//	-	Throughout the routine (and other routines called), the m_fMoveOffset
//		values are adjusted as necessary.  Eventually they are applied and a
//		collision detection is performed.  If a collision occurs, regress to
//		the original location by subtracting the m_fMoveOffset values.
//
//	POST:	m_fx,					(May be) new X position for Link.
//			m_fy,					(May be) new Y position for Link.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::updatePlayGame()
{
	BEGIN_ERRORHANDLER_GAME

	// update the frozen timer and if time's up reset swing status
	if ( (m_lFrozenTimer -= m_pGameState->getTimeLapseMS()) <= 0.0f )
	{
		thaw();
	}

	// Error out, Link is always active.
	if (m_eState == INACTIVE)
		THROW_GAME_INT("Link's state is inactive.  That's impossible!", m_eState);


	// Reset movement amount to be nothing.
	m_fMoveOffsetX = 0.0f;
	m_fMoveOffsetY = 0.0f;
	
	if (m_eState & FLASHING)		// Update flashing if necessary.
		flash();

	if (m_eState & RECOILING)		// Update recoiling if necessary.
		recoil();
	else									
		if (!isSwinging())			// Only get input from user if they aren't
			getInputFromUser();		// recoiling or sword swinging.

	// Update the position of Link by adding any offsets obtained above.
	m_fx += m_fMoveOffsetX;
	m_fy += m_fMoveOffsetY;


	// If Link is not flashing,
	// perform collision check against damaging enemies sprites.
	if (!(m_eState & FLASHING))
		m_pGameState->getCollidingSprites(this, static_cast<eSPRITE_COLLIDE>
				(SPRITE_COLLIDE_ENEMY | SPRITE_COLLIDE_DAMAGE) );


	// Adjust the height to half and check against solid objects.
	m_fEpsilonY = m_fHeightHalf - fEPSILON_BORDER;
	m_pGameState->getCollidingSprites(this, SPRITE_COLLIDE_SOLID);

	// Only check event collision if he's not recoiling
	if ( !(m_eState & RECOILING) )
	{
		eventCollide();
	}

	// Put collision height back to what it normally is.
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;


	// Check if scroll to another screen needs to occur.
	if (isOutOfPlayfield())
	{
		// Yup Link has hit the boundry. 
		
		// Reset Inventory
		m_pInventory->reset();

		// Link may not recoil into another screen, so check his state.
		if (m_eState & RECOILING)
		{
			// Stop recoiling...
			moveBack();
			m_eState = static_cast<eSPRITE_STATE>( m_eState ^ RECOILING );
			m_lRecoilTime = 0;	// Reset timer.
		}
		else
		{
			// Yup Link has hit the boundry. 

			// If Link is in a cave, he should exit, not scroll.
			// ???TMH - Remove this when special screens are implemented.  It
			//				be up to the screen to interpret the scroll state.
			if ( m_pGameState->getMap() == m_pGameState->getMap( MAP_CAVES ) )
			{
				// Change game state to portal.
				m_pGameState->setState(PORTAL_GAME);
				m_pGameState->setPortalState(PORTAL_OPEN);
			}
			else
			{
				// Setup the scrolling information...

				// Change game state to scrolling.
				m_pGameState->setState(SCROLL_GAME);
				
				// Set the scroll direction based on where Link went out of boundries.
				// Note:		This cannot be done by looking at Link's direction, as he 
				//				may have trigged a scroll during a recoil.  If that
				//				happened, then the screen would scroll in the opposite 
				//				direction of where he's at (and that's just wacky).
				
				// Set the direction based on where Link touched the screen.
				if (m_fx < 0.0f)
					m_pGameState->m_eScrollDirection = LEFT;
				else if ((m_fx + m_fWidth) > m_pGameState->getPlayfieldWidth())
					m_pGameState->m_eScrollDirection = RIGHT;
				else if (m_fy < 0.0f)
					m_pGameState->m_eScrollDirection = DOWN;
				else
					m_pGameState->m_eScrollDirection = UP;
			}
		}
	}

	// Check for button releases
	cont_cond_t cond = m_pGameState->cond;	// dreamcast controller
	
	if (!KEYS(CONT_X))
	{
		m_bSelectNextInventoryItemButtonReleased = true;
	}

	if (!KEYS(CONT_A))
	{
		m_bActivateSwordButtonReleased = true;
	}

	if (!KEYS(CONT_B))
	{
		m_bActivateInventoryItemButtonReleased = true;
	}

	if( m_bInventoryItemStartSelectNext == true )
	{
		m_pInventory->nextItem();
		m_bInventoryItemStartSelectNext = false;
	}

	// ???TMH - Link cannot shoot "into" a screen while on the outer edge of
	//				tiles.  He can shoot to tiles next to him.  This logic needs
	//				to be tweaked to account for that game quirk.
	if ( !isOutOfPlayfield())
	{
		// Set sword at Link's location and facing Link's direction.
		// This also "activiates" the sword.
		if( m_bSwordStartActivate == true )
		{
			m_pInventory->getSword()->activate(m_fx, m_fy, m_eDirection);
			m_bSwordStartActivate = false;
			startSwing();
		}	
	
		// Check to see if B Item is to be activated
		if (m_bInventoryItemStartActivate == true)
		{
			if ( m_pInventory->getCurrentItem()->
					activate(m_fx, m_fy, m_eDirection) )
			{
				m_bInventoryItemStartActivate = false;
				startSwing();
			}
		// reset this variable whether Link threw or not,
		// otherwise you could try to throw in mid flight and 
		// it would immediately throw it again on return
		m_bInventoryItemStartActivate = false;
		}
	}

	END_ERRORHANDLER_GAME
}



//////////////////////////////////////////////////////////////////////////////
//
//	-	Update Link's location based on player' movement.
//	-	???TMH - This needs to be in some sort of cPlayControl class which
//					is accessed via the game state.
/// 
//	POST:	m_eDirection,		New direction Link is facing.
//			m_fMoveOffsetX,	Amount Link needs to move horizontally.
//			m_fMoveOffsetY,	Amount Link needs to move vertically.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::getInputFromUser()
{
	BEGIN_ERRORHANDLER_GAME

	cont_cond_t cond = m_pGameState->cond;	// dreamcast controller

	// Obtain the multipler to use for all movement updates.
	float fFrameFactor = m_pGameState->getFrameFactor();

	// ----- Weapons -----

	// If Link is swinging he can't do anything else
	if( !isSwinging() )
	{
		// ----- Check for item selection change -----
		if ( m_bSelectNextInventoryItemButtonReleased &&
				(KEYS(CONT_X)) )
		{
			m_bInventoryItemStartSelectNext = true;
			m_bSelectNextInventoryItemButtonReleased = false;
		}

		// ----- Check for Item/Sword activations. -----
		if ( m_bActivateInventoryItemButtonReleased &&
				(KEYS(CONT_B)) )
		{
			m_bInventoryItemStartActivate = true;
			m_bActivateInventoryItemButtonReleased = false;
		}

		// Check if the sword is activated.
		else if ( m_bActivateSwordButtonReleased &&
				(KEYS(CONT_A)) )
		{
			m_bSwordStartActivate = true;
			m_bActivateSwordButtonReleased = false;
		}


		// ----- Move Link. -----

		// If a direction key is pressed, then link is moving and the animation
		// timer is updated.
	
		// Link can only move one direction at a time.  He's bound to an invisible
		// 8x8 grid.  Whenever he turns 90 degrees, he snaps back to the grid.
	
		// Link's equivalent grid position.
		int iCurrentXPosMod8 = (int)m_fx % 8;
		int iCurrentYPosMod8 = (int)m_fy % 8;
	
		// Move downward.
		if (KEYS(CONT_DPAD_DOWN))
		{
			//printf(">> input - down\n");
			// Is Link aligned to grid?
			if (iCurrentXPosMod8 == 0)
			{
				// Yes, so let him turn.
				m_eDirection = DOWN;									// Change direction.
				m_fMoveOffsetY = -(m_fSpeed * fFrameFactor);	// Move position.
			}
			else
			{
				// Link is not on the grid, so move him to the nearest vertical
				// grid line.
			
				// Check if the grid line on Link's right is closer.
				if ( iCurrentXPosMod8 > 4 )
				{
					// Yes it is.
					m_eDirection = RIGHT;							// Change direction.
					m_fMoveOffsetX += 8 - iCurrentXPosMod8;	// Move position.
				}
				else
				{
					// Left grid line is closer, move to it.
					m_eDirection = LEFT;								// Change direciton.
					m_fMoveOffsetX -= iCurrentXPosMod8;			// Move position.
				}
			}
		} // END if moving down.
	
	
		// Move upward.
		if (KEYS(CONT_DPAD_UP))
		{
			//printf(">> input - up\n");
			// Is Link aligned to grid?
			if (iCurrentXPosMod8 == 0)
			{
				// Yes, so let him turn.
				m_eDirection = UP;									//	Change direction.
				m_fMoveOffsetY = m_fSpeed * fFrameFactor;		// Move position.
			}
			else 
			{
				// Link is not on the grid, so move him to the nearest vertical
				// grid line.
	
				// Check if the grid line on Link's right is closer.
				if ( iCurrentXPosMod8 > 4 )
				{
					// Yes it is.
					m_eDirection = RIGHT;							// Change direction.
					m_fMoveOffsetX += 8 - iCurrentXPosMod8;	// Move position.
				}
				else
				{
					// Left grid line is closer, move to it.
					m_eDirection = LEFT;								// Change direction.
					m_fMoveOffsetX -= iCurrentXPosMod8;			// Move position.
				}
			}
		} // END if moving up.
	
	
		// Move left.
		if (KEYS(CONT_DPAD_LEFT))
		{
			//printf(">> input - left\n");
			// Is Link aligned to grid?
			if (iCurrentYPosMod8 == 0)
			{
				// Yes, so let him turn.
				m_eDirection = LEFT;									// Change direction.
				m_fMoveOffsetX = -(m_fSpeed * fFrameFactor);	// Move position
			}
			else 
			{
				// Link is not on the grid, so move him to the nearest horizontal
				// grid line.
	
				// Check if the grid line above Link is closer.
				if ( iCurrentYPosMod8 > 4 )
				{
					// Yes it is.
					m_eDirection = UP;								// Change direction.
					m_fMoveOffsetY += 8 - iCurrentYPosMod8;	// Move position
				}
				else
				{
					// Bottom grid line is closer, move to it.
					m_eDirection = DOWN;								// Change direction.
					m_fMoveOffsetY -= iCurrentYPosMod8;			// Move position
				}
			}
		} // END if moving left.
	
	
		// Move right.
		if (KEYS(CONT_DPAD_RIGHT))
		{
			//printf(">> input - right\n");
			if (iCurrentYPosMod8 == 0)
			{
				m_eDirection = RIGHT;								// Change direction.
				m_fMoveOffsetX = m_fSpeed * fFrameFactor;		// Move position.
			}
			else 
			{
				// Check if the grid line above Link is closer.
				if ( iCurrentYPosMod8 > 4 )		
				{
					// Yes it is.
					m_eDirection = UP;								// Change direction.
					m_fMoveOffsetY += 8 - iCurrentYPosMod8;	// Move position.
				}
				else
				{
					// Bottom grid line is closer, move to it.
					m_eDirection = DOWN;								// Change direction.
					m_fMoveOffsetY -= iCurrentYPosMod8;			// Move position.
				}
			}  
		} // END if moving right.
			// ----- Finish Link move and perform animation update. -----	
		// If any movement occurred...
		if (m_fMoveOffsetX != 0.0f || m_fMoveOffsetY != 0.0f)
			updateAdjustWalkingGfx();
	} // END if not swinging.
	
	END_ERRORHANDLER_GAME 
}


//////////////////////////////////////////////////////////////////////////////
//
//	Start the counter for Link to swing his sword.
//
//	POST:	m_bFrozenTimer,	Contains time Link will be in mid-swing.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::startSwing()
{
	BEGIN_ERRORHANDLER_GAME

	// Link is frozen because he is swinging.

	m_bIsSwinging = true;

	// ???TMH - Change explicitly setting frozen timer to calling setState()
	//				(on cSprite) to be frozen.  Logic in setState should set the
	//				sprite to be frozen.
	m_lFrozenTimer = m_pInventory->getSword()->getSwingDelay();

	END_ERRORHANDLER_GAME
}



//////////////////////////////////////////////////////////////////////////////
//
//	Make's Link look like he is walking by updating which tile to use.
//
//	POST:	m_dwAnimationTimer,	Adjusted by the # of milleseconds that have
//										passed since last update cycle.
//			m_iAnimationOffset,	If enough time has passed, will be a new
//										value representing the tile to draw Link with.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::updateAdjustWalkingGfx()
{
	BEGIN_ERRORHANDLER_GAME

	// Update the animation 
	m_lAnimationTime += m_pGameState->getTimeLapseMS();

	// Reset the animation timer when limit hit.
	if (m_lAnimationTime > lANIMATION_WALK_MAX)
		m_lAnimationTime -= lANIMATION_WALK_MAX;

	// Set animation index according to amount of time that has passed.
	if (m_lAnimationTime > (lANIMATION_WALK_MAX / 2))
		m_iAnimateOffset =1;
	else
		m_iAnimateOffset =0;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Draw Link and any sprites he explicity controls.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::draw()
{
	BEGIN_ERRORHANDLER_GAME


	// Determine which tiles to use when drawing Link, based on whether or
	// not he is attacking.
   if (isSwinging())
	{
		// Draw Link as attacking.
		m_pTiles->draw(m_fx, m_fy, m_eZ, 
			ID_LINK_ATTACK + m_eDirection,
			m_yColorSet);
	}
	else
	{
		// Link is not swinging.  Draw "walking" link tiles.
		m_pTiles->draw(m_fx, m_fy, m_eZ, 
			ID_LINK + (m_eDirection*2) + m_iAnimateOffset +
				(m_bMagicSheild ? OFFSET_LINK_SHIELD : 0),
			m_yColorSet);
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// -	If Links's boundries extend beyond the playfield.
//	-	Overloaded the default implementation in cSprite, because Link can
//		travel to the edge of the screen.  Heck, he needs to travel to the edge
//		in order to activate a scroll condition.
//
//	PRE:	m_pGameState,	Set to game state.
//
//	POST:	Game state is changed to SCROLL_GAME if Link extends beyond the
//			playfield boundries.  If this is true, a direction for scrolling
//			is set as well.
//
//	RETURNS:	true,		If any portion of Link exceeds playfield boundries.
//				false,	Link is within the boundries of the playfield.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectLynk::isOutOfPlayfield()
{
	BEGIN_ERRORHANDLER_GAME


	// Perform quick check if Link is out of his playfield boundry.

	// SPRITE					PLAYFIELD
	// -------------------  ---------------------------------------
	if ((m_fx					< 0.0f) ||
		((m_fx + m_fWidth)	> m_pGameState->getPlayfieldWidth()) ||
		(m_fy						< 0.0f) ||
		((m_fy + m_fHeight)	> m_pGameState->getPlayfieldHeight())	)
	{
		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Make Link and his sword (if swinging) recoil.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::recoil()
{
	BEGIN_ERRORHANDLER_GAME

	// Have base class perform the recoil.
	cSprite::recoil();

	// If in a swing, have the sword move to Link's new position.
	if (isSwinging())
	{
		m_pInventory->getSword()->reset(m_fx, m_fy);
		m_pInventory->getCurrentItem()->reset(m_fx, m_fy);
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Positions Link on the other side of the screen after a recoil.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectLynk::flipScreenPositionAfterScroll()
{
	// Switch position on screen based on direction.
	switch(m_pGameState->m_eScrollDirection)
	{
		case DOWN:
				m_fy = m_pGameState->getPlayfieldHeight() - (m_fHeight / 1.0f);
			break;

		case UP:
				m_fy = 1.0f;
			break;

		case LEFT:
				m_fx = m_pGameState->getPlayfieldWidth() - (m_fWidth + 1.0f);
			break;

		case RIGHT:
				m_fx = 1.0f;
			break;

		default:
		{
			// Unknown scroll direction. Throw an error...
			THROW_GAME_INT("During Link's attempt to flip sides after a scroll, "
				"the game state contained an unknown direction value.",
				m_pGameState->m_eScrollDirection);
		}

	} // END switch direction

}
// vi:set ts=3:
