//////////////////////////////////////////////////////////////////////////////
//
//	OBJECTOctorock.cpp
//	Implementation for the cObjectOctorock class.
//
//	-	???TMH - Correct spelling is cObjectOctorok.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////


#include "objectOctorock.h"

// Different speeds of Octorocks.
const float fSPEED_SLOW		= 300.0f;
const float fSPEED_AVERAGE = 300.0f;
const float fSPEED_FAST		= 300.0f;


//////////////////////////////////////////////////////////////////////////////
//
//	CONSTRUCTOR
//
//	ARGS:	pGameState,		Point to game state object.
//			oSpriteData,	Contains data on how to construct sprite.
//
//////////////////////////////////////////////////////////////////////////////
cObjectOctorock::cObjectOctorock(cGameState* pGameState, 
	cSpriteData& oSpriteData) :	
		cSprite(pGameState, oSpriteData)
{
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectOctorock::~cObjectOctorock()
{
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Collision check logic.
//	-	Performs hit if collision is with player.
//
//	ARGS:	pSprite,	The sprite colliding with the Octorock.
//
//	RETURNS:	true,		Collision.
//				false,	No collision.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectOctorock::collide(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// Make sure target sprite is the player, then check if it's colliding.
	if (	(pSprite->getCollisionMask() & SPRITE_COLLIDE_PLAYER) &&
			cSprite::collide(pSprite) )
	{
		pSprite->hit(this);
		return true;
	}

	return false;
	
	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// Moves Octorok back to it's previous location on a collision.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectOctorock::moveBack()
{
	BEGIN_ERRORHANDLER_GAME

	cSprite::moveBack();								// Perform move (back).	
	m_fMoveTime	= TIME_FOR_A_NEW_DIRECTION;	// Change movement direction.

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//	
//	-	Resolve a sprite hit event happening to this sprite.
//	-	???TMH - Eventually move this logic to general enemy class.
//
//	ARGS:	pSprite,		The sprite causing the hit action.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectOctorock::hit(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// React according to type of hit sprite is causing.
	switch ( pSprite->getHitType() )
	{
		// Was hit by boomerang.
		case HIT_TYPE_BOOMERANG:

			// Set state to be frozen.
			setState( static_cast<eSPRITE_STATE>(getState() | FROZEN) );

			// Reset the amount of time sprite is frozen.
			m_lFrozenTimer = lFREEZE_COUNTDOWN;
			break;

		// Was hit by something else, do default action.
		default:
			cSprite::hit(pSprite);
	}

	END_ERRORHANDLER_GAME
}



//////////////////////////////////////////////////////////////////////////////
//
//	-	Sets up sprite to run.
//
//	POST:	All member variables set to defaults.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectOctorock::reset()
{
	BEGIN_ERRORHANDLER_GAME

	// Will hold the clear map tile sprites on which Octorock can initially
	// appear on.
	vector<cSprite*>*	pvpClearMapSprites;

	// A randomly picked index of the clear map sprite which the Octorock
	// will use to initially set it's x,y coordinates.
	int iMapSprite;

	cSprite::reset();	// Default reset.

	// Override defaults...
	m_iDamage		= 1;		// Causes a little damage.
	m_eDirection	= static_cast<eDIRECTION>(rand() % 4);	// Random direction.

	// Non-defaults.
	m_bMoving	= false;								// Not moving.
	m_fMoveTime = TIME_FOR_A_NEW_DIRECTION;	// Pick direction to move.
	

	// Set members according to if enemy is easy or hard.
	if (m_yColorSet == COLOR_SET_OVERWORLD_ENEMIES_BLUE)
	{
		// Hard (blue) Octoroks have...
		m_iHitPoints = 2;					//	Two hit points.
		m_fSpeed = 0.5f;					// Move slow.		
	}
	else if (m_yColorSet == COLOR_SET_OVERWORLD_ENEMIES_RED)
	{
		// Easy (red) Octoroks have...
		m_iHitPoints = 1;					// Easy kill.
		m_fSpeed = 0.75f;					// move fast.
	}
	else
	{
		// Support for BLACK Octoroks.  Muhaha.
		m_iHitPoints = 3;				// Takes a few hits to kill this guy.
		m_fSpeed = 1.0f;				// Very fast.
		m_iDamage = 2;					// Hurts you much!
	}

	// Obtain collection of clear map tile sprites.
	pvpClearMapSprites = m_pGameState->getSprites(SPRITE_COLLIDE_DEFAULT);

	// Make sure at least one map sprite is clear.
	if (pvpClearMapSprites->size() < 1)
		THROW_GAME("Octorok cannot reset because screen has no clear tiles.");

	// Loop until a clear tile is picked which is in the enemy's playfield.
	do
	{
		// Randomly pick the collection's index of one of the clear tile sprites.
		iMapSprite = rand() % pvpClearMapSprites->size();

		// Set Octorok's position to that of the clear tile sprite.
		m_fx = ((*pvpClearMapSprites)[iMapSprite])->getX();
		m_fy = ((*pvpClearMapSprites)[iMapSprite])->getY();

	} while (isOutOfPlayfield());


	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Perfrom AI for direction of movement, and whether or not to create
//		an enemy projectile sprite.
//
//	POST:	m_dwAnimationTime,	How much time has passed since the first tile
//										of the sprite's animation sequence has been
//										displayed.
//			m_fMoveTime,			How much time has passed since the sprite
//										last changed it's direction of movement.
//			m_iAnimationOffset,	How many tiles to skip over to display the
//										sprite in it's current animation sequence.
//			m_eDirection,			Direction in which sprite is traveling.
//			m_fx,						New horizontal position on screen.
//			m_fy,						New vertical position on screen.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectOctorock::update()
{
	BEGIN_ERRORHANDLER_GAME

	
	// Obtain the multipler to use for all movement updates.
	float fFrameFactor = m_pGameState->getFrameFactor();

	// Reset movement.
	m_fMoveOffsetX = 0;
	m_fMoveOffsetY = 0;


	// ----- Animation update. -----

	// Update the animation time with the time that passed.
	m_lAnimationTime += m_pGameState->getTimeLapseMS();

		// Reset the animation timer every 500ms (1/2 second)
	if (m_lAnimationTime > lANIMATION_WALK_MAX)
		m_lAnimationTime -= lANIMATION_WALK_MAX;

	// Set which of the animate sprites to draw based on how much time has
	// passed.
	if (m_lAnimationTime > (lANIMATION_WALK_MAX / 2) )
		m_iAnimateOffset =1;				// Nose out sprite
	else
		m_iAnimateOffset =0;				// Nose in sprite (no offset)


	// ----- State update. -----

	if (m_eState & FLASHING)		// Update flashing if necessary.
		flash();

	if (m_eState & RECOILING)		// Update recoiling if necessary.
		recoil();

	if (m_eState & FROZEN)			// Update Frozen status if necessary.
		thaw();


	// Check if it's okay to move sprite.
	if (m_bMoving && !(m_eState & FROZEN) && !(m_eState & RECOILING))
	{	
		// Determine how far from current position sprite will move.
		switch(m_eDirection)
		{
			case UP:
				m_fMoveOffsetY = m_fSpeed * fFrameFactor;
				break;

			case RIGHT:
				m_fMoveOffsetX = m_fSpeed * fFrameFactor;
				break;

			case DOWN:
				m_fMoveOffsetY = -(m_fSpeed * fFrameFactor);
				break;

			case LEFT:
				m_fMoveOffsetX = -(m_fSpeed * fFrameFactor);
				break;
			
			default:
				THROW_GAME_INT("Octorok direction value is invalid",
					m_eDirection);
				break;
		}
	}

	// Perform actual movement using offsets.
	m_fx += m_fMoveOffsetX;
	m_fy += m_fMoveOffsetY;


	// Determine if the new x,y location puts this Octorock colliding with a
	// solid piece of the screen.
	m_pGameState->getCollidingSprites(this, SPRITE_COLLIDE_SOLID);

	// Perform collision checks against player weapons.
	m_pGameState->getCollidingSprites(this, static_cast<eSPRITE_COLLIDE>
		(SPRITE_COLLIDE_PLAYER | SPRITE_COLLIDE_SHOT) );

	// Check playfield boundries.
	if ( isOutOfPlayfield() )
		moveBack();


	// ----- Choose new direction to move. -----

	// Add to the total amount of time since Octorock last changed direction.
	m_fMoveTime += m_pGameState->getTimeLapseMS();

	// Determine if enough time has passed that movement should change.
	if ((m_fMoveTime > 500.0f) &&  !(m_eState & RECOILING) )
	{
		// Randomly choose a direction.

		m_bMoving = true;	// Set to move.

		// Pick a random direction to move.
		//
		// If Octorock is not being forced due to a collision, it will
		// pick one of 10 ways to move.  5 of these are to move in the 
		// current direction.
		switch(rand() % ((m_fMoveTime==TIME_FOR_A_NEW_DIRECTION) ? 5 : 10) )
		{
			case 0:	m_eDirection = UP;		break;
			case 1:	m_eDirection = RIGHT;	break;
			case 2:	m_eDirection = DOWN;		break;
			case 3:	m_eDirection = LEFT;		break;
			case 4:
				m_bMoving = false;
				break;
			default:
				// A new direction is not picked, and so whatever direction
				// it held before, will be it's new direction.
				break;
		} 

		m_fMoveTime = 0.0f;	// Reset time since new direction was picked.

	}	// END if time to move

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Draws Octorok
//
//////////////////////////////////////////////////////////////////////////////
void cObjectOctorock::draw()
{
	BEGIN_ERRORHANDLER_GAME

	// Draw Octorock.
	m_pTiles->draw(m_fx, m_fy, Z_PLANE_CHARACTER,
		ID_OCTOROCK +					// 1. Base offset for octorock tiles
			(m_eDirection * 2) +		// 2. Modified by a (valid) direction
			m_iAnimateOffset,			// 3. Modified by current animation step
		m_yColorSet);

	END_ERRORHANDLER_GAME
}
// vi:set ts=3:
