//////////////////////////////////////////////////////////////////////////////
//
//	OBJECTProjectileFire.cpp
//	Implementation for the cObjectProjectileFire class.
//
//	-	Encapsulation for the (non-moving) sword Link uses.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////


#include "objectProjectileFire.h"

const float fPROJECTILE_FIRE_SPEED = 0.7f;
const long lPROJECTILE_ANIMATION_COUNTDOWN = 150;
const long lPROJECTILE_FIRE_MOVE_COUNTDOWN = 400;
const long lPROJECTILE_FIRE_LIFE_COUNTDOWN = 1800;

//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
// -	The sword is a brain-dead object which is not affected by other
//		sprites.  It's constantly reset by the "Link" object, each time
//		with a possibly different orientation and location on the screen.
//		
//	ARGS:	pGameState,		Game state object.
//
//////////////////////////////////////////////////////////////////////////////
cObjectProjectileFire::cObjectProjectileFire(cGameState* pGameState)		
{
	// Set game state object.
	m_pGameState = pGameState;

	m_fSpeed = fPROJECTILE_FIRE_SPEED;
	m_lMoveTime = lPROJECTILE_FIRE_MOVE_COUNTDOWN;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(SPRITE_SET_ITEMS);

	// Don't auto-delete when sword "dies" (stops swinging).
	m_bDeleteable = false;

	// Set defaults.
	cSprite::reset();

	// Link starts with a green sword.
	m_yColorSet = m_yColorSetBeforeFlashing = COLOR_SET_ITEMS_RED;

	// Override defaults...
	m_iDamage = 1;			// Amount of damage sprite inflicts.
	m_iHitPoints = 0;		// Fire starts off "dead".
	m_iAnimateOffset = 0;
	m_eCollisionMask = static_cast<eSPRITE_COLLIDE>
		(SPRITE_COLLIDE_PLAYER | SPRITE_COLLIDE_SHOT);

	// set the ID
	m_iInventoryDisplayID = ID_ITEM_ROD_SHOT + 1;
	// Flying sword gives no delay to use.
	m_lSwingDelay = 0;

	// Dimensions of sword.
	m_fWidth	= m_pTiles->getWidth();
	m_fHeight= m_pTiles->getHeight();

	// Border around sprite used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectProjectileFire::~cObjectProjectileFire()
{
}


//////////////////////////////////////////////////////////////////////////////
//
//	Override default implementation to call an enemy's hit.
//
//	ARGS:	pSprite,	The sprite which may be colliding with the sword.
//
//	POST:	If pSprite is an enemy and it collides then hit() is called on it.
//
//	RETURNS:	true,		Target sprite is an enemy and overlaps this sprite.
//				false,	Fire is not interacting with an enemy and/or they are
//							not colliding with each other.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectProjectileFire::collide(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// Make sure target sprite is an enemy, then check if it's colliding.
	if (	(pSprite->getCollisionMask() & SPRITE_COLLIDE_ENEMY) &&
			cSprite::collide(pSprite) )
	{
		pSprite->hit(this);	// Hit the enemy sprite.
		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Called when a sword swing is first performed.
//	-	Called to reset the location of the sword.
//
//	PRE:	m_fx,				Screen horizontal coordinate in pixels of Link's
//								location.
//			m_fy,				Screen vertical coordinate in pixels of Link's
//								location.
//			m_eDirection,	Which direction to make the sword facing.
//
//	POST:	m_fx,				Adjusted horizontal position.
//			m_fy,				Adjusted vertical position.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileFire::reset()
{
	m_iHitPoints = 0;
	m_eState = INACTIVE;
	m_fSpeed = fPROJECTILE_FIRE_SPEED;
	m_lMoveTime = lPROJECTILE_FIRE_MOVE_COUNTDOWN;
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Overload, which uses the current direction.
//	-	Assume sword position is moving, but does not perform a full reset
//		since the timer (ie: the "swing") doesn't begin again.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileFire::reset(float fx, float fy)
{ 
	BEGIN_ERRORHANDLER_GAME

		
	// Set the new location.
	m_fx = fx;
	m_fy = fy;

		// Set the direction of the sword.
	switch(m_eDirection)
	{
		// If direction is valid, store it in member variable and adjust
		// sword based on direction.
		case DOWN:
			m_fy -= 14.0f;
			break;

		case UP:	
			m_fy += 10.0f;
			break;

		case LEFT:
			m_fx -= 12.0f;
			m_fy -= 2.0f;
			break;

		case RIGHT:	
			m_fx += 12.0f;
			m_fy -= 2.0f;
			break;

		// Anything else is illegal which should cause an error.
		default:
		{
			// Construct an error message and throw.
			THROW_GAME_INT("Only up, down, left, and right are legal directions "
				"values for a sword to be reset facing.", m_eDirection);
			break;
		}
	} // END direction of swing
	
	reset();	// Perform the reset.

	END_ERRORHANDLER_GAME

}


//////////////////////////////////////////////////////////////////////////////
//
//	Sets the sword swinging.
//
//	ARGS:	fx,			Horizontal location of Link when swinging.
//			fy,			Vertical location of Link when swinging.
//			eDirection,	Direction of Link during swing.
//
//	POST:	m_fx,				Fire's new hortizontal location.
//			m_fy,				Fire's new vertical location.
//			m_eDirection,	Direction sword is facing.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectProjectileFire::activate(float fx, float fy, eDIRECTION eDirection)
{ 
	BEGIN_ERRORHANDLER_GAME

	if( this->isDead() )
	{
		// Set the location and direction of the sword.
		m_eDirection = eDirection;			
		reset(fx, fy);

		m_lAnimationTime = 0;		// Reset animation time holder. 
		m_iHitPoints = lPROJECTILE_FIRE_LIFE_COUNTDOWN;	

		m_eState = ACTIVE;							// Force to be active.
		m_pGameState->addActiveSprite(this);	// Inject into active sprites.
		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Updates the movement of the sword.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileFire::update()
{
	BEGIN_ERRORHANDLER_GAME

	float fFrameFactor = m_pGameState->getFrameFactor();

	if ( !this->isDead() )
	{
		switch (m_eDirection)
		{
		case DOWN:
			m_fy -= m_fSpeed * fFrameFactor;
			break;
		case UP:
			m_fy += m_fSpeed * fFrameFactor;
			break;
		case LEFT:
			m_fx -= m_fSpeed * fFrameFactor;
			break;
		case RIGHT:
			m_fx += m_fSpeed * fFrameFactor;
			break;
		}

			// Update the animation time with the time that passed.
		m_lAnimationTime += m_pGameState->getTimeLapseMS();

			// Reset the animation timer every 500ms (1/2 second)
		if (m_lAnimationTime > lPROJECTILE_ANIMATION_COUNTDOWN)
			m_lAnimationTime -= lPROJECTILE_ANIMATION_COUNTDOWN;

		// Set which of the animate sprites to draw based on how much time has
		// passed.
		if (m_lAnimationTime > (lPROJECTILE_ANIMATION_COUNTDOWN / 2) )
			m_iAnimateOffset =1;	// Fire frame 1
		else
			m_iAnimateOffset =0;	// Fire frame 0

		if ( (m_lMoveTime -= m_pGameState->getTimeLapseMS()) <= 0 )
		{
			m_fSpeed = 0;
		}

		m_iHitPoints -= m_pGameState->getTimeLapseMS();
	}


	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// Draw the fire in flight.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileFire::draw()
{
	BEGIN_ERRORHANDLER_GAME

	// Draw Fire.
	m_pTiles->draw(m_fx, m_fy, Z_PLANE_SHOT,
		ID_ITEM_FIRE + m_iAnimateOffset,
		m_yColorSet);

	END_ERRORHANDLER_GAME
}

// vi:set ts=3:
