//////////////////////////////////////////////////////////////////////////////
//
//	OBJECTProjectileMagic.cpp
//	Implementation for the cObjectProjectileMagic class.
//
//	-	Encapsulation for the (non-moving) sword Link uses.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "objectProjectileMagic.h"

const float fPROJECTILE_MAGIC_SPEED = 2.0f;

//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
// -	The sword is a brain-dead object which is not affected by other
//		sprites.  It's constantly reset by the "Link" object, each time
//		with a possibly different orientation and location on the screen.
//		
//	ARGS:	pGameState,		Game state object.
//
//////////////////////////////////////////////////////////////////////////////
cObjectProjectileMagic::cObjectProjectileMagic(cGameState* pGameState)		
{
	// Set game state object.
	m_pGameState = pGameState;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(SPRITE_SET_ITEMS);

	// Don't auto-delete when sword "dies" (stops swinging).
	m_bDeleteable = false;

	// Set defaults.
	cSprite::reset();

	// Link starts with a green sword.
	m_yColorSet = m_yColorSetBeforeFlashing = COLOR_SET_ITEMS_BLUE;

	// Override defaults...
	m_iDamage = 1;			// Amount of damage sprite inflicts.
	m_iHitPoints = 0;		// Magic starts off "dead".
	m_eCollisionMask = static_cast<eSPRITE_COLLIDE>
		(SPRITE_COLLIDE_PLAYER | SPRITE_COLLIDE_SHOT);

	// set the ID
	m_iInventoryDisplayID = ID_ITEM_ROD_SHOT + 1;
	// Flying sword gives no delay to use.
	m_lSwingDelay = 0;

	// Set the speed at which it travels
	m_fSpeed = fPROJECTILE_MAGIC_SPEED;

	// Dimensions of sword.
	m_fWidth	= m_pTiles->getWidth();
	m_fHeight= m_pTiles->getHeight();

	// Border around sprite used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectProjectileMagic::~cObjectProjectileMagic()
{
}


//////////////////////////////////////////////////////////////////////////////
//
//	Override default implementation to call an enemy's hit.
//
//	ARGS:	pSprite,	The sprite which may be colliding with the sword.
//
//	POST:	If pSprite is an enemy and it collides then hit() is called on it.
//
//	RETURNS:	true,		Target sprite is an enemy and overlaps this sprite.
//				false,	Magic is not interacting with an enemy and/or they are
//							not colliding with each other.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectProjectileMagic::collide(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// Make sure target sprite is an enemy, then check if it's colliding.
	if (	(pSprite->getCollisionMask() & SPRITE_COLLIDE_ENEMY) &&
			cSprite::collide(pSprite) )
	{
		pSprite->hit(this);	// Hit the enemy sprite.
		m_iHitPoints = 0;		// Dies if collides

		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Called when a sword swing is first performed.
//	-	Called to reset the location of the sword.
//
//	PRE:	m_fx,				Screen horizontal coordinate in pixels of Link's
//								location.
//			m_fy,				Screen vertical coordinate in pixels of Link's
//								location.
//			m_eDirection,	Which direction to make the sword facing.
//
//	POST:	m_fx,				Adjusted horizontal position.
//			m_fy,				Adjusted vertical position.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileMagic::reset()
{
	m_iHitPoints = 0;
	m_eState = INACTIVE;
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Overload, which uses the current direction.
//	-	Assume sword position is moving, but does not perform a full reset
//		since the timer (ie: the "swing") doesn't begin again.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileMagic::reset(float fx, float fy)
{ 
	BEGIN_ERRORHANDLER_GAME
	
		// Set the new location.
	m_fx = fx;
	m_fy = fy;

	reset();	// Perform the reset.

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Sets the sword swinging.
//
//	ARGS:	fx,			Horizontal location of Link when swinging.
//			fy,			Vertical location of Link when swinging.
//			eDirection,	Direction of Link during swing.
//
//	POST:	m_fx,				Magic's new hortizontal location.
//			m_fy,				Magic's new vertical location.
//			m_eDirection,	Direction sword is facing.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectProjectileMagic::activate(float fx, float fy, eDIRECTION eDirection)
{ 
	BEGIN_ERRORHANDLER_GAME

	if( this->isDead() )
	{
		// Set the location and direction of the sword.
		m_eDirection = eDirection;	
		m_fx = fx;
		m_fy = fy;
		reset();

		m_lAnimationTime = 0;		// Reset animation time holder. 
		m_iHitPoints = 1;							// Enough to keep sprite alive.	

		m_eState = ACTIVE;							// Force to be active.
		m_pGameState->addActiveSprite(this);	// Inject into active sprites.
		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Updates the movement of the sword.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileMagic::update()
{
	
	BEGIN_ERRORHANDLER_GAME

	float fFrameFactor = m_pGameState->getFrameFactor();

	if ( !this->isDead() )
	{
		switch (m_eDirection)
		{
		case DOWN:
			m_fy -= m_fSpeed * fFrameFactor;
			break;
		case UP:
			m_fy += m_fSpeed * fFrameFactor;
			break;
		case LEFT:
			m_fx -= m_fSpeed * fFrameFactor;
			break;
		case RIGHT:
			m_fx += m_fSpeed * fFrameFactor;
			break;
		}

		// Flying magic shot flashes
		flash();

			// Dies if leaves play.
		if ( isOutOfPlayfield() )
		{
			m_iHitPoints = 0;
		}
	}


	END_ERRORHANDLER_GAME
}


/////////////////////////////////////////////////////////////////////////////
//
//	Make the Projectile Sword flash
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileMagic::flash()
{
	BEGIN_ERRORHANDLER_GAME

		// Adjust when the color changes.
	m_lFlashChangeTime -= m_pGameState->getTimeLapseMS();
		// If the flash time is exceeded...
	if (m_lFlashChangeTime < 0)
	{
		// Reset the the flash timer and jump to the next color.
		m_lFlashChangeTime = lFLASH_CHANGE_TIME;	
		m_yColorSet++;
			// If color is out of range reset it back to the first color.
		if (m_yColorSet > m_pTiles->getLastColorSet())
			m_yColorSet = m_pTiles->getFirstColorSet();
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// Draw the sword in flight.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectProjectileMagic::draw()
{
	BEGIN_ERRORHANDLER_GAME

		// Draw Magic.
	m_pTiles->draw(m_fx, m_fy, Z_PLANE_SHOT, ID_ITEM_ROD_SHOT + m_eDirection, 
		m_yColorSet);

	END_ERRORHANDLER_GAME
}

// vi:set ts=3:
