//////////////////////////////////////////////////////////////////////////////
//
//	OBJECTStillSword.cpp
//	Implementation for the cObjectStillSword class.
//
//	-	Encapsulation for the (non-moving) sword Link uses.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "objectStillSword.h"

const long fLINK_SWING_SWORD_COUNTDOWN = 180; // time for a swing to complete.


//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
// -	The sword is a brain-dead object which is not affected by other
//		sprites.  It's constantly reset by the "Link" object, each time
//		with a possibly different orientation and location on the screen.
//		
//	ARGS:	pGameState,		Game state object.
//
//////////////////////////////////////////////////////////////////////////////
cObjectStillSword::cObjectStillSword(cGameState* pGameState)		
{

	m_pProjectileSword = new cObjectProjectileSword(pGameState);

	// Set game state object.
	m_pGameState = pGameState;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(SPRITE_SET_ITEMS);

	// Don't auto-delete when sword "dies" (stops swinging).
	m_bDeleteable = false;

	// Set defaults.
	cSprite::reset();


	// Link starts with a green sword.
	m_yColorSet = m_yColorSetBeforeFlashing = COLOR_SET_ITEMS_BLUE;

	// Override defaults...
	m_iDamage = 1;			// Amount of damage sprite inflicts.
	m_iHitPoints = 0;		// Sword starts off "dead".
	m_eCollisionMask = static_cast<eSPRITE_COLLIDE>
		(SPRITE_COLLIDE_PLAYER | SPRITE_COLLIDE_SHOT);

	// set the ID
	m_iInventoryDisplayID = ID_ITEM_SWORD + 1;
	// Time it take for a swing of the sword to complete.
	m_lSwingDelay = fLINK_SWING_SWORD_COUNTDOWN;

	// Dimensions of sword.
	m_fWidth	= m_pTiles->getWidth();
	m_fHeight= m_pTiles->getHeight();

	// Border around sprite used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cObjectStillSword::~cObjectStillSword()
{
	delete m_pProjectileSword;
}


//////////////////////////////////////////////////////////////////////////////
//
//	Override default implementation to call an enemy's hit.
//
//	ARGS:	pSprite,	The sprite which may be colliding with the sword.
//
//	POST:	If pSprite is an enemy and it collides then hit() is called on it.
//
//	RETURNS:	true,		Target sprite is an enemy and overlaps this sprite.
//				false,	Sword is not interacting with an enemy and/or they are
//							not colliding with each other.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectStillSword::collide(cSprite* pSprite)
{
	// Make sure target sprite is an enemy, then check if it's colliding.
	if (	(pSprite->getCollisionMask() & SPRITE_COLLIDE_ENEMY) &&
			cSprite::collide(pSprite) )
	{
		pSprite->hit(this);	// Hit the enemy sprite.
		return true;
	}

	return false;
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Called when a sword swing is first performed.
//	-	Called to reset the location of the sword.
//
//	PRE:	m_fx,				Screen horizontal coordinate in pixels of Link's
//								location.
//			m_fy,				Screen vertical coordinate in pixels of Link's
//								location.
//			m_eDirection,	Which direction to make the sword facing.
//
//	POST:	m_fx,				Adjusted horizontal position.
//			m_fy,				Adjusted vertical position.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectStillSword::reset()
{		
	m_iHitPoints = 0;
	m_eState = INACTIVE;
	m_pProjectileSword->reset();
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Overload, which uses the current direction.
//	-	Assume sword position is moving, but does not perform a full reset
//		since the timer (ie: the "swing") doesn't begin again.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectStillSword::reset(float fx, float fy)
{ 
	BEGIN_ERRORHANDLER_GAME

	// Set the new location.
	m_fx = fx;
	m_fy = fy;
	switch(m_eDirection)
	{
		// If direction is valid, store it in member variable and adjust
		// sword based on direction.
		case DOWN:
			m_fy -= 14.0f;
			break;

		case UP:	
			m_fy += 10.0f;
			break;

		case LEFT:
			m_fx -= 12.0f;
			m_fy -= 2.0f;
			break;

		case RIGHT:	
			m_fx += 12.0f;
			m_fy -= 2.0f;
			break;

		// Anything else is illegal which should cause an error.
		default:
		{
			// Construct an error message and throw.
			THROW_GAME_INT("Only up, down, left, and right are legal directions "
				"values for a sword to be reset facing.", m_eDirection);
			break;
		}
	} // END direction of swing
	

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Sets the sword swinging.
//
//	ARGS:	fx,			Horizontal location of Link when swinging.
//			fy,			Vertical location of Link when swinging.
//			eDirection,	Direction of Link during swing.
//
//	POST:	m_fx,				Sword's new hortizontal location.
//			m_fy,				Sword's new vertical location.
//			m_eDirection,	Direction sword is facing.
//
//////////////////////////////////////////////////////////////////////////////
bool cObjectStillSword::activate(float fx, float fy, eDIRECTION eDirection)
{ 
	BEGIN_ERRORHANDLER_GAME

	// Set the location and direction of the sword.
	m_eDirection = eDirection;
	reset(fx, fy);										// Call the reset.
	m_lAnimationTime = fLINK_SWING_SWORD_COUNTDOWN;		// Reset animation time holder. 
	m_iHitPoints = 1;							// Enough to keep sprite alive.

	m_eState = ACTIVE;							// Force to be active.
	m_pGameState->addActiveSprite(this);	// Inject into active sprites.

	return true;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Updates the movement of the sword.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectStillSword::update()
{
	BEGIN_ERRORHANDLER_GAME

	// Animation time counts down for sword.  If 0, it's no longer moving
	// through it's motions and can be removed from the active sprites.  (This
	// is done by making it's hitpoints 0.)
	if (m_lAnimationTime > 0)
	{
		// Update the animation time with the time that passed.
		m_lAnimationTime -= m_pGameState->getTimeLapseMS();

		// Snap to 0 if animation timer is done.
		if (m_lAnimationTime < 0)
			m_lAnimationTime = 0;
	}
	else if( !isDead() )
	{
			// If link is at full hit points, activate the projectile
		if ( m_pGameState->getLink()->isAtFullHitPoints() )
		{
			m_pProjectileSword->activate(m_fx, m_fy, m_eDirection);
		}	
			m_iHitPoints = 0;			// Set to be removed from active sprites.
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Draw the sword being swung.
//
//////////////////////////////////////////////////////////////////////////////
void cObjectStillSword::draw()
{
	// Draw Sword.
	m_pTiles->draw(m_fx, m_fy, Z_PLANE_SHOT, ID_ITEM_SWORD + m_eDirection, 
		m_yColorSet);
}

// vi:set ts=3:
