//////////////////////////////////////////////////////////////////////////////
//
//	screen.cpp
//	Implementation for the cScreen class.
//
//	-	Reads data for screen and populates collection with sprites which
//		make up the background, enemies, items, etc... for the screen.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "screen.h"             // Defines for this class.
#include "GameException.h"      // Exception handling.
#include "spriteFactory.h"      // Class factory for creating sprites.
#include "eventFactory.h"       // Class factory for creating events.
#include "screenTemplate.h" // A collection of sprites used for many screens.
#include "spriteData.h"         // Reads data from a file to create sprites.
#include "eventData.h"          // Reads data from a file to create events.

// Defines for forward delcarations.
#include "sprite.h"
#include "event.h"
#include "map.h"
#include "gameState.h"

//////////////////////////////////////////////////////////////////////////////
//
//	-	CONSTRUCTOR
//	-	Loads a screen and constructs the sprites associated with it.
//
//		Offset		Size	Info
//		------		----	-------------------------------------------
//		0				1		x = Width of the screen in screen tiles.
//		1				1		y = Height of the screen in screen tiles.
//		2				1		Type of screen (0 = default)
//		3				1		o = # of object sprites in screen
//		4				1		e = # of events/portals for screen
//		5				1		ID of the screen template utilized
//		6				1		0x00, reserved for future
//		7				1		0x00, reserved for future
//		8				x*y	Sprites, screen tiles data
//		8+(x*y)		o		Sprites, characters data
//		8+(x*y)+o	e		Events data
//
//	ARGS:	pGameState,		An object containing the "state" of the game.
//			pMap,				An object containing the currently developing map.
//			pFileHandle,	An open file handle, pointing to a screen object.
//
//	POST:	m_pvSprites,	Contains sprites to draw.
//
//	THROWS:	cGameException,	On an error reading screen data.
//
//////////////////////////////////////////////////////////////////////////////
cScreen::cScreen(cGameState* pGameState, cMap* pMap, FILE* pFileHandle)
{
	BEGIN_ERRORHANDLER_GAME

	char					szScreenHeader[8];// Holds header data for a screen.
	BYTE					yCharacters;		// # of character sprites in screen.

	cScreenTemplate*	pScreenTemplate;	// Temporary to read in template information;

	cSpriteData*		pSpriteData;		// Temporary to read in sprite data.
	cSpriteFactory*	pSpriteFactory;	// Object to change data to sprites.
	cSprite*				pSprite;				// Temporary to construct sprites.
	int					iSpriteLoop;		// To loop through sprite creation.

	cEventData*			pEventData;			// Temporary to read in event data.
	cEventFactory*		pEventFactory;		// Object to change data to events.
	cEvent*				pEvent;				// Temporary to construct events.
	int					iEventLoop;			// To loop through event creation.
	

	// Make sure the pointer to a game state is legal.
	if (pGameState == NULL)
		THROW_GAME("NULL game state pointer passed into screen constructor.");

	
	// Make sure the pointer to the parent map is legal.
	if (pMap == NULL)
		THROW_GAME("NULL map pointer passed into screen constructor.");
		
	m_pMap = pMap;	// Grab a pointer to the map


	// Make sure file handle is legal.
	if (pFileHandle == NULL)
		THROW_GAME("Empty file handle given to screen.");

	
	// Read in header values for this screen object.
	if(	fread(&szScreenHeader, 1, sizeof(szScreenHeader), pFileHandle) != 
			sizeof(szScreenHeader)	)
		THROW_GAME("Unable to properly read screen header.");


	// Fill in member variables from the blob of screen header data.
	m_yWidth		= szScreenHeader[0];	// Width of screen (in tiles).
	m_yHeight	= szScreenHeader[1];	// Height of screen (in tiles).
	m_yType		= szScreenHeader[2];	// Type of screen (0 being regular).
	yCharacters	= szScreenHeader[3];	// # of character sprites.
	m_yEvents	= szScreenHeader[4];	// # of events.
	m_yTemplateID=szScreenHeader[5];	// Template ID (0 is none)


	// Check that the screen dimensions are valid.
	if ((m_yWidth == 0) || (m_yHeight == 0))
	{
		// At least one of the dimensions is bogus so throw an error.

		char szTemp[10];	// Temporary for converting int to ascii.

		string sErrorMessage("A Screen could not be constructed, due to"
			"bad dimensions: ");
	
		// no itoa() in unix.. dont get any ideas
		sprintf(szTemp, "%i", m_yWidth);
		sErrorMessage += szTemp;
		sErrorMessage += "x";
		sprintf(szTemp, "%i", m_yHeight);
		sErrorMessage += szTemp;

		sprintf(szTemp, "%s", sErrorMessage);
		
		THROW_GAME( szTemp );
	}


	// Initialize the sprite factory...time to make some sprites!
	pSpriteFactory = new cSpriteFactory(pGameState);

	// Events are made very similar to sprites
	pEventFactory = new cEventFactory(pGameState);


	// Check if screen information is kept in a template.	 
	if ( 0 != m_yTemplateID )
	{
		// There is a template ID, so use that template to build this screen.

		// Obtain template object contained in map.
		pScreenTemplate = m_pMap->getTemplate( m_yTemplateID - 1 );

		// Go through each sprite data on the template and create a sprite for the  vector
		for ( int iSpriteData = 0;
				iSpriteData < pScreenTemplate->getSize();
				iSpriteData++ )
		{
			// Create new sprite data object (read data)
			pSpriteData = pScreenTemplate->getSpriteData( iSpriteData );

			//	1.	Send raw sprite data to sprite factory.
			// 2.	Receive a pointer to an allocated sprite, created by the 
			//		sprite factory.
			pSprite = pSpriteFactory->create(*pSpriteData);

			// Add the new sprite to the screen's collection of sprites.
			m_vpSprites.push_back(pSprite);
		}
	}
	else	// If there is no template, create the screen with included sprites
	{

		// Creates sprites for the screen.
		// The first bunch of sprites are what make up the background.
		// The second bunch of sprites are the enemeies/objects.

		for(	iSpriteLoop = 0; 
				iSpriteLoop < (m_yHeight * m_yWidth) + yCharacters;
				iSpriteLoop++)
		{
			// Create new sprite data object (read data)
			pSpriteData = new cSpriteData(pFileHandle);

			//	1.	Send raw sprite data to sprite factory.
			// 2.	Receive a pointer to an allocated sprite, created by the 
			//		sprite factory.
			pSprite = pSpriteFactory->create(*pSpriteData);

			// Add the new sprite to the screen's collection of sprites.
			m_vpSprites.push_back(pSprite);

			// Free the memory held by the sprite data object.
			delete pSpriteData;
		}
	}	


	// Creates events for the screen.
	// Links events to sprites, for those that require it.

	for( iEventLoop = 0; iEventLoop < m_yEvents; iEventLoop++ )
	{
		// Read in data by creating new event data object
		pEventData = new cEventData(pFileHandle);

		// 1. Send raw event data to event factory.
		// 2. Receive a pointer to an allocated event, created by the
		//    event factory
		pEvent = pEventFactory->create(this, *pEventData);

		// Add the new event to the screen's collection of events
		m_vpEvents.push_back(pEvent);

		// Free the memory held by the event data object
		delete pEventData;
	}
	
	delete pEventFactory;	// Free event factory (done making events.)	
	delete pSpriteFactory;	// Free sprite factory (done making sprites.)


	END_ERRORHANDLER_GAME
}

//////////////////////////////////////////////////////////////////////////////
//	DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cScreen::~cScreen()
{
	SPRITE_ITR itrSprite;	// STL iterator to loop through sprites.
	EVENT_ITR itrEvent;		// STL iterator to loop through events


	// Loop through the collection of sprite objects (tiles, enemies, etc...)
	// and free the memory that they held.
	for (itrSprite = m_vpSprites.begin(); itrSprite != m_vpSprites.end(); itrSprite++)
		delete *itrSprite;
	m_vpSprites.clear();	// Free memory allocated to the collection itself.

	// Loop through the collection of events
	// and free the memory that they held.
	for (itrEvent = m_vpEvents.begin(); itrEvent != m_vpEvents.end(); itrEvent++)
		delete *itrEvent;
	m_vpEvents.clear();	// Free memory allocated to the collection itself
}

// Obtains a copy of the sprites used in the screen.
/*inline*/ vector<cSprite*> cScreen::getSprites() {
	return m_vpActiveSprites = m_vpSprites; 
}

// Obtains a copy of the events used in the screen
/*inline*/ vector<cEvent*> cScreen::getEvents() {
	return m_vpEvents; 
}

// Retrieve a pointer to a specific sprite.
/*inline*/ cSprite* cScreen::getSprite(int iSpriteID) {
	return m_vpSprites[ iSpriteID ]; 
}

// Retrieve a pointer to the screen's map
/*inline*/ cMap* cScreen::getMap() {
	return m_pMap; 
}
// vi:set ts=3:
