// screenTemplate.cpp: implementation of the cScreenTemplate class.
//
//////////////////////////////////////////////////////////////////////

#include "screenTemplate.h"

//////////////////////////////////////////////////////////////////////////////
//
//	-	CONSTRUCTOR
//	-	Loads a screen template and constructs the sprites associated with it.
//
//		Offset		Size	Info
//		------		----	-------------------------------------------
//		0				1		Size of the template in screen tiles.
//		1				1		0x00, reserved for future
//		2				1		0x00, reserved for future
//		3				1		0x00, reserved for future
//		4				1		0x00, reserved for future
//		5				1		0x00, reserved for future
//		6				1		0x00, reserved for future
//		7				1		0x00, reserved for future
//
//	ARGS:	pGameState,		An object containing the "state" of the game.
//			pFileHandle,	An open file handle, pointing to a screen template 
//								object.
//
//	POST:	m_pvSprites,	Contains sprites to draw.
//
//	THROWS:	cGameException,	On an error reading screen template data.
//
//////////////////////////////////////////////////////////////////////////////
cScreenTemplate::cScreenTemplate(cGameState* pGameState, FILE* pFileHandle)
{
	BEGIN_ERRORHANDLER_GAME

	char					szTemplateHeader[8];// Holds header data for a template.

	cSpriteData*		pSpriteData;		// Temporary to read in sprite data.
	int					iSpriteLoop;		// To loop through sprite creation.
	
	// Make sure the pointer to a game state is legal.
	if (pGameState == NULL)
	{
		// Bad pointer.  Throw with an appropriate error message.
		THROW_GAME("NULL game state pointer passed into screen template constructor.");
	}

	// Make sure file handle is legal.
	if (pFileHandle == NULL)
	{
		// Bad file handle.  Throw with an appropriate error message.
		THROW_GAME("Empty file handle given to screen template.");
	}

	// Read in header values for this screen object.
	if(	fread(&szTemplateHeader, 1, sizeof(szTemplateHeader), pFileHandle) != 
			sizeof(szTemplateHeader)	)
	{
		// Unable to read screen header.
		THROW_GAME("Unable to properly read screen template header.");
	}

	m_ySize		= szTemplateHeader[0];	// Number of tiles in template (other values unused as of now).


	// Check that the screen template size is valid.
	if (m_ySize == 0)
	{
		// Size is bogus so throw an error.

		char szTemp[10];	// Temporary for converting int to ascii.

		string sErrorMessage("A Screen template could not be constructed, due to"
			"bad dimensions: ");

		sprintf(szTemp, "%i", m_ySize);
		sErrorMessage += szTemp;
		sErrorMessage += " size";
		
		THROW_GAME( sErrorMessage.c_str() );
	}


	// Creates sprite data for the screen template.
	for(	iSpriteLoop = 0; 
			iSpriteLoop < m_ySize;
			iSpriteLoop++)
	{
		// Create new sprite data object (read data)
		pSpriteData = new cSpriteData(pFileHandle);

		// Add the new sprite data to the screen's collection of sprite data.
		m_vpSpriteData.push_back(pSpriteData);
	}

	END_ERRORHANDLER_GAME
}

cScreenTemplate::~cScreenTemplate()
{
	SPRITEDATA_ITR itrSpriteData;	// STL iterator to loop through sprites.

	// Loop through the collection of sprite objects (tiles, enemies, etc...)
	// and free the memory that they held.
	for (itrSpriteData = m_vpSpriteData.begin(); itrSpriteData != m_vpSpriteData.end(); itrSpriteData++)
		delete *itrSpriteData;

	m_vpSpriteData.clear();	// Free memory allocated to the collection itself.
}

// Obtains a copy of the sprites used in the template.
/*inline*/ vector<cSpriteData*> cScreenTemplate::getAllSpriteData()
{ return m_vpSpriteData; }

// Obtains a pointer to a specific sprite in the template
/*inline*/ cSpriteData* cScreenTemplate::getSpriteData( int iID ) 
{ return m_vpSpriteData[ iID ]; }

// Obtains the number of sprites in the template
/*inline*/ int cScreenTemplate::getSize() { return m_ySize; }
// vi:set ts=3:
