//////////////////////////////////////////////////////////////////////////////
//
//	sprite.cpp
//	Implementation for the cSprite class.
//
//	-	Base class to which all sprites must sub-class.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "sprite.h"             // Defines for this class.
#include "GameException.h"      // Exception handling.
#include "objectSparkle.h"       // Creates sparkles on death.

// Defines for forward delcarations.
#include "spriteData.h"
#include "gameState.h"
#include "event.h"
#include "tiles.h"

//////////////////////////////////////////////////////////////////////////////
//	CONSTANTS
//////////////////////////////////////////////////////////////////////////////

// ???DAC - shouldnt these be static class members?
const long	lHITFLASH_TIME	= 1000;	// Time (MS) spent flashing when hit.
const long	lRECOIL_TIME	= 100;	// Max time (MS) a recoil occurs.
const float fRECOIL_SPEED	= 4.0f;	// Speed of recoils

//////////////////////////////////////////////////////////////////////////////
//
// CONSTRUCTOR
//
//	ARGS:	pGameState,		Game state object.
//			oSpriteData,	Reference to a populated sprite data object.
//
//////////////////////////////////////////////////////////////////////////////
cSprite::cSprite(cGameState* pGameState, cSpriteData& oSpriteData)
{
	// Set game state object.
	m_pGameState = pGameState;

	// Set graphics set used by this sprite.
	m_pTiles = pGameState->getSpriteSetTiles(oSpriteData.getSpriteSetID());

	// By default sprites are not auto-deleted when killed.
	m_bDeleteable = false;

	// Set members whose values change due to game-play.
	reset();

	// Common sprite attributes read from data file.
	m_yColorSet = oSpriteData.getColorSet();				// Color set.
	m_yColorSetBeforeFlashing = m_yColorSet;				// ...
	m_eCollisionMask = oSpriteData.getCollideMask();	// Collision behavior.
	m_eSpawnType = oSpriteData.getSpawnType();			// How sprite spawns.
	m_fx = oSpriteData.getX();									// Starting X position
	m_fy = oSpriteData.getY();									// Starting Y position
	m_eZ = oSpriteData.getZ();									// Z-Plane


	// Dimensions of the sprite come from the tiles.
	m_fWidth	=	m_pTiles->getWidth();
	m_fHeight=	m_pTiles->getHeight();


	// Border around sprite used when determining collisions.
	m_fEpsilonX = m_fWidth - fEPSILON_BORDER;
	m_fEpsilonY = m_fHeight - fEPSILON_BORDER;
}


//////////////////////////////////////////////////////////////////////////////
// DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cSprite::~cSprite()
{
	// Forget locations to other objects.
	m_pGameState	= NULL;
	m_pTiles			= NULL;
	m_pEvent			= NULL;

	// Fill common members with invalid values.
	m_fx		= -999.0f;
	m_fy		= -999.0f;
	m_fWidth	= -999.0f;
	m_fHeight= -999.0f;
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Sets non-critical members variables to defaults.
//	-	Any variables requiring external parameters to be set should be set
//		in the constructor.
//
//////////////////////////////////////////////////////////////////////////////
void cSprite::reset()
{
	// Game play attributes...
	m_eDirection		= DOWN;		// Facing down.
	m_fSpeed			= 0.0f;		// Sprite doesn't move.
	m_iHitPoints		= 1;		// Number of hit points until sprite dies.
	m_iDamage			= 0;		// Amount of damage sprite inflicts.
	m_eState			= ACTIVE;	// It's active.

	// Timers...
	m_lAnimationTime	= 0;		// Time until next animation step change.
	m_lRecoilTime		= 0;		// Not recoiling (being damaged).	
	m_lFlashTime		= 0;		// Not flashing.
	m_lFlashChangeTime= 0;		// Time until next color for flash efx.
	m_lFrozenTimer		= 0;		// Not frozen.

	// Offsets...
	m_iAnimateOffset	= 0;		// Start on first frame of animation.

	// Position/Collision related...
	m_fMoveOffsetX		= 0.0f;
	m_fMoveOffsetY		= 0.0f;

	// Connected event defaults to non-existent.
	// ???TMH - Ack this may not work long term, because reset() is called
	//				once a screen.  Check and be sure this (the base class's)
	//				reset() is only called at startup for sprites which may
	//				hold an event.
	m_pEvent = NULL;
}


//////////////////////////////////////////////////////////////////////////////
//
// -	Checks if another sprite's boundries falls within this sprite's area.
//	-	The diagram below explains how collision is detected (if you don't have
//		tabs set to 3 non-destructive spaces, it's going to look like ass):
//
//                     
//			[----------------] Epsilon X
//			 ___________________ 
// 		|                   |	
//		__	|  ...............  |	The solid line represent's the target 
//		!	|  :             :  |	sprite's true boundries.
//		!	|  :             :  |	
//		!	|  :             :  |	The dotted line represent's the target 
//		!	|  :             :  |	sprite's collision boundry (which is just a 
//		!	|  :             :  |	little smaller than it's true boundry.)
//		!	|  :             :  |	
//		!	|  :             :  |	If any of the target sprite's dotted boundry
//		!	|  :.............:  |	is in this sprite's solid boundry, then a
//		!	|                   |	collision has occurred.
//		!	|___________________|
//		~~	0,0
//	Epsilon Y
//
//
//	ARGS:	pSprite,	The target sprite to check against.
//
//	RETURNS:	true,		Target sprite overlaps this sprite.
//				false,	Target sprite does not overlaps this sprite, or target
//							sprite and this sprite are the same sprite.
//
//////////////////////////////////////////////////////////////////////////////
bool cSprite::collide(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// Leave routine if...
	if (	(pSprite == this) ||			// sprite tries to check against itself.
			(m_eState == INACTIVE)	)	// this is an inactive sprite.
		return false;

	
	// Check the Epsilon adjusted sides of the target sprite against the
	// sides of this sprites.
	if (	((pSprite->getX() - m_fEpsilonX) < m_fx) &&
			((pSprite->getY() - m_fEpsilonY) < m_fy) &&
			(m_fx < (pSprite->getX() + pSprite->getEpsilonX() )) &&			
			(m_fy < (pSprite->getY() + pSprite->getEpsilonY() ))		)
	{
		return true;
	}

	return false;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
// Moves the sprite back to it's previous location on a collision.
//
//	PRE:	m_fx & m_fy	Have moved to the new updated location.
//			m_fMoveOffsetX & m_fMoveOffsetY contains offsets used for move.
//
//	POST:	m_fx & m_fy,	Contain the old location.
//			m_fMoveOffsetX & m_fMoveOffsetY are zero.
//
//////////////////////////////////////////////////////////////////////////////
void cSprite::moveBack()
{
	// Revert back to previous location values.
	m_fx -= m_fMoveOffsetX;	
	m_fy -= m_fMoveOffsetY;

	// Zero out offsets.
	m_fMoveOffsetX = 0.0f;
	m_fMoveOffsetY = 0.0f;
}


//////////////////////////////////////////////////////////////////////////////
//
//	The sprite has been "hit" (standard touching) in a collision.
//
//	ARGS:	pSprite,	The sprite which is colliding with this one.
//
//	PRE:	m_fx,					Sprite's updated X coordinate.
//			m_fy,					Sprite's updated Y coordinate.
//			m_fMoveOffsetX,	How much the sprite has just moved horizontally.
//			m_fMoveOffsetY,	How much the sprite has just moved vertically.
//
//	POST:	If collision with a player sprite...
//			m_iHitPoints,	The new amount of hit points.
//			m_fx,				Sprite's old X coordinate.
//			m_fy,				Sprite's old Y coordinate.
//			m_eState,		Sprite is flashing and recoiling.
//
//////////////////////////////////////////////////////////////////////////////
void cSprite::hit(cSprite* pSprite)
{
	BEGIN_ERRORHANDLER_GAME

	// Sanity check that the state isn't inactive (should be impossible.)
	if (m_eState == INACTIVE)
		THROW_GAME("How the heck was hit called on an inactive sprite?");
	
	if (m_eState & FLASHING)		// Can't be damaged if still flashing.
		return;

	// Subtract life based on amount of damage caused to it.
 	m_iHitPoints -= pSprite->getDamage();

	// Check if dead.
	if (m_iHitPoints > 0)
	{
		// Still alive, so do some flashing...

		// ???TMH - Sprites should not recoil unless they are on line with
		//				the 8x8 pixel grid.  When they changed to move along
		//				the grid, fix this logic.

		// Update count down timers...
 		m_lFlashTime	= lHITFLASH_TIME;		// Start hit.
		m_lRecoilTime	= lRECOIL_TIME;		// Start recoiling.
		
		// Update state.
		m_eState = static_cast<eSPRITE_STATE>(ACTIVE | RECOILING | FLASHING);
		
		// Change sprite location, back to where it was.
		moveBack();
	}
	else
	{
		// Sprite is dead...
		
		// Create a sparkle object with some basic bounty.
		cObjectSparkle* pZeldaSparkle = 
			new cObjectSparkle(m_pGameState, m_fx, m_fy, GOODIES_BASIC);

		// Inject sparkle object into the active sprite list.
		m_pGameState->addActiveSprite(pZeldaSparkle);
	}


	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	Set the state of the Sprite.
//
//	ARGS:	eState,	New state to put sprite in.
//
//	POST:	m_eState,		New state.
//			m_yColorSet,	Proper color (in case of being called inactive
//								while flashing).
//
//////////////////////////////////////////////////////////////////////////////
void cSprite::setState(eSPRITE_STATE eState)
{ 
	m_eState		= eState;							// Set state.
	m_yColorSet = m_yColorSetBeforeFlashing;	// Insure proper color.
}


//////////////////////////////////////////////////////////////////////////////
//
// -	If a sprite's boundries extend beyond the playfield.
//
//	PRE:	m_pGameState,	Set to game state.
//
//	RETURNS:	true,		If any portion of sprite exceeds playfield boundries.
//				false,	The sprite is within the boundries of the playfield.
//
//////////////////////////////////////////////////////////////////////////////
bool cSprite::isOutOfPlayfield()
{
	BEGIN_ERRORHANDLER_GAME

	// Perform quick check if sprite is out of its playfield boundry.

	// SPRITE					PLAYFIELD
	// -------------------  ---------------------------------------
	return (
		(m_fx						< m_pGameState->getSpriteBoundryMinX()) ||
		((m_fx + m_fWidth)	> m_pGameState->getSpriteBoundryMaxX()) ||
		(m_fy						< m_pGameState->getSpriteBoundryMinY()) ||
		((m_fy + m_fHeight)	> m_pGameState->getSpriteBoundryMaxY())	);

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Handles a sprite in a flashing state.
//	-	Updates the color of the sprite and the time remaining for flashing.
//
//	PRE:	m_yColorBeforeFlashing,	The color set of the sprite before it
//											began flashing.
//
//	POST:	m_fFlashTime,	Time remaining before flashing stops.
//
//////////////////////////////////////////////////////////////////////////////
void cSprite::flash()
{
	BEGIN_ERRORHANDLER_GAME		

	// Adjust how much time is left for flashing.
	m_lFlashTime -= m_pGameState->getTimeLapseMS();	

	// Check if flashing should stop.
	if (m_lFlashTime <= 0)
	{
		// Stop flashing...
		m_eState = static_cast<eSPRITE_STATE>( m_eState ^ FLASHING );

		m_lFlashTime = 0;									// Stop timer.
		m_yColorSet = m_yColorSetBeforeFlashing;	// Reset color set.

	} else {

		// Still flashing...

		// Adjust when the color changes.
		m_lFlashChangeTime -= m_pGameState->getTimeLapseMS();

		// If the flash time is exceeded...
		if (m_lFlashChangeTime < 0)
		{
			// Reset the the flash timer and jump to the next color.
			m_lFlashChangeTime = lFLASH_CHANGE_TIME;	
			m_yColorSet++;

			// If color is out of range reset it back to the first color.
			if (m_yColorSet > m_pTiles->getLastColorSet())
				m_yColorSet = m_pTiles->getFirstColorSet();
		}
	}

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//
//	-	Update sprite's location based on recoiling.
//
//	POST:	m_fMoveOffsetX,	Amount sprite needs to move horizontally.
//			m_fMoveOffsetY,	Amount sprite needs to move vertically.
//			m_fRecoilTime,		How much time (in milliseconds) is left in the 
//									recoil animation, or 0.0f if done.
//
//////////////////////////////////////////////////////////////////////////////
void cSprite::recoil()
{
	BEGIN_ERRORHANDLER_GAME


	// Obtain the multipler to use for all movement updates.
	float fFrameFactor = m_pGameState->getFrameFactor();


	// Recoil in the opposite direction being faced.
	switch(m_eDirection)
	{
		case UP:
			m_fMoveOffsetY = -(fRECOIL_SPEED * fFrameFactor);
			break;

		case RIGHT:
			m_fMoveOffsetX = -(fRECOIL_SPEED * fFrameFactor);
			break;

		case DOWN:
			m_fMoveOffsetY = fRECOIL_SPEED * fFrameFactor;
			break;

		case LEFT:
			m_fMoveOffsetX = fRECOIL_SPEED * fFrameFactor;
			break;
		
		// Anything else is illegal which should cause an error.
		default:
		{
			// Construct an error message and throw.
			THROW_GAME_INT("Only up, down, left, and right are legal direction "
				"values for a sprite to recoil.", m_eDirection);
			break;
		}
	}

	// Adjust how much time is left in recoil.
	m_lRecoilTime -= m_pGameState->getTimeLapseMS();

	// Check if sprite should stop recoiling.
	if (m_lRecoilTime < 0)
	{
		// Stop recoiling...
		m_eState = static_cast<eSPRITE_STATE>( m_eState ^ RECOILING );
		
		m_lRecoilTime = 0;	// Reset timer.
	}


	END_ERRORHANDLER_GAME
}



/////////////////////////////////////////////////////////////////////////////////////
//
//	-	Update Sprite's condition while in a state of being frozen.
//
//	POST:	m_lFrozenTimer,	Refelects new amount of time left until unfrozen.
//
/////////////////////////////////////////////////////////////////////////////////////
void cSprite::thaw()
{
	BEGIN_ERRORHANDLER_GAME

	// Adjust how much time is left in freeze.
	m_lFrozenTimer -= m_pGameState->getTimeLapseMS();

	// Check if sprite should stop being frozen.
	if ( m_lFrozenTimer < 0 )
	{
		// Stop being frozen.
		setState( static_cast<eSPRITE_STATE>(getState() ^ FROZEN) );
		
		m_lFrozenTimer = 0;	// Reset timer.
	}

	END_ERRORHANDLER_GAME
}

// The type of damage/hit this sprite is going to cause.
eHIT_TYPE cSprite::getHitType() {return HIT_TYPE_TOUCH;}

// Attach an event to the Sprite
void cSprite::setEvent( cEvent* pEvent ) { m_pEvent = pEvent; }

// Set position/direction manually in code.
void cSprite::setPosition( float fx, float fy ) { m_fx = fx; m_fy = fy; }
void cSprite::setDepth( eZ_PLANE eZ ) { m_eZ = eZ; }
void cSprite::setDirection( eDIRECTION	eDirection ) { m_eDirection = eDirection; }

// ACCESORS
// What type of sprite this is.
/*inline*/ eSPRITE_COLLIDE cSprite::getCollisionMask(){ return m_eCollisionMask; }

// How many hit points this sprite has left (before it dies).
/*inline*/ int cSprite::getHitPoints(){ return m_iHitPoints; }

// Whether or not the sprite's memory can be deleted when it dies.
/*inline*/ bool cSprite::isDeleteable(){ return m_bDeleteable;	}

// Whether or not the sprite is dead (and should no longer be in game).
/*inline*/ bool cSprite::isDead(){ return (m_iHitPoints <= 0);	}

// How much damage collision with this sprite will cause.
/*inline*/ int cSprite::getDamage(){ return m_iDamage; }

// What state the sprite is in.
/*inline*/ eSPRITE_STATE cSprite::getState(){ return m_eState; }	

// Get the event linked to the sprite.
/*inline*/ cEvent* cSprite::getEvent(){ return m_pEvent; }	

// Get position of sprite on the playfield.
 /*inline*/ float cSprite::getX()			{ return m_fx; }
 /*inline*/ float cSprite::getY()			{ return m_fy; }

// Get the dimensions of the sprite.
 /*inline*/ float cSprite::getWidth()		{ return m_fWidth;  }
 /*inline*/ float cSprite::getHeight()		{ return m_fHeight; }

// Get the "collision dimensions" of the sprite.
 /*inline*/ float cSprite::getEpsilonX()	{ return m_fEpsilonX; }
 /*inline*/ float cSprite::getEpsilonY()	{ return m_fEpsilonY; }
// vi:set ts=3:
