//////////////////////////////////////////////////////////////////////////////
//
//	sprite.h
//	Defines for the cSprites class.
//
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////


#ifndef __SPRITE_H__
#define __SPRITE_H__

// Turn off 4786: "identifier was truncated to '255' characters..."
#ifdef WIN32
#pragma warning(disable:4786)
#endif

#include "inc/system.h"

#include "ColorRGB.h"           // Color and BYTE support.
#include "ColorRGBPalette.h"           // Color and BYTE support.
#include "inc/SpriteIDs.h" // IDs for sprites.
#include "inc/EventIDs.h"      // IDs for events.

// Forward defines (truly defined later).
class cSpriteData;
class cGameState;
class cEvent;
class cTiles;

// Types of hits between sprites.
enum eHIT_TYPE
{
	HIT_TYPE_TOUCH = 0,
	HIT_TYPE_SWORD,
	HIT_TYPE_BOOMERANG,
	HIT_TYPE_MAGIC,
	HIT_TYPE_FIRE,
	HIT_TYPE_ARROW,
	HIT_TYPE_BOMB,
};

// Forward declare the classes instead of #including them to prevent
// circular references when compiling the project.  The actual defines
// for these classes are #included in the .CPP.
class cGameState;

// Forward declare so typedefs doesn't choke.
class cSprite;

// Forward declare so event member doesn't cause whole class to choke
class cEvent;

// Define type to iterate through collection of sprites.
typedef vector<cSprite*>::iterator SPRITE_ITR;

// Time in milliseconds in which 2 step walking animation repeats.
const long lANIMATION_WALK_MAX = 300;

// For sprites that keep track of how long it's been since they last move,
// this value (in millseconds) should be large enough to force the sprite
// to pick a new movement.
const long TIME_FOR_A_NEW_DIRECTION = 999999;

// Because all sprites flash at the same rate, this is the constant they
// want to set the flashing change timers to.
const long lFLASH_CHANGE_TIME = 25;

// Collision among all sprites is the same.  This is how "picky" the collision
// can be.  As the # approaches 0.0f it gets more exact, but then it becomes
// a pain in the ass to maneuver Link between tight places.
const float fEPSILON_BORDER = 2.4f;

// Time in milliseconds a sprite should stay frozen when put into that state.
// Individual sprites may use their own value if it's not globally identical.
const long lFREEZE_COUNTDOWN = 2500;


// Bit mask for what state the sprite is in.
enum eSPRITE_STATE
{
	INACTIVE		= 0x0000,	// Sprite isn't in collision checks or drawing.
	ACTIVE		= 0x0001,	// Sprite is interacting with other game elements.
	RECOILING	= 0x0002,	// Got hit and is being thrown back.
	FLASHING		= 0x0004,	// Just got damaged and is flashing.
	FROZEN		= 0x0008		// Can't move (or accept input).	
};

class cSprite  
{
	///////////////////////////////////////////////////////////////////////////
	//	MEMBERS
	///////////////////////////////////////////////////////////////////////////
protected:
	// Low level game related...
	cGameState*			m_pGameState;	// Keeps track of the state of the game.
	cTiles*				m_pTiles;		// Graphics used to draw this sprite.
	bool					m_bDeleteable;	// Can sprite be auto-deleted when dead?
												// Commonly used for projectiles, and 
												//	other sprites owned by a screen.

	BYTE					m_yColorSet;		// Color set of sprite.
	BYTE					m_yColorSetBeforeFlashing;	// ...
	eSPRITE_COLLIDE	m_eCollisionMask;	// Collision bit mask
	eSPAWN_TYPE			m_eSpawnType;		// How it initially appears.
	eZ_PLANE				m_eZ;					// Z plane (may be negative)

	// Game play attributes...
	eDIRECTION			m_eDirection;		// Direction sprite is traveling.
	float					m_fSpeed;			// Speed of sprite, (if moves at all).
	int					m_iHitPoints;		// How much "life" is left.
	int					m_iDamage;			// How much damage this inflicts.
	eSPRITE_STATE		m_eState;			// State of the sprite.

	// Timer related...
	long		m_lAnimationTime;	// Count down until geneal animation change.
	long		m_lRecoilTime;		// Count down until recoil stops.
	long		m_lFlashTime;			// Count down until flashing stops.
	long		m_lFlashChangeTime;	// Count down until next flash color change.
	long		m_lFrozenTimer;

	// Offsets...
	int		m_iAnimateOffset;		// Which sprite image to draw.	

	// Position/Collision related...
	float	m_fx;					// Sprite's X pixel coordinate.
	float	m_fy;					// Sprite's Y pixel coordinate.
	float	m_fWidth;			//	Width of sprite in pixels.
	float	m_fHeight;			//	Height of sprite in pixels.
	float	m_fEpsilonX;		// X collision range.
	float	m_fEpsilonY;		// Y collision range.
	float m_fMoveOffsetX;	// Used to determine amount of left/right movement.
	float m_fMoveOffsetY;	// Used to determine amount of up/down movement.

	// Event-related
	cEvent*				m_pEvent;		// Event linked to this sprite


	///////////////////////////////////////////////////////////////////////////
	//	METHODS
	///////////////////////////////////////////////////////////////////////////
protected:
	cSprite(){}	// Default constructor non-creatable.

	virtual bool isOutOfPlayfield();	// Check that sprite is in boundries.
	virtual void flash();				// Make sprite flash.
	virtual void recoil();				// Make sprite recoil (from a hit).
	virtual void thaw();					// Return from frozen status.


public:
	// Constructor
	cSprite(cGameState* pGameState, cSpriteData& oSpriteData);

	// Destructor
	virtual ~cSprite();

	// Called when a sprite is (re)loaded.
	virtual void reset();

	// Checks if another sprite's boundries falls within this sprite's area.
	virtual bool collide(cSprite* pSprite);

	// Moves the sprite back to it's previous location on a collision.
	virtual void moveBack();

	// This sprite is being hit by another sprite.
	virtual void hit(cSprite* pSprite);
	
	//	Set the state of the Sprite.
	virtual void setState(eSPRITE_STATE eState);

	// The type of damage/hit this sprite is going to cause.
	virtual eHIT_TYPE getHitType();

	// Attach an event to the Sprite
	void setEvent( cEvent* pEvent );

	// Set position/direction manually in code.
	void setPosition( float fx, float fy );
	void setDepth( eZ_PLANE eZ );
	void setDirection( eDIRECTION	eDirection );


	// --- PURE VIRTUALS ---

	// Update the movement & apply collision detection.
	virtual void update() = 0;

	// Draw the sprite.
	virtual void draw() = 0;


	///////////////////////////////////////////////////////////////////////////
	//	ACCESSORS
	///////////////////////////////////////////////////////////////////////////

	// What type of sprite this is.
	/*inline*/ eSPRITE_COLLIDE getCollisionMask();

	// How many hit points this sprite has left (before it dies).
	/*inline*/ int getHitPoints();

	// Whether or not the sprite's memory can be deleted when it dies.
	/*inline*/ bool isDeleteable();

	// Whether or not the sprite is dead (and should no longer be in game).
	/*inline*/ bool isDead();

	// How much damage collision with this sprite will cause.
	/*inline*/ int getDamage();

	// What state the sprite is in.
	/*inline*/ eSPRITE_STATE getState();

	// Get the event linked to the sprite.
	/*inline*/ cEvent* getEvent();

	// Get position of sprite on the playfield.
	virtual /*inline*/ float getX();
	virtual /*inline*/ float getY();

	// Get the dimensions of the sprite.
	virtual /*inline*/ float getWidth();		
	virtual /*inline*/ float getHeight();

	// Get the "collision dimensions" of the sprite.
	virtual /*inline*/ float getEpsilonX();
	virtual /*inline*/ float getEpsilonY();
};


#endif  // __SPRITE_H__
// vi:set ts=3:
