//////////////////////////////////////////////////////////////////////////////
//
//	tiles.cpp
//	Implementation for the cTiles class.
//
//	-	Holds the graphic "tiles" used for scenery in the maps.
//	-	Tabs set to 3 spaces.
// 
//////////////////////////////////////////////////////////////////////////////

#include "tiles.h"

//////////////////////////////////////////////////////////////////////////////
//
// -	CONSTRUCTOR
//	-	Loads graphics for sprites.
//
//	ARGS:	szTileSetname,	The name of the set of tiles to load.
//			iWidth,			A tile's width in pixels.
//			iHeight,			A tile's height in pixels.
//
//	POST:	m_fWidth,			Set to tile width.
//			m_fHeight,			Set to tile height.
//			m_iTilesInImageX,	Set to number of tiles in image across.
//			m_iTilesInImageY,	Set to number of tiles in image vertically.
//
//////////////////////////////////////////////////////////////////////////////
cTiles::cTiles(char szTileSetName[MAX_TILE_NAME], int iWidth, int iHeight)
{
	BEGIN_ERRORHANDLER_GAME

	// Build path to tile data.
	string sTileSetName("/rd/");
	sTileSetName += szTileSetName;
	sTileSetName += ".pcx";		// modified  for dreamcast

	// By default there is only one color set.
	m_yFirstColorSet = m_yLastColorSet = 0;

	printf(">>loading %s\n", sTileSetName.c_str());

	// Load graphic which contains a grid of tile images set.
	if (!m_oTileImages.loadPCX(sTileSetName.c_str()))
	{
		// Throw error object, containing error messsage.
		//THROW_GAME( sErrorMessage.c_str() );
		printf(">> Failure loading tile images in file '%s'\n", sTileSetName.c_str());
	}

	// Build path to palette file.
	sTileSetName = "/rd/";
	sTileSetName += szTileSetName;
	sTileSetName += ".pal";
	
	printf(">>loading palette %s\n", sTileSetName.c_str() );

	// Attempt to load palette;
	m_yLastColorSet = m_oTileImages.loadPalette(sTileSetName.c_str());
	
	if (m_yLastColorSet != 0)	// If a palette file was successfully read...
		m_yFirstColorSet = 1;	// ...ignore the first (0) bitmap.

	// Set dimensions of a tile.
	m_fWidth	= float(iWidth);
	m_fHeight= float(iHeight);

	// Determine the # of tiles in the image.
	m_iTilesInImageX = m_oTileImages.getWidth() / iWidth;
	m_iTilesInImageY = m_oTileImages.getHeight() / iHeight;

	END_ERRORHANDLER_GAME
}


//////////////////////////////////////////////////////////////////////////////
//	DESTRUCTOR
//////////////////////////////////////////////////////////////////////////////
cTiles::~cTiles()
{
}

// Accessors
/*inline*/ float cTiles::getWidth() {
	return m_fWidth;	
}

/*inline*/ float cTiles::getHeight() {
	return m_fHeight; 
}	

// Return first and last values for colors sets.
/*inline*/ BYTE cTiles::getFirstColorSet() {
	return m_yFirstColorSet; 
}

/*inline*/ BYTE cTiles::getLastColorSet() {
	return m_yLastColorSet; 
}

//////////////////////////////////////////////////////////////////////////////
//
//	-	Renders a tile.
//	-	The tiles object holds a single OpenGL texture which is all tiles.  This
//		method draws a square and textures it with just a small portion of the
//		OpenGL texture in memory.
//
//	ARGS:	x,				Horizontal pixel position on screen.
//			y,				Vertical pixel position on screen.
//			z,				Height to draw tile.
//			yTileID,		ID of tile to render.
//			yColorSet,	[0] The colorset of the tile to render.
//
//////////////////////////////////////////////////////////////////////////////
void cTiles::draw(float x, float y, float z, BYTE yTileID, BYTE yColorSet)
{
	BEGIN_ERRORHANDLER_GAME

	// Set the coordinates of the upper-left corner of what tile to draw.
	float cx = float(yTileID % m_iTilesInImageX) / m_iTilesInImageX;
	float cy = float(yTileID / m_iTilesInImageX) / m_iTilesInImageY;

	// A value which we apply while grabbing a portion of the OpenGL texture,
	// so that the area is ever-so slightly less than what we really
	// want.  This prevents blending from neighboring pixels.
	float fBorder = 0.0001f;

	// Obtain the ID for the texture which contains all the tile images.
	//int g = yColorSet - 1; if (g < 1) g = 0;

	//printf(">> getID(yColorSet=%u) = %u\n",yColorSet, m_oTileImages.getID(yColorSet));

	// XXX HACK TIME!
	glBindTexture(GL_TEXTURE_2D, m_oTileImages.getID(0));
	//glBindTexture(GL_TEXTURE_2D, m_oTileImages.getID(yColorSet));

	//static int farx = 0;
	
	//printf(" >> requested %u, max=%u\n", yColorSet, m_oTileImages.m_vID.size());

	// Draw a square (tile) using a small portion of the tile images.
	glBegin(GL_QUADS);
		
		glTexCoord2f(														// Top Left
			cx + fBorder,	
			1.0f - (cy + 1.0f/m_iTilesInImageY) + fBorder);
		glVertex3f(x, y, z);
		
		glTexCoord2f(														// Top Right
			cx + 1.0f/m_iTilesInImageX - fBorder,
			1.0f - (cy + 1.0f/m_iTilesInImageY) + fBorder);
		glVertex3f(x + m_fWidth, y, z);
		
		glTexCoord2f(														// Bottom Right
			cx + 1.0f/m_iTilesInImageX - fBorder,
			1.0f - cy - fBorder);
		glVertex3f(x + m_fWidth, y + m_fHeight, z);
		
		glTexCoord2f(														// Bottom Left
			cx + fBorder,
			1.0f - cy - fBorder);
		glVertex3f(x, y + m_fHeight, z);

	glEnd();

	END_ERRORHANDLER_GAME
}

//////////////////////////////////////////////////////////////////////////////
//
//	-	Renders a text string.
//	-	The tiles object holds a single OpenGL texture which is all tiles.  This
//		method draws a square and textures it with just a small portion of the
//		OpenGL texture in memory.
//	-	The following are special characters in string:
//
//			COLOR CHANGE CHARACTER:
//					#				This character will be followed by a color set 
//									byte to change the color set.
//
//			ZELDA SPECIAL CHARACTERS:
//					$				Rupee character
//					@				Bomb character
//					%				Key character
//
//
//	ARGS:	szMessage	Text string, with possible escape characters for color
//							changes.
//			x,				Horizontal pixel position on screen.
//			y,				Vertical pixel position on screen.
//
//////////////////////////////////////////////////////////////////////////////
void cTiles::drawString(const string szMessage, float fx, float fy)
{
	float fPositionX;	// Current horizontal position of text.
	float fPositionY; // Current vertical position of text.
	int	iColorSet = 0;


	// Set the starting screen position of the message.
	fPositionX = fx;
	fPositionY = fy;
	
	// Loop through the string, character by character.
	for (	unsigned int iCurrentCharacter = 0; iCurrentCharacter < szMessage.length(); 
			iCurrentCharacter++ )
	{
		// Draw different tiles depending on the character
		switch( szMessage.at( iCurrentCharacter ) )
		{
			// Color switch character...
			case '#':
			{
				iCurrentCharacter++;
				iColorSet = atoi(&szMessage.at( iCurrentCharacter ));
				break;
			}

			// Draw a letter or symbol...
			default:	
			{
				if ( charTileID[ szMessage.at( iCurrentCharacter ) ] != 0xFF )
				{
					// Perform actual draw.
					draw( fPositionX, fPositionY, Z_PLANE_CEILING, 
						charTileID[ szMessage.at( iCurrentCharacter ) ], iColorSet);

					// Move to next position.
					fPositionX += 8.0f;

					// Word wrapping...yay!
					// ???KWB - Is there a "screen limit" member somewhere?
					if ( fPositionX + 8.0f > 256.0f )
					{
						fPositionX = fx;
						fPositionY -= 8.0f;
					}
				}
				break;
			}

		} // END switch for character type

	} // END for each character in string
}
// vi:set ts=3:
